/*!
	\file assertfailederror.cxx
	\brief Implements the AssertFailedError class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-17
// Last time modified:	2008-04-28

#include "assertfailederror.h"

namespace HYPER_Compiler {
	
	using HYPER_Common::UTF8String;
	
	AssertFailedError::AssertFailedError(char const* assertion,
		char const* file, char const* function,
		unsigned line)
	 : fAssert(UTF8String::fromCString(assertion)),
		fFile(UTF8String::fromCString(file)),
		fFunction(UTF8String::fromCString(function)),
		fLine(line)
	{
		// no action
	}
	
	AssertFailedError::~AssertFailedError() {
		// no action
	}
	
	HYPER_Common::UTF8String AssertFailedError::getAssertion() const {
		return fAssert;
	}
	
	HYPER_Common::UTF8String AssertFailedError::getFile() const {
		return fFile;
	}
	
	HYPER_Common::UTF8String AssertFailedError::getFunction() const {
		return fFunction;
	}
	
	unsigned AssertFailedError::getLine() const {
		return fLine;
	}
	
	HYPER_Common::UTF8String AssertFailedError::getDescription() const {
		HYPER_Common::UTF8StringBuilder sb;
		sb += UTF8STR("Assertion failed :\n");
		sb += UTF8STR("  assertion: ");
		sb += fAssert;
		sb.newLine();
		sb += UTF8STR("  file : ");
		sb += fFile;
		sb.newLine();
		sb += UTF8STR("  function : ");
		sb += fFunction;
		sb.newLine();
		sb += UTF8STR("  line : ");
		sb += fLine;
		
		return sb.toString();
	}
}
