/*!
	\file assertfailederror.h
	\brief Contains the AssertFailedError class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-17
// Last time modified:	2008-04-08

#ifndef DEF_ASSERTFAILEDERROR_H
 #define DEF_ASSERTFAILEDERROR_H
 
 #include "internalcompilererror.h"

namespace HYPER_Compiler {
	
	/// Exception thrown when an assertion fails
	class AssertFailedError : public InternalCompilerError {
	public:
		AssertFailedError(char const* assertion, char const* file,
			char const* function, unsigned line);
		~AssertFailedError();
		
		HYPER_Common::UTF8String getAssertion() const;
		HYPER_Common::UTF8String getFile() const;
		HYPER_Common::UTF8String getFunction() const;
		unsigned getLine() const;
		
		HYPER_Common::UTF8String getDescription() const;
		
	private:
		HYPER_Common::UTF8String fAssert;
		HYPER_Common::UTF8String fFile;
		HYPER_Common::UTF8String fFunction;
		unsigned fLine;
	};
}
#endif
