/*!
	\file asserts.h
	\brief Contains the various asserts
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-17
// Last time modified:	2007-05-09

#ifndef DEF_ASSERTS_H
 #define DEF_ASSERTS_H
 
 #include "assertfailederror.h"
 #include "unreachablereachederror.h"
 
 #ifdef __GNUC__
 
  // assert condition
  #define hyp_assert(EXPR) \
		if (!(EXPR)) { \
			throw AssertFailedError(#EXPR, \
				__FILE__, __PRETTY_FUNCTION__, __LINE__); \
		}
 
  // assert unreachable code
  #define hyp_assert_unreachable() \
		throw UnreachableReachedError( \
			__FILE__, __PRETTY_FUNCTION__, __LINE__);
 
 #else

  // assert condition
  #define hyp_assert(EXPR) \
		if (!(EXPR)) { \
			throw AssertFailedError(#EXPR, \
				__FILE__, __FUNCTION__, __LINE__); \
		}
 
  // assert unreachable code
  #define hyp_assert_unreachable() \
		throw UnreachableReachedError( \
			__FILE__, __FUNCTION__, __LINE__);

 #endif

#endif
