/*!
	\file basiccompileinfo.cxx
	\brief Implements the BasicCompileInfo class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-06-21
// Last time modified:	2008-06-29

#include "basiccompileinfo.h"

#include "asserts.h"
#include "compileoptions.h"
#include "easydiagnostics.h"
#include "errormanager.h"
//#include "hostplatform.h"

namespace HYPER_Compiler {
	
	BasicCompileInfo::BasicCompileInfo()
	 : fEM(0), fDiag(0), fOpts(0)
	{
		//
	}
	
	BasicCompileInfo::~BasicCompileInfo() {
		//
	}
	
	ErrorManager& BasicCompileInfo::getErrorManager() const {
		hyp_assert(fEM != 0);
		return *fEM;
	}
	
	EasyDiagnostics const& BasicCompileInfo::diag() const {
		return *fDiag;
	}
	
	bool BasicCompileInfo::errorsPresent() const {
		hyp_assert(fEM != 0);
		return fEM->errorsPresent();
	}
	
	void BasicCompileInfo::setOptions(CompileOptions const& o) {
		fOpts = new CompileOptions(o);
		fEM =
			new ErrorManager(
				o.fgOutputStream, o.fgBufferErrors, fFileMap
			);
		fDiag = new EasyDiagnostics(*fEM);
	}
	
	CompileOptions const& BasicCompileInfo::getOptions() const {
		hyp_assert(fOpts != 0);
		return *fOpts;
	}
	
	void
	BasicCompileInfo::registerFile(FileID ID,
		HYPER_Common::FilePath const& path)
	{
		// prevent overwriting existing entries
		hyp_assert(fFileMap.find(ID) == fFileMap.end());
		
		fFileMap[ID] = &path;
	}
	
	HYPER_Common::FilePath const*
	BasicCompileInfo::getFilename(FileID ID) const
	{
		FileMap::const_iterator it = fFileMap.find(ID);
		
		if (it == fFileMap.end()) {
			return 0; // not found
		}
		else {
			return it->second;
		}
	}
	
	BasicCompileInfo::FileMap const&
	BasicCompileInfo::getFileMap() const
	{
		return fFileMap;
	}
	
	//Platform const& BasicCompileInfo::getHostPlatform() const {
	//	return gHostPlatform;
	//}
	
	Platform const& BasicCompileInfo::getTargetPlatform() const {
		return *fOpts->fgTargetPlatform;
	}
}
