/*!
	\file basiccompileinfo.h
	\brief Contains the BasicCompileInfo class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-06-21
// Last time modified:	2008-06-29

#ifndef DEF_BASICCOMPILEINFO_H
 #define DEF_BASICCOMPILEINFO_H
 
 #include "common/paths_fwdecl.h"
 
 #include "location.h"
 
 #include <map>

namespace HYPER_Compiler {
	
	class CompileOptions;
	class EasyDiagnostics;
	class ErrorManager;
	class Platform;
	
	/// Provides basic info used everywhere
	class BasicCompileInfo {
	public:
		BasicCompileInfo();
		~BasicCompileInfo();
		
		void setOptions(CompileOptions const& o);
		CompileOptions const& getOptions() const;
		
		ErrorManager& getErrorManager() const;
		EasyDiagnostics const& diag() const;
		bool errorsPresent() const;
		
		void registerFile(FileID ID, HYPER_Common::FilePath const& path);
		HYPER_Common::FilePath const* getFilename(FileID ID) const;
		typedef std::map<FileID, HYPER_Common::FilePath const*> FileMap;
		FileMap const& getFileMap() const;
		
		//Platform const& getHostPlatform() const;
		Platform const& getTargetPlatform() const;
		
	private:
		ErrorManager* fEM;
		EasyDiagnostics* fDiag;
		CompileOptions* fOpts;
		FileMap fFileMap;
	};
}
#endif
