/**
	\file compileoptions.cxx
	\brief Implements the CompileOptions class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-06-19
// Last time modified:	2008-11-16

#include "compileoptions.h"

#include "hostplatform.h"

#include "common/utf8textflowstream.h"
#include "common/utf8string.h"

using HYPER_Common::UTF8String;

// FIXME: don't just use host platform as target platform

namespace HYPER_Compiler {
	
	CompileOptions::CompileOptions(
		HYPER_Common::UTF8TextFlowStream& out)
	 : fgNoOutput(false), fgDontLink(false), fgSyntaxOnly(false),
		fgOptimize(false), fgPrint_AST(false),
		fgDebug(false),
		fgBufferErrors(true),
		fgToDoWarnings(true), fgFixMeWarnings(true),
		fgWhitespaceWarnings(true), fgHideParameterWarnings(true),
		fgOutputStream(out), fgTargetPlatform(&gHostPlatform)
	{
		// nothing here
	}
	
	#define printOption(OPTION) fgOutputStream.print(UTF8STR(#OPTION ": ")); fgOutputStream.print(UTF8STR(OPTION ? "true\n" : "false\n"));
	void CompileOptions::printOptions()
	{
		printOption(fgNoOutput);
		printOption(fgOptimize);
		printOption(fgDontLink);
		printOption(fgDebug);
		printOption(fgBufferErrors);
		printOption(fgToDoWarnings);
		printOption(fgFixMeWarnings);
		printOption(fgWhitespaceWarnings);
		printOption(fgHideParameterWarnings);
	}
}
