/*!
	\file debug.cxx
	\brief Implements facilities for easy debug output
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-10-01
// Last time modified:	2008-02-15

#include "debug.h"

#include "common/utf8stdoutstream.h"
#include "common/switchablepushstream.h"

#include <iostream>

namespace HYPER_Compiler {
	
	using HYPER_Common::SwitchablePushStream;
	
	HYPER_Common::UTF8StdOutStream debugStdOut(std::cout);
	SwitchablePushStream<HYPER_Common::UTF8Byte> debugSwitchStream(debugStdOut);
	HYPER_Common::UTF8TextFlowStream debug(debugSwitchStream);
	
	IndentDebug::IndentDebug()
	 : HYPER_Common::Indent(debug)
	{
		// no action
	}
	
	
	void Debug::enableDebugOutput(bool value) {
		debugSwitchStream.enable(value);
	}
	
	void Debug::fixMe(HYPER_Common::UTF8String str) {
		debug.line(UTF8STR("FIXME: ") + str);
	}
	
	void Debug::toDo(HYPER_Common::UTF8String str) {
		debug.line(UTF8STR("TODO: ") + str);
	}
}
