/*!
	\file diagnostic.cxx
	\brief Implements the Diagnostic class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-02-27
// Last time modified:	2008-02-14

#include "diagnostic.h"

#include "asserts.h"
#include "location.h"

namespace HYPER_Compiler {
	
	// *** Diagnostic ***
	
	Diagnostic::Diagnostic(Type t, DiagnosticPair dp)
	 : fType(t), fData(dp)
	{
		//
	}
	
	Diagnostic::Type Diagnostic::getType() const {
		return fType;
	}
	
	HYPER_Common::UTF8String Diagnostic::getTypeString() const {
		switch (fType) {
		case kNote:
			return UTF8STR("note");
		case kWarning:
			return UTF8STR("warning");
		case kError:
			return UTF8STR("error");
		default:
			hyp_assert_unreachable();
		};
	}
	
	DiagnosticPair const& Diagnostic::getData() const {
		return fData;
	}
	
	const Location& Diagnostic::getLocation() const {
		return *fData.location;
	}
	
	void Diagnostic::addInfo(DiagnosticPair dp) {
		fInfos.push_back(dp);
	}
	
	Diagnostic::InfoIterator Diagnostic::getInfoBegin() const {
		return fInfos.begin();
	}
	
	Diagnostic::InfoIterator Diagnostic::getInfoEnd() const {
		return fInfos.end();
	}
}
