/*!
	\file diagnostic.h
	\brief Contains the Diagnostic class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-02-26
// Last time modified:	2008-02-14

#ifndef DEF_DIAGNOSTIC_H
 #define DEF_DIAGNOSTIC_H
 
 #include "common/streams_fwdecl.h"
 #include "common/utf8string.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class Location;
	
	/// Combines a text message and a diagnostics location
	struct DiagnosticPair {
		HYPER_Common::UTF8String message;
		const Location* location;
		
		DiagnosticPair(HYPER_Common::UTF8String msg, const Location* loc)
		 : message(msg), location(loc) {}
	};
	
	/// Represents a diagnostic message
	class Diagnostic {
	public:
		enum Type {
			kNote,
			kWarning,
			kError
		};
		
		/// Constructor
		Diagnostic(Type t, DiagnosticPair dp);
		
		/// Get diagnostic type
		Type getType() const;
		
		/// Get diagnostic type as a string
		HYPER_Common::UTF8String getTypeString() const;
		
		/// Get the diagnostic data
		DiagnosticPair const& getData() const;
		
		/// Get the diagnostic location (shortcut)
		Location const& getLocation() const;
		
		/// Add an extra message with corresponding location
		void addInfo(DiagnosticPair dp);
		
		typedef std::vector<DiagnosticPair>::const_iterator InfoIterator;
		InfoIterator getInfoBegin() const;
		InfoIterator getInfoEnd() const;
		
	private:
		Type fType;
		DiagnosticPair fData;
		std::vector<DiagnosticPair> fInfos;
	};
} 
#endif
