/*!
	\file easydiagnostics.cxx
	\brief Contains the EasyDiagnostics class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-08-09
// Last time modified:	2008-03-01

#include "easydiagnostics.h"

#include "errormanager.h"

namespace HYPER_Compiler {
	
	// *** EasyDiagnosticRef::DiagnosticCommitter ***
	
	EasyDiagnosticRef::DiagnosticCommitter::DiagnosticCommitter(
		Diagnostic const& d, ErrorManager& errMan)
	 : fDiag(d), fErrMan(errMan)
	{
		//
	}
	
	EasyDiagnosticRef::DiagnosticCommitter::~DiagnosticCommitter() {
		fErrMan.add(fDiag); // diagnostic added to list
	}
	
	
	// *** EasyDiagnosticRef ***
	
	EasyDiagnosticRef EasyDiagnosticRef::createRef(Diagnostic const& d,
		ErrorManager& errMan)
	{
		return EasyDiagnosticRef(new DiagnosticCommitter(d, errMan));
	}
	
	EasyDiagnosticRef::EasyDiagnosticRef(DiagnosticCommitter* dc)
	 : fDC(dc)
	{
		//
	}
	
	EasyDiagnosticRef const&
	EasyDiagnosticRef::addInfo(HYPER_Common::UTF8String msg,
		Location const* loc) const
	{
		fDC->fDiag.addInfo(DiagnosticPair(msg, loc));
		return *this;
	}
	
	
	// *** EasyDiagnostics ***
	
	EasyDiagnostics::EasyDiagnostics(ErrorManager& errMan)
	 : fErrMan(errMan)
	{
		//
	}
	
	EasyDiagnosticRef EasyDiagnostics::add(Diagnostic const& d) const {
		return EasyDiagnosticRef::createRef(d, fErrMan);
	}
	
	EasyDiagnosticRef
	EasyDiagnostics::error(HYPER_Common::UTF8String msg,
		Location const* loc) const
	{
		return add(
			Diagnostic(Diagnostic::kError, DiagnosticPair(msg, loc))
		);
	}
	
	EasyDiagnosticRef
	EasyDiagnostics::warning(HYPER_Common::UTF8String msg,
		Location const* loc) const
	{
		return add(
			Diagnostic(Diagnostic::kWarning, DiagnosticPair(msg, loc))
		);
	}
	
	EasyDiagnosticRef
	EasyDiagnostics::note(HYPER_Common::UTF8String msg,
		Location const* loc) const
	{
		return add(
			Diagnostic(Diagnostic::kNote, DiagnosticPair(msg, loc))
		);
	}
}
