/*!
	\file errormanager.h
	\brief Contains the ErrorManager class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-05-11
// Last time modified:	2008-09-06

#ifndef DEF_ERRORMANAGER_H
 #define DEF_ERRORMANAGER_H
 
 #include "diagnostic.h"
 #include "location.h"
 
 #include "common/paths_fwdecl.h"
 #include "common/streams_fwdecl.h"
 
 #include <list>
 #include <map>

namespace HYPER_Compiler {
	
	/// Error keeping
	class ErrorManager {
		typedef std::map<FileID, HYPER_Common::FilePath const*> FileMap;
	public:
		ErrorManager(HYPER_Common::UTF8TextFlowStream& out,
			bool bufferErrors, FileMap const& filemap);
		
		void add(Diagnostic const& d);
		
		bool errorsPresent();
		
		unsigned int count() const;
		unsigned int count(Diagnostic::Type t) const;
		
		typedef std::list<Diagnostic>::const_iterator DiagnosticIterator;
		DiagnosticIterator begin() const;
		DiagnosticIterator end() const;
		
		HYPER_Common::FilePath const* getFileName(FileID ID) const;
		
		/// Print a given diagnostic to a given outputstream
		void print(HYPER_Common::UTF8TextFlowStream& out,
			Diagnostic const& d) const;
		
		/// Print a given Location to a given outputstream
		void print(HYPER_Common::UTF8PushStream& out,
			Location const& loc) const;
		
	private:
		ErrorManager(ErrorManager const&);
		
		void print(Diagnostic const& d) const;
		
		bool strictlyOrdered(Location const& loc1, Location const& loc2);
		
		std::list<Diagnostic> fList;
		std::list<Diagnostic>::iterator fLastAdd;
		unsigned fTotalCount;
		std::map<Diagnostic::Type, unsigned int> fCounts;
		bool fBufferErrors;
		HYPER_Common::UTF8TextFlowStream& fOut;
		FileMap const& fFileMap;
	};
}
#endif
