/*!
	\file hostplatform.cxx
	\brief Determines the host platform.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-12-07
// Last time modified:	2008-04-30
 
#include "hostplatform.h"

namespace HYPER_Compiler {
	
#if defined(__amd64__) || defined(_M_X64)
	// x86-64  (64-bit)
	#define HOST_PLATFORM_WORDSIZE k64Bit
	#define HOST_PLATFORM_ENDIANNESS kLittleEndian
#elif defined(__i386__) || defined(_M_IX86)
	// x86  (32-bit)
	#define HOST_PLATFORM_WORDSIZE k32Bit
	#define HOST_PLATFORM_ENDIANNESS kLittleEndian
#elif defined(__sparc__) && defined(__arch64__)
	// 64-bit SPARC -- untested
	#define HOST_PLATFORM_WORDSIZE k64Bit
	#define HOST_PLATFORM_ENDIANNESS kBigEndian
#elif defined(__sparc__)
	// 32-bit SPARC -- untested
	#define HOST_PLATFORM_WORDSIZE k32Bit
	#define HOST_PLATFORM_ENDIANNESS kBigEndian
#else
	#error "HOST platform not recognized!"
#endif
	
	// TODO: add more platform properties to detect
	
	Platform gHostPlatform(Platform::HOST_PLATFORM_WORDSIZE,
		Platform::HOST_PLATFORM_ENDIANNESS);
	
}
