/*!
	\file location.cxx
	\brief Implementation of the Location class and its derived classes.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-05-12
// Last time modified:	2008-06-29

#include "location.h"

#include "asserts.h"

namespace HYPER_Compiler {
	
	// *** Location ***
	
	Location const*
	Location::createSourceLoc(FileID fileID, unsigned line,
		unsigned column)
	{
		return new Location(kSource, fileID, line, column);
	}
	
	Location const* Location::getAutoGenerated() {
		static Location const* loc =
			new Location(kAutoGenerated, FileID(0), 0, 0);
		
		return loc;
	}
	
	Location const* Location::getBuiltIn() {
		static Location const* loc =
			new Location(kBuiltIn, FileID(0), 0, 0);
		
		return loc;
	}
	
	Location::Location(Type t, FileID file, unsigned line,
		unsigned column)
	 : fType(t), fFileID(file), fLine(line), fCol(column)
	{
		//
	}
	
	Location::Type Location::getType() const {
		return fType;
	}
	
	inline bool Location::isSpecial() const {
		hyp_assert(
			fType == kSource || fType == kBuiltIn || fType == kAutoGenerated
		);
		
		return fType != kSource;
	}
	
	FileID Location::getFileID() const {
		hyp_assert(!isSpecial());
		return fFileID;
	}
	
	unsigned Location::getLine() const {
		hyp_assert(!isSpecial());
		return fLine;
	}
	
	unsigned Location::getColumn() const {
		hyp_assert(!isSpecial());
		return fCol;
	}
	
	bool Location::operator==(Location const& loc) const {
		if (fType != loc.fType) {
			return false;
		}
		else if (fType != kSource) {
			return true;
		}
		
		return fFileID == loc.fFileID && fLine == loc.fLine
			&& fCol == loc.fCol;
	}
	
	Location::Location(Location const& loc)
	{
		hyp_assert_unreachable();
	}
	
	void Location::operator=(Location const& loc) {
		hyp_assert_unreachable();
	}
}
