/*!
	\file location.h
	\brief Contains the Location class and its derived classes.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-05-12
// Last time modified:	2008-06-29

#ifndef DEF_LOCATION_H
 #define DEF_LOCATION_H
 
 #include <boost/strong_typedef.hpp>
 
namespace HYPER_Compiler {
	
	BOOST_STRONG_TYPEDEF(unsigned, FileID)
	
	/// Represents a location for a compiler diagnostic.
	/// It can refer to sourcecode or built-in declarations.
	class Location {
	public:
		static Location const* createSourceLoc(FileID fileID, 
			unsigned line, unsigned column);
		
		static Location const* getAutoGenerated();
		static Location const* getBuiltIn();
		
		enum Type {kSource, kAutoGenerated, kBuiltIn};
		
		Type getType() const;
		bool isSpecial() const;
		
		FileID getFileID() const;
		unsigned getLine() const;
		unsigned getColumn() const;
		
		bool operator==(Location const& loc) const;
		bool operator!=(Location const& loc) const {
			return !operator==(loc);
		}
		
	private:
		Location(Type t, FileID file, unsigned line, unsigned col);
		
		// get rid of the assignment operator
		void operator=(Location const& loc);
		
		// get rid of the copy constructor
		Location(Location const& loc);
		
		Type fType;
		FileID fFileID;
		unsigned fLine;
		unsigned fCol;
	};
}
#endif
