/*!
	\file platform.cxx
	\brief Implements the Platform class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-06-23
// Last time modified:	2008-02-27

#include "platform.h"

#include "asserts.h"

namespace HYPER_Compiler {
	
	Platform::Platform(WordSize ws, Endianness e)
	 : fWS(ws), fEndian(e)
	{
		// no action
	}
	
	Platform::WordSize Platform::getWordSize() const {
		return fWS;
	}
	
	Platform::Endianness Platform::getEndianness() const {
		return fEndian;
	}
	
	unsigned Platform::toNumeric(WordSize w) {
		switch (w) {
		case k32Bit:
			return 32;
		case k64Bit:
			return 64;
		default:
			hyp_assert_unreachable();
		}
	}
	
	char const* Platform::toString(Endianness e) {
		switch (e) {
		case kLittleEndian:
			return "little";
		case kBigEndian:
			return "big";
		default:
			hyp_assert_unreachable();
		}
	}
}
