/*!
	\file platform.h
	\brief Contains the Platform class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-06-23
// Last time modified:	2009-02-08

#ifndef DEF_PLATFORM_H
 #define DEF_PLATFORM_H
 
namespace HYPER_Compiler {
	
	/// Represents a platform to compile for
	class Platform {
	public:
		enum WordSize {
			k32Bit = 1,
			k64Bit = 2
		};
		
		enum Endianness {
			kBigEndian,
			kLittleEndian
		};
		
		Platform(WordSize ws, Endianness e);
		
		WordSize getWordSize() const;
		Endianness getEndianness() const;
		
		/// Get the numeric value of a wordsize (number of bytes)
		static unsigned toNumeric(WordSize w);
		
		/// Get the string representation for an endianness ("big" or "little")
		static char const* toString(Endianness e);
		
	private:
		WordSize fWS;
		Endianness fEndian;
	};
}
#endif
