/*!
	\file unreachablereachederror.cxx
	\brief Implements the UnreachableReachedError class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-17
// Last time modified:	2008-04-28

#include "unreachablereachederror.h"

namespace HYPER_Compiler {
	
	using HYPER_Common::UTF8String;
	
	UnreachableReachedError::UnreachableReachedError(
		char const* file, char const* function, unsigned line)
	 : fFile(UTF8String::fromCString(file)),
		fFunction(UTF8String::fromCString(function)), fLine(line)
	{
		// no action
	}
	
	UnreachableReachedError::~UnreachableReachedError() {
		// no action
	}
	
	HYPER_Common::UTF8String UnreachableReachedError::getFile() const {
		return fFile;
	}
	
	HYPER_Common::UTF8String UnreachableReachedError::getFunction() const {
		return fFunction;
	}
	
	unsigned UnreachableReachedError::getLine() const {
		return fLine;
	}
	
	HYPER_Common::UTF8String UnreachableReachedError::getDescription() const {
		HYPER_Common::UTF8StringBuilder sb;
		sb += UTF8STR("Unreachable code reached :\n");
		sb += UTF8STR("  file : ");
		sb += fFile;
		sb.newLine();
		sb += UTF8STR("  function : ");
		sb += fFunction;
		sb.newLine();
		sb += UTF8STR("  line : ");
		sb += fLine;
		
		return sb.toString();
	}
}
