/*!
	\file bufferedpullstream.h
	\brief Contains the BufferedPullStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-20
// Last time modified: 	2008-02-20

#ifndef DEF_BUFFEREDPULLSTREAM_H
 #define DEF_BUFFEREDPULLSTREAM_H
 
 #include "pullstream.h"
 
 #include <vector>

namespace HYPER_Common {
	
	/// Abstract class that represents a stream you pull data from
	template<class T>
	class BufferedPullStream : public PullStream<T> {
	public:
		BufferedPullStream(PullStream<T>& stream);
		~BufferedPullStream() { /*empty*/ }
		
		bool streamOK() const;
		bool look(T& c);
		bool pullNext();
		typename PullStream<T>::PositionType streamPosition() const;
		
		bool goBackTo(typename PullStream<T>::PositionType pos);
		
	private:
		PullStream<T>& fStream;
		std::vector<T> fBuffer;
		typename PullStream<T>::PositionType fPos;
	};
}
#endif

// include implementation
#include "bufferedpullstream.cxx"
