/*!
	\file console.cxx
	\brief Implements the Console class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-04-24
// Last time modified:	2008-03-24

#include "console.h"

#include "utf8stdoutstream.h"
#include "utf8textflowstream.h"

#include <boost/shared_ptr.hpp>
#include <iostream>

namespace HYPER_Common {
	
	UTF8PushStream& Console::getStdOut() {
		static boost::shared_ptr<UTF8StdOutStream> s;
		if (s.get() == 0) { // not yet initialized...
			s.reset(new UTF8StdOutStream(std::cout));
		}
		return *s;
	}
	
	UTF8PushStream& Console::getStdErr() {
		static boost::shared_ptr<UTF8StdOutStream> s;
		if (s.get() == 0) { // not yet initialized...
			s.reset(new UTF8StdOutStream(std::cerr));
		}
		return *s;
	}
	
	UTF8TextFlowStream& Console::getTextFlowStdOut() {
		static boost::shared_ptr<UTF8TextFlowStream> s;
		if (s.get() == 0) { // not yet initialized...
			s.reset(create(getStdOut()));
		}
		return *s;
	}
	
	UTF8TextFlowStream& Console::getTextFlowStdErr() {
		static boost::shared_ptr<UTF8TextFlowStream> s;
		if (s.get() == 0) { // not yet initialized...
			s.reset(create(getStdErr()));
		}
		return *s;
	}
	
	UTF8TextFlowStream* Console::create(UTF8PushStream& s) {
		TextFlowOptions opt;
		opt.enableWrapping(75);
		return new UTF8TextFlowStream(s, opt);
	}
};
