/*!
	\file cxx_macros.h
	\brief Contains macro definitions
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-01-22
// Last time modified:	2009-03-08

#ifndef DEF_CXX_MACROS_H
 #define DEF_CXX_MACROS_H
 
 // --- Function attributes ---
 // ATTR_WARN_UNUSED_RESULT : warn if function return value ignored
 // ATTR_DEPRECATED : warn that the function is deprecated (obsolete) when used
 // ATTR_NO_RETURN : inform the compiler that a function never returns
 
 #ifdef __GNUC__
 
  // definitions for GCC
  #define ATTR_WARN_UNUSED_RESULT __attribute__ ((warn_unused_result))
  #define ATTR_DEPRECATED __attribute__ ((deprecated))
  #define ATTR_NO_RETURN __attribute__ ((noreturn))
  
 #else
 
  // other compilers we don't know (yet)
  #define ATTR_WARN_UNUSED_RESULT
  #define ATTR_DEPRECATED
  #define ATTR_NO_RETURN
  
 #endif
 
#endif
