/*!
	\file filepath.cxx
	\brief Implements the FilePath class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2005-01-31
// Last time modified: 	2008-03-06

#include "filepath.h"

#include <algorithm>

namespace HYPER_Common {
	
	// *** FilePath ***
	
	FilePath*
	FilePath::getLargestCommonPrefix(
		std::list<FilePath const*> const& paths)
	{
		if (paths.empty()) {
			return 0;
		}
		
		if (paths.size() == 1) {
			FilePath const& p = *paths.front();
			if (p.countLevels() > 1) {
				return p.getParentDirectory();
			}
			else {
				return 0;
			}
		}
		
		// we have at least two paths
		
		std::list<FilePath const*>::const_iterator it = paths.begin();
		std::list<FilePath const*>::const_iterator end = paths.end();
		FilePath const* prevPath = *it; // save first path
		++it;
		FilePath* prefix = prevPath->getLargestCommonPrefix(**it);
		++it;
		
		if (it == end) { // no more than two paths
			return prefix;
		}
		
		// three or more paths
		
		do {
			FilePath* newPrefix =
				prevPath->getLargestCommonPrefix(**it);
			
			if (newPrefix->countLevels() < prefix->countLevels()) {
				// the new prefix is shorter, so we adapt it instead
				prefix = newPrefix;
				prevPath = *it;
			}
			else {
				// we keep the old common prefix
			}
			++it;
		} while (it != end);
		
		return prefix;
	}
	
	bool FilePath::hasExtension(std::list<UTF8String> const& exts) const {
		UTF8String ext = this->getExtension();
		
		if (ext.isEmpty() || exts.empty()) {
			return false;
		}
		
		std::list<UTF8String>::const_iterator result =
			std::find(exts.begin(), exts.end(), ext);
		
		return (result != exts.end());
	}
}
