/*!
	\file filesystem.h
	\brief Contains the FileSystem class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-05-02
// Last time modified:	2008-03-24

#ifndef DEF_FILESYSTEM_H
 #define DEF_FILESYSTEM_H
 
 #include "streams_fwdecl.h"
 #include "utf8string.h"

namespace HYPER_Common {
	
	class FilePath;
	
	/// Contains utilities for filesystem access
	class FileSystem {
	public:
		/** Create a FilePath object for a native string path
			\param path Native pathname as a string
		*/
		static FilePath* pathFromString(UTF8String path);
		
		/** Get the current working directory of this program.
			\param cached Use saved result from last time if true
		*/
		static FilePath const* getWorkingDirectory(bool cached = true);
		
		/// Open a file for input. The UTF-8 stream is verified.
		static PullStream<UTF8Byte>* openInputFile(FilePath const& f);
		
		/// Open a file for output. Encoding is UTF-8.
		static UTF8PushStream* openOutputFile(FilePath const& f);
		
	private:
		FileSystem();
		FileSystem(FileSystem const&);
		
		static FilePath const* queryWorkingDirectory();
	};
}
#endif
