/*!
	\file overlaypullstream.cxx
	\brief Implements the OverlayPullStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-08
// Last time modified: 	2008-04-26

#ifndef DEF_OVERLAYPULLSTREAM_CXX
 #define DEF_OVERLAYPULLSTREAM_CXX
 
 #include "overlaypullstream.h"

namespace HYPER_Common {
	
	template <class T>
	OverlayPullStream<T>::OverlayPullStream(PullStream<T>& stream,
		unsigned long length, bool showRealPosition)
	 : fStream(stream), fPos(0), fEndPos(length), fShowRealPos(showRealPosition)
	{
		// no action
	}
	
	template <class T>
	bool OverlayPullStream<T>::streamOK() const {
		return fPos < fEndPos && fStream.streamOK();
	}
	
	template <class T>
	bool OverlayPullStream<T>::look(T& c) {
		if (fPos < fEndPos) { // still in overlay
			return fStream.look(c);
		}
		else {
			// end of overlay
			return false;
		}
	}
	
	template <class T>
	bool OverlayPullStream<T>::pullNext() {
		if (!fStream.pullNext()) {
			return false;
		}
		++fPos;
		return fPos < fEndPos;
	}
	
	template <class T>
	typename PullStream<T>::PositionType
	OverlayPullStream<T>::streamPosition() const
	{
		return fShowRealPos ? fStream.streamPosition() : fPos;
	}
	
	template <class T>
	typename PullStream<T>::PositionType
	OverlayPullStream<T>::realStreamPosition() const
	{
		return fStream.streamPosition();
	}
}
#endif
