/*!
	\file overlaypullstream.h
	\brief Contains the OverlayPullStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-08
// Last time modified: 	2008-03-08

#ifndef DEF_OVERLAYPULLSTREAM_H
 #define DEF_OVERLAYPULLSTREAM_H
 
 #include "pullstream.h"

namespace HYPER_Common {
	
	/// Stream that uses part of the content of another input stream
	template<class T>
	class OverlayPullStream : public PullStream<T> {
	public:
		OverlayPullStream(PullStream<T>& stream, unsigned long length,
			bool showRealPosition = false);
		~OverlayPullStream() { /*empty*/ }
		
		bool streamOK() const;
		bool look(T& c);
		bool pullNext();
		typename PullStream<T>::PositionType streamPosition() const;
		typename PullStream<T>::PositionType realStreamPosition() const;
		
	private:
		PullStream<T>& fStream;
		typename PullStream<T>::PositionType fPos;
		typename PullStream<T>::PositionType fEndPos;
		bool fShowRealPos; // FIXME: this optional mode should be removed
	};
}
#endif

// include implementation
#include "overlaypullstream.cxx"
