/*!
	\file sequenceview.h
	\brief Contains the SequenceView class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-09-26
// Last time modified:	2008-09-26

#ifndef DEF_SEQUENCEVIEW_H
 #define DEF_SEQUENCEVIEW_H
 
 #include <vector>
 
namespace HYPER_Common {
	
	/// A universal wrapper for a sequence of pointers to const elements in a
	/// container. The pointers should not be null. The interface of this class
	/// is made somewhat similar to that of std::vector.
	template <class T>
	class SequenceView {
	public:
		typedef size_t size_type;
		typedef T element_type;
		typedef element_type const& reference;
		typedef element_type const* const_iterator;
		
		SequenceView(const std::vector<element_type*>& v)
		 : fStartPointer(&v[0]), fSize(v.size())
		{
			//
		}
		
		SequenceView(const std::vector<element_type const*>& v)
		 : fStartPointer(&v[0]), fSize(v.size())
		{
			//
		}
		
		SequenceView(element_type const* firstElement, size_type size)
		 : fStartPointer(firstElement), fSize(size)
		{
			//
		}
		
		bool empty() const {
			return fSize == 0;
		}
		
		size_t size() const {
			return fSize;
		}
		
		reference operator[](size_type index) const {
			return fStartPointer + index;
		}
		
		const_iterator begin() const {
			return fStartPointer;
		}
		
		const_iterator end() const {
			return fStartPointer + fSize;
		}
		
		reference front() const {
			return fStartPointer[0];
		}
		
		reference back() const {
			return fStartPointer[fSize - 1];
		}
		
	private:
		element_type const* fStartPointer;
		size_type fSize;
	};
}
#endif
