/*!
	\file streambase.h
	\brief Contains the StreamBase class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-03
// Last time modified: 	2008-02-06

#ifndef DEF_STREAMBASE_H
 #define DEF_STREAMBASE_H

namespace HYPER_Common {
	
	/// Base class for streams
	template<class T>
	class StreamBase {
	public:
		virtual ~StreamBase() { /*empty*/ }
		
		/// Type of data that flows through the stream
		typedef T DataType;
		
		/// Type used for stream positions
		typedef unsigned long PositionType;
		
		/// Is the stream still OK?
		virtual bool streamOK() const = 0;
		
		/// Get the current position in this stream. First position is zero.
		virtual PositionType streamPosition() const = 0;
		
	protected:
		StreamBase() { /*empty*/ }
	};
}
#endif
