/*!
	\file streamprocessor.h
	\brief Contains the StreamProcessor class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-11
// Last time modified: 	2008-03-16

#ifndef DEF_STREAMPROCESSOR_H
 #define DEF_STREAMPROCESSOR_H
 
 #include "streams_fwdecl.h"
 
 #include <boost/shared_ptr.hpp>
 #include <deque>
 
namespace HYPER_Common {
	
	/// Abstract base for a stream processing element
	template<class IN, class OUT>
	class StreamProcessor {
	protected:
		StreamProcessor() {
			//
		}
		
		class State {
		public:
			State(unsigned inputNeeded = 1) : fgInputNeeded(inputNeeded) {}
			virtual ~State() {}
			
			unsigned fgInputNeeded;
		};
		
		virtual boost::shared_ptr<State> getStartState() const {
			return boost::shared_ptr<State>(new State());
		}
		
	public:
		virtual ~StreamProcessor() { /*empty*/ }
		
		enum ProcessStatus {
			kOK, kNeedMoreData
		};
		
		/// Process some data.
		/// The processor should get its input from the front of the deque, and
		/// put its output on the front of the output deque.
		virtual ProcessStatus processData(std::deque<IN>& input,
			bool endOfInputReached, std::deque<OUT>& output,
			State& processState) const = 0;
		
		PullStream<OUT>* puller(PullStream<IN>& source) const;
		PushStream<IN>* pusher(PushStream<OUT>& dest) const;
		
	private:
		class PullWrapper;
		class PushWrapper;
	};
}
#endif

// include implementation
#include "streamprocessor.cxx"
