/*!
	\file switchablepushstream.cxx
	\brief Implements the SwitchablePushStream class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-10-20
// Last time modified:	2008-02-21

#ifndef DEF_SWITCHABLEPUSHSTREAM_CXX
 #define DEF_SWITCHABLEPUSHSTREAM_CXX
 
 #include "switchablepushstream.h"

namespace HYPER_Common {
	
	template <class T>
	SwitchablePushStream<T>::SwitchablePushStream(PushStream<T>& output)
	: fOut(&output), fEnabled(true), fStreamPos(0)
	{
		//
	}
	
	template <class T>
	bool SwitchablePushStream<T>::streamOK() const {
		return !fEnabled || fOut->streamOK();
	}
	
	template <class T>
	bool SwitchablePushStream<T>::push(T c) {
		bool success;
		if (fEnabled) {
			success = fOut->push(c);
		}
		else {
			success = true;
		}
		
		// The position is always increased, in order to maintain the illusion
		// of a continuous and consistent stream. Switching back end should not
		// give the stream position a completely different value.
		if (success) {
			fStreamPos++;
		}
		
		return success;
	}
	
	template <class T>
	typename PushStream<T>::PositionType
	SwitchablePushStream<T>::streamPosition() const {
		return fStreamPos;
	}
	
	template <class T>
	void SwitchablePushStream<T>::enable(bool value) {
		fEnabled = value;
	}
	
	template <class T>
	bool SwitchablePushStream<T>::enabled() const {
		return fEnabled;
	}
	
	template <class T>
	void
	SwitchablePushStream<T>::switchTo(PushStream<T>& output)
	{
		fOut = &output;
	}
}
#endif
