/*!
	\file switchablepushstream.h
	\brief Contains the SwitchablePushStream class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-10-20
// Last time modified:	2008-02-21

#ifndef DEF_SWITCHABLEPUSHSTREAM_H
 #define DEF_SWITCHABLEPUSHSTREAM_H
 
 #include "pushstream.h"

namespace HYPER_Common {
	
	/// Output class that supports on/off switching and back end switching
	template <class T>
	class SwitchablePushStream : public PushStream<T> {
	public:
		/// Creates a new (enabled) SwitchableCharOutputStream that writes
		/// to a given back end stream.
		SwitchablePushStream(PushStream<T>& output);
		
		bool streamOK() const;
		bool push(T c);
		typename PushStream<T>::PositionType streamPosition() const;
		
		/// Enables the stream, or disables it if 'false' argument is given
		void enable(bool value = true);
		
		/// Disables the stream. Writing to it will have no effect until it is
		/// reenabled.
		void disable() {
			this->enable(false);
		}
		
		/// Determine if the stream is enabled or not
		bool enabled() const;
		
		/// Switches the output back end to another stream. This does NOT
		/// automatically reenable the stream if it is disabled!
		void switchTo(PushStream<T>& output);
		
	private:
		PushStream<T>* fOut;
		bool fEnabled;
		typename PushStream<T>::PositionType fStreamPos;
	};
}
#endif

// include implementation
#include "switchablepushstream.cxx"
