/*!
	\file unixfilepath.h
	\brief Represents a UNIX type file or directory pathname
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2005-09-12
// Last time modified: 	2008-03-06

#ifndef DEF_UNIXFILEPATH_H
 #define DEF_UNIXFILEPATH_H
 
 #include "filepath.h"
 
 #include <deque>

namespace HYPER_Common {
	
	/// Implementation of FilePath for Unix pathnames.
	class UnixFilePath : public FilePath {
	public:
		static UnixFilePath* createFromString(UTF8String path);
		
		bool isAbsolute() const;
		void makeAbsolute(FilePath const& baseDir);
		UnixFilePath* getRelative(FilePath const& baseDir) const;
		
		unsigned countLevels() const;
		UnixFilePath* getParentDirectory() const;
		UnixFilePath* getChildPath(UTF8String sub) const;
		
		void markAsDirectory();
		bool markedAsDirectory() const;
		void markAsFile();
		
		UnixFilePath* getLargestCommonPrefix(
			FilePath const& otherPath) const;
		
		unsigned int getExtensionLength() const;
		UTF8String getExtension() const;
		void replaceExtension(UTF8String newExt);
		void addExtension(UTF8String ext);
		
		UTF8String getShortName() const;
		UTF8String toString() const;
		
		UnixFilePath* copy() const;
		
		bool operator==(FilePath const&) const;
		
		bool isRootDir() const;
		
	private:
		bool simplify();
		static bool checkValidComponent(UTF8String s);
		static UnixFilePath const* toUnixPath(FilePath const& p);
		
		typedef std::deque<UTF8String> UPDeque;
		
		UnixFilePath();
		UnixFilePath(UPDeque const&, bool leadingSlash,
			bool trailingSlash);
		
		bool fHasRoot;
		bool fTrailingSlash;
		UPDeque fLst;
	};
}
#endif
