/*!
	\file utf8decoder.cxx
	\brief Implements the UTF8Decoder class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-13
// Last time modified: 	2008-03-17

#include "utf8decoder.h"

namespace HYPER_Common {
	
	UTF8Decoder::ProcessStatus
	UTF8Decoder::processData(std::deque<UTF8Byte>& input,
		bool endOfInputReached, std::deque<CodePoint>& output,
		State& processState) const
	{
		// check that we have enough input bytes
		UTF8Byte firstB = input.front();
		unsigned bytesNeeded = 1 + firstB.getFollowerCount();
		if (bytesNeeded > input.size()) {
			processState.fgInputNeeded = bytesNeeded;
			return kNeedMoreData;
		}
		
		// put bytes in buffer
		input.pop_front(); // pop first byte
		UTF8Byte buf[4];
		buf[0] = firstB;
		for (unsigned i = 1; i < bytesNeeded; ++i) {
			buf[i] = input.front();
			input.pop_front();
		}
		
		// decode
		output.push_front(UTF8Byte::decodeFromValidStorage(buf));
		processState.fgInputNeeded = 1; // next byte could be single char
		
		return kOK;
	}
}
