/*!
	\file utf8encoder.cxx
	\brief Implements the UTF8Encoder class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-12
// Last time modified: 	2008-03-16

#include "utf8encoder.h"

namespace HYPER_Common {
	
	class UTF8Encoder::UTF8Dequer : public UTF8Consumer {
	public:
		UTF8Dequer(std::deque<UTF8Byte>& d) : fDeque(d)
		{
			//
		}
		
		void consume(UTF8Byte b) {
			fDeque.push_front(b);
		}
		
	private:
		std::deque<UTF8Byte>& fDeque;
	};
	
	UTF8Encoder::ProcessStatus
	UTF8Encoder::processData(std::deque<CodePoint>& input,
		bool endOfInputReached, std::deque<UTF8Byte>& output,
		State& processState) const
	{
		CodePoint c = input.front();
		input.pop_front();
		UTF8Dequer d(output);
		UTF8Byte::encode(c, d);
		return kOK;
	}
}
