/*!
	\file utf8pushstream.h
	\brief Contains the UTF8PushStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-06
// Last time modified: 	2008-03-19

#ifndef DEF_UTF8PUSHSTREAM_H
 #define DEF_UTF8PUSHSTREAM_H
 
 #include "pushstream.h"
 #include "utf8.h"
 #include "utf8string.h"

namespace HYPER_Common {
	
	typedef PushStream<UTF8Byte> UTF8BasicPushStream;
	
	/// Adapter class to have a PushStream for UTF-8 bytes act as
	/// an UTF8Consumer
	class UTF8ConsumerStreamAdapter : public UTF8Consumer {
	public:
		UTF8ConsumerStreamAdapter(UTF8BasicPushStream& stream)
		 : fStream(stream)
		{
			//
		}
		
		void consume(UTF8Byte b) {
			fStream.push(b);
		}
		
	private:
		UTF8BasicPushStream& fStream;
	};
	
	/// Specialized class for UTF-8 output
	class UTF8PushStream : public UTF8BasicPushStream, public UTF8Consumer {
	public:
		~UTF8PushStream() { /*empty*/ }
		
		void consume(UTF8Byte b) {
			push(b);
		}
		
		void print(CodePoint c) {
			UTF8Byte::encode(c, *this);
		}
		
		void print(UTF8String s) {
			s.print(*this);
		}
		
		void line() {
			push(UTF8Byte('\n'));
		}
		
		void line(CodePoint c) {
			print(c);
			line();
		}
		
		void line(UTF8String s) {
			print(s);
			line();
		}
		
	protected:
		UTF8PushStream() { /*empty*/ }
	};
}
#endif
