/*!
	\file utf8stdinstream.cxx
	\brief Implements the UTF8StdInStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-08
// Last time modified: 	2008-03-20

#include "utf8stdinstream.h"

namespace HYPER_Common {
	
	UTF8StdInStream::UTF8StdInStream(std::istream& s)
	 : fStream(s), fPos(0)
	{
		//
	}
	
	bool UTF8StdInStream::streamOK() const {
		return !!fStream;
	}
	
	bool UTF8StdInStream::look(UTF8Byte& b) {
		char c;
		if (!!fStream.get(c) && !!fStream.putback(c)) {
			b = UTF8Byte(c);
			return true;
		}
		else {
			return false;
		}
	}
	
	bool UTF8StdInStream::pullNext() {
		char c;
		if (!!fStream.get(c)) {
			++fPos;
			return true;
		}
		else {
			return false;
		}
	}
	
	UTF8StdInStream::PositionType UTF8StdInStream::streamPosition() const {
		return fPos;
	}
}
