/*!
	\file utf8stdinstream.h
	\brief Contains the UTF8StdInStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-08
// Last time modified: 	2008-03-20

#ifndef DEF_UTF8STDINSTREAM_H
 #define DEF_UTF8STDINSTREAM_H
 
 #include "pullstream.h"
 #include "utf8.h"
 
 #include <istream>

namespace HYPER_Common {
	
	/// UTF-8 input stream that gets its input from a std::istream
	class UTF8StdInStream : public PullStream<UTF8Byte> {
	public:
		UTF8StdInStream(std::istream& s);
		~UTF8StdInStream() { /*empty*/ }
		
		bool streamOK() const;
		bool look(UTF8Byte& c);
		bool pullNext();
		PositionType streamPosition() const;
		
	private:
		std::istream& fStream;
		PositionType fPos;
	};
}
#endif
