/*!
	\file utf8stdoutstream.cxx
	\brief Implements the UTF8StdOutStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-02-03
// Last time modified: 	2008-02-03

#include "utf8stdoutstream.h"

namespace HYPER_Common {
	
	UTF8StdOutStream::UTF8StdOutStream(std::ostream& s)
	: fStream(s), fPos(0)
	{
		//
	}
	
	bool UTF8StdOutStream::push(UTF8Byte d) {
		bool ok;
		Byte b = d.toByte();
		
		if (b == static_cast<Byte>('\n')) {
			ok = !!(fStream << std::endl);
		}
		else {
			ok = !!fStream.put(b);
		}
		
		if (ok) {
			fPos++;
		}
		
		return ok;
	}
	
	bool UTF8StdOutStream::streamOK() const {
		return !!fStream;
	}
	
	UTF8StdOutStream::PositionType
	UTF8StdOutStream::streamPosition() const
	{
		return fPos;
	}
}
