/*!
	\file utf8textflowstream.cxx
	\brief Implements the UTF8TextFlowStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-02-13
// Last time modified:	2008-03-14

#include "utf8textflowstream.h"

namespace HYPER_Common {
	
	UTF8TextFlowStream::UTF8TextFlowStream(PushStream<UTF8Byte>& output)
	 : fOut(output), fEnc(), fDec(), fEncoder(fEnc.pusher(fOut)),
		fTextFlow(*fEncoder), fDecoder(fDec.pusher(fTextFlow))
	{
		//
	}
	
	UTF8TextFlowStream::UTF8TextFlowStream(PushStream<UTF8Byte>& output,
		TextFlowOptions opt)
	 : fOut(output), fEnc(), fDec(), fEncoder(fEnc.pusher(fOut)),
		fTextFlow(*fEncoder, opt), fDecoder(fDec.pusher(fTextFlow))
	{
		//
	}
	
	UTF8TextFlowStream::~UTF8TextFlowStream() {
		delete fDecoder;
		delete fEncoder;
	}
	
	bool UTF8TextFlowStream::streamOK() const {
		return fDecoder->streamOK();
	}
	
	bool UTF8TextFlowStream::push(UTF8Byte c) {
		return fDecoder->push(c);
	}
	
	unsigned long UTF8TextFlowStream::streamPosition() const {
		return fDecoder->streamPosition();
	}
	
	void UTF8TextFlowStream::increaseIndent(unsigned int amount) {
		fTextFlow.increaseIndent(amount);
	}
	
	void UTF8TextFlowStream::decreaseIndent(unsigned int amount) {
		fTextFlow.decreaseIndent(amount);
	}
}
