/*!
	\file utf8textflowstream.h
	\brief Contains the UTF8TextFlowStream class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-02-13
// Last time modified:	2008-03-14

#ifndef DEF_UTF8TEXTFLOWSTREAM_H
 #define DEF_UTF8TEXTFLOWSTREAM_H
 
 #include "textflowstream.h"
 #include "utf8decoder.h"
 #include "utf8encoder.h"
 #include "utf8pushstream.h"
 
 #include <deque>

namespace HYPER_Common {
	
	/// UTF8 wrapper for TextFlowStream
	class UTF8TextFlowStream : public UTF8PushStream, public TextFlowInterface {
	public:
		
		/**
			Creates a new TextFlowCharOutputStream for some output stream
			@param output The output stream to send output to
		*/
		UTF8TextFlowStream(PushStream<UTF8Byte>& output);
		
		~UTF8TextFlowStream();
		
		/**
			Creates a new TextFlowCharOutputStream for some output stream
			@param output The output stream to send output to
			@param opt	Specify options other than the default
		*/
		UTF8TextFlowStream(PushStream<UTF8Byte>& output, TextFlowOptions opt);
		
		bool streamOK() const;
		bool push(UTF8Byte c);
		unsigned long streamPosition() const;
		
		void increaseIndent(unsigned int amount = 1);
		void decreaseIndent(unsigned int amount = 1);
		
	private:
		PushStream<UTF8Byte>& fOut;
		UTF8Encoder fEnc;
		UTF8Decoder fDec;
		PushStream<CodePoint>* fEncoder;
		TextFlowStream fTextFlow;
		PushStream<UTF8Byte>* fDecoder;
	};
}
#endif
