/*!
	\file utf8verifier.h
	\brief Contains the UTF8Verifier class
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2008-03-16
// Last time modified: 	2008-03-19

#ifndef DEF_UTF8VERIFIER_H
 #define DEF_UTF8VERIFIER_H
 
 #include "streamprocessor.h"
 #include "utf8.h"

namespace HYPER_Common {
	
	/// Class that verifies a stream of UTF-8 bytes and corrects illegal bytes
	/// and sequences
	class UTF8Verifier : public StreamProcessor<UTF8Byte, UTF8Byte> {
		class VerifyState : public State {
		public:
			VerifyState(bool maySkipBOM)
			 : State(1), fgAtStartOfInput(true), fgMaySkipBOM(maySkipBOM)
			{
				//
			}
			
			bool fgAtStartOfInput;
			bool fgMaySkipBOM;
		};
	public:
		
		UTF8Verifier(bool maySkipBOM) : fMaySkipBOM(maySkipBOM) {}
		
		ProcessStatus processData(std::deque<UTF8Byte>& input,
			bool endOfInputReached, std::deque<UTF8Byte>& output,
			State& processState) const;
		
		boost::shared_ptr<State> getStartState() const {
			return boost::shared_ptr<State>(new VerifyState(fMaySkipBOM));
		}
		
	private:
		class UTF8Dequer;
		bool fMaySkipBOM;
	};
}
#endif
