/*!
	\file accesschecker.h
	\brief Contains the AccessChecker class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-11-07
// Last time modified:	2008-02-29

#ifndef DEF_ACCESSCHECKER_H
 #define DEF_ACCESSCHECKER_H
 
 #include "important_fwdecl.h"
 
 #include "common/cxx_macros.h"
 #include "common/utf8string.h"
 
namespace HYPER_Compiler {
	
	class ClassMemberAccess;
	
	/// Class containing methods for access checking
	class AccessChecker {
	public:
		
		/// Check access from somewhere to something
		static bool ATTR_WARN_UNUSED_RESULT
		check(ClassContext const& from, DeclaredItem const& item,
			Location const* loc, CompileInfo& info);
		
		/// Check access to a class member, assuming that we already have
		/// access to the class.
		static bool ATTR_WARN_UNUSED_RESULT
		checkMember(ClassContext const& from, DeclaredItem const& item,
			HYPER_Common::UTF8String memberName, Location const* loc,
			CompileInfo& info);
		
	private:
		static void
		getInfo(DeclaredItem const& item,
			ClassMemberAccess& isPublicMember, Class const*& memberOwner);
		
		/// Check if class a is somewhere inside class b
		static bool isInside(Class const* a, Class const* b);
		
		/*static bool ATTR_WARN_UNUSED_RESULT
		checkAccessInto(Class const* a, Class const* b, Location const* loc,
			CompileInfo& info);*/
	};
}
#endif
