/*!
	\file arrayclass.h
	\author Kevin André
	\brief Contains the Array class
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-29
// Last time modified:	2008-10-15

#ifndef DEF_ARRAYCLASS_H
 #define DEF_ARRAYCLASS_H
 
 #include "builtinclass.h"

namespace HYPER_Compiler {
	
	class Type;
	
	/// Class to represent the built-in `array' classes
	class Array : public BuiltInClass {
	public:
		Array(CompileInfo& info, const Type* baseType, unsigned int dimensions);
		virtual ~Array();
		
		virtual HYPER_Common::UTF8String getTypeName() const;
		
		void buildSymbolTable();
		
		Type const* getBaseType() const;
		unsigned int getDimensions() const;
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
		InitializationHandler* getInitHandler() const;
		
	private:
		class ArrayConstructor;
		class ArrayInitHandler;
		class IndexOperator;
		class ProcedureSize;
		
		void createConstructors();
		void createMembers();
		void createOperators();
		
		unsigned int fDims;
		Type const* fBaseType;
	};
}
#endif
