/*!
	\file arrayoperator.cxx
	\brief Implements the ArrayOperator class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-29
// Last time modified:	2008-11-09

#include "arrayoperator.h"

#include "base/asserts.h"

#include "parameter.h"
#include "type.h"

namespace HYPER_Compiler {
	
	ArrayOperator::ArrayOperator(std::vector<Parameter const*> parameters,
		Type const* result, Location const* loc)
	 : fParams(parameters), fResult(result), fLoc(loc)
	{
		// no action
	}
	
	ArrayOperator::~ArrayOperator() {
		// no action
	}
	
	HYPER_Common::UTF8String ArrayOperator::headerToString() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		sb += UTF8STR("operator[](");
		
		for (unsigned int i = 0; i < fParams.size(); ++i) {
			if (i > 0) {
				sb += UTF8STR(", ");
			}
			
			sb += fParams[i]->toString();
		}
		sb += UTF8STR(") : ");
		sb += fResult->toString();
		
		return sb.toString();
	}
	
	unsigned ArrayOperator::getParameterCount() const {
		return fParams.size();
	}
	
	std::vector<Parameter const*> const*
	ArrayOperator::getParameterList() const {
		return &fParams;
	}
	
	Type const* ArrayOperator::getResultType() const {
		return fResult;
	}
	
	unsigned int ArrayOperator::getDimensionCount() const {
		return fParams.size();
	}
	
	Location const* ArrayOperator::getLocation() const {
		return fLoc;
	}
	
	void ArrayOperator::markAsDuplicate() {
		// should not be possible yet
		hyp_assert_unreachable();
	}
	
	bool ArrayOperator::isDuplicate() const {
		return false;
	}
	
	HYPER_Common::UTF8String ArrayOperator::getOverloadHash() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		sb += UTF8STR("["); // indicates array operator
		
		for (unsigned int i = 0; i < fParams.size(); ++i) {
			sb += fParams[i]->getType()->getOverloadHash();
			sb += UTF8STR("]"); // separator & terminator character
		}
		
		return sb.toString();
	}
}
