/*!
	\file arrayoperator.h
	\brief Contains the ArrayOperator class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-29
// Last time modified:	2008-11-09

#ifndef DEF_ARRAYOPERATOR_H
 #define DEF_ARRAYOPERATOR_H
 
 #include "operators.h"
 #include "overloadable.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class ConstantValue;
	class Location;
	class Parameter;
	class Type;
	
	/// Represents an array indexing operator
	class ArrayOperator : public Overloadable  {
	public:
		ArrayOperator(std::vector<Parameter const*> parameters,
			Type const* result, Location const* loc);
		
		~ArrayOperator();
		
		HYPER_Common::UTF8String headerToString() const;
		
		/// Get the number of parameters
		unsigned getParameterCount() const;
		
		std::vector<Parameter const*> const* getParameterList() const;
		Type const* getResultType() const;
		
		unsigned int getDimensionCount() const;
		
		Location const* getLocation() const;
		
		void markAsDuplicate();
		bool isDuplicate() const;
		
		HYPER_Common::UTF8String getOverloadHash() const;
		
		virtual ConstantValue const* evaluateCompileTime(
			ConstantValue const* operand,
			std::vector<ConstantValue const*> params) = 0;
		
		// TODO : change return type
		virtual void generateCode() = 0;
		
	private:
		std::vector<Parameter const*> fParams;
		Type const* fResult;
		Location const* fLoc;
	};
}
#endif
