/*!
	\file arrayoperatorexpression.h
	\brief Represents an array operator expression
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-10
// Last time modified:	2008-10-15

#ifndef DEF_ARRAYOPERATOREXPRESSION_H
 #define DEF_ARRAYOPERATOREXPRESSION_H
 
 #include "expression.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class ArrayOperatorExpression : public Expression {
	public:
		ArrayOperatorExpression(Expression* arrayExpr,
			Expression* arraySelector);
		~ArrayOperatorExpression();
		
		void addSelector(Expression* s);
		
		HYPER_Common::UTF8String toString() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
		bool doFullResolve(Trace const& trace);
		
		TriBool isClassLValue() const;
		TriBool isPointerLValue() const;
		TriBool isCall() const;
		TriBool hasResult() const;
		TriBool resultAtCompileTime() const;
		
	private:
		bool checkSelectors(const std::vector<Expression const*>& selectors,
			unsigned firstSelector,
			Type const* curType) ATTR_WARN_UNUSED_RESULT;
		
		HYPER_Common::UTF8String printSelectors(unsigned begin,
			unsigned end) const;
		
		HYPER_Common::UTF8String printSelectorTypes(unsigned begin,
			unsigned end) const;
		
		HYPER_Common::UTF8String printSelectionBase(unsigned beginExclude) const;
		
		Expression* fArrayExpr;
		std::vector<Expression*> fSelectors;
		TriBool fIsClassLValue;
		TriBool fIsPointerLValue;
	};
}
#endif
