/*!
	\file arraytype.h
	\brief Contains the ArrayType class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-12-16
// Last time modified:	2008-10-19

#ifndef DEF_ARRAYTYPE_H
 #define DEF_ARRAYTYPE_H
 
 #include "declaredtype.h"

namespace HYPER_Compiler {
	
	/// Declared array type
	class ArrayType : public DeclaredType {
	public:
		ArrayType(std::vector<Expression*> dimensions, DeclaredType* base,
			const Location* loc);
		
		bool isPointer() const;
		bool isDoublePointer() const;
		bool isArray() const;
		bool isClass() const;
		
		bool isConst() const;
		void makeConst();
		
		HYPER_Common::UTF8String toString() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
	protected:
		bool doInterfaceResolve(Trace const& trace);
		
	private:
		ArrayType(ArrayType const& at);
		
		bool resolveDim(Expression* dim);
		
		DeclaredType* fBaseType;
		std::vector<Expression*> fArrayDims;
	};
}
#endif
