/*!
	\file assignmentoperator.h
	\brief Contains the AssignmentOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-12-17
// Last time modified:	2008-09-06

#ifndef DEF_ASSIGNMENTOPERATOR_H
 #define DEF_ASSIGNMENTOPERATOR_H
 
 #include "operators.h"
 
namespace HYPER_Compiler {
	
	class Location;
	class Type;
	
	/// Represents an assignment operator
	class AssignmentOperator {
	public:
		AssignmentOperator(Operators::AssignmentOperator op,
			Type const* rValueType, Location const* loc);
		
		virtual ~AssignmentOperator();
		
		Operators::AssignmentOperator getOperator() const;
		
		Type const* getRValueType() const;
		
		Location const* getLocation() const;
		
	private:
		Operators::AssignmentOperator fOp;
		Type const* fRValueT;
		Location const* fLoc;
	};
}
#endif
