/*!
	\file autogeneratedconstructor.cxx
	\brief Implements the AutoGeneratedConstructor class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-10-15
// Last time modified:	2009-01-05

#include "autogeneratedconstructor.h"

#include "base/asserts.h"
#include "base/location.h"

#include "class.h"
#include "parameter.h"
#include "type.h"

namespace HYPER_Compiler {
	
	AutoGeneratedConstructor*
	AutoGeneratedConstructor::createDefaultConstructor(Class* c)
	{
		AutoGeneratedConstructor* co = new AutoGeneratedConstructor(c, false);
		
		// TODO: generate initializer list
		
		return co;
	}
	
	AutoGeneratedConstructor*
	AutoGeneratedConstructor::createCopyConstructor(Class* c)
	{
		AutoGeneratedConstructor* co = new AutoGeneratedConstructor(c, true);
		
		Parameter* par =
			new Parameter(
				Type::createClassType(false, c),
				Parameter::kInByReference, UTF8STR("copySource"),
				Location::getAutoGenerated()
			);
		
		co->fParam.push_back(par);
		
		// TODO: generate initializer list
		
		return co;
	}
	
	AutoGeneratedConstructor::~AutoGeneratedConstructor() {
		//
	}
	
	Class* AutoGeneratedConstructor::getClass() const {
		return fClass;
	}
	
	ClassMemberAccess AutoGeneratedConstructor::getAccess() const {
		return ClassMemberAccess::createPublic();
	}
	
	const Location* AutoGeneratedConstructor::getLocation() const {
		return Location::getAutoGenerated();
	}
	
	bool AutoGeneratedConstructor::isImplicit() const {
		return fImplicit;
	}
	
	void AutoGeneratedConstructor::makeImplicit() {
		hyp_assert_unreachable();
	}
	
	void AutoGeneratedConstructor::markAsDuplicate() {
		hyp_assert_unreachable();
	}
	
	bool AutoGeneratedConstructor::isDuplicate() const {
		return false;
	}
	
	unsigned AutoGeneratedConstructor::getParameterCount() const {
		return fParam.size();
	}
	
	const std::vector<Parameter const*>*
	AutoGeneratedConstructor::getParameterList() const
	{
		return &fParam;
	}
	
	AutoGeneratedConstructor::AutoGeneratedConstructor(Class* c,
		bool isImplicit)
	 : Constructor(c->getCompileInfo()), fClass(c), fImplicit(isImplicit)
	{
		//
	}
}
