/*!
	\file autogeneratedconstructor.h
	\brief Contains the AutoGeneratedConstructor class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-10-15
// Last time modified:	2008-11-08

#ifndef DEF_AUTOGENCONSTRUCTOR_H
 #define DEF_AUTOGENCONSTRUCTOR_H
 
 #include "constructor.h"

namespace HYPER_Compiler {
	
	/// Class for autogenerated constructors
	class AutoGeneratedConstructor : public Constructor {
	public:
		
		static AutoGeneratedConstructor* createDefaultConstructor(Class* c);
		static AutoGeneratedConstructor* createCopyConstructor(Class* c);
		
		~AutoGeneratedConstructor();
		
		Class* getClass() const;
		
		ClassMemberAccess getAccess() const;
		const Location* getLocation() const;
		
		unsigned getParameterCount() const;
		const std::vector<Parameter const*>* getParameterList() const;
		
		bool isImplicit() const;
		void makeImplicit();
		
		void markAsDuplicate();
		bool isDuplicate() const;
		
	private:
		AutoGeneratedConstructor(Class* c, bool isImplicit);
		
		Class* fClass;
		bool fImplicit;
		std::vector<Parameter const*> fParam;
	};
}
#endif
