/*!
	\file binaryoperator.cxx
	\brief Implements the BinaryOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-26
// Last time modified:	2008-10-13

#include "overloadedbinaryoperator.h"

#include "type.h"

namespace HYPER_Compiler {
	
	HYPER_Common::UTF8String
	BinaryOperator::getPrintPrototype(Operators::BinaryOperator op,
		Type const* leftT, Type const* rightT)
	{
		return UTF8STR("operator%1(%2, %3)").compose(
			Operators::toString(op), leftT->toString(), rightT->toString()
		);
	}
	
	BinaryOperator::BinaryOperator(
		Operators::BinaryOperator op, Type const* leftOperand,
		Type const* rightOperand, bool commutative, Type const* result,
		Location const* loc)
	 : fOpSym(op), fLeftT(leftOperand), fRightT(rightOperand),
		fCommut(commutative), fResult(result), fLoc(loc)
	{
		// no action
	}
	
	BinaryOperator::~BinaryOperator() {
		// no action
	}
	
	Operators::BinaryOperator BinaryOperator::getOperator() const {
		return fOpSym;
	}
	
	Type const* BinaryOperator::getLeftOperandType() const {
		return fLeftT;
	}
	
	Type const* BinaryOperator::getRightOperandType() const {
		return fRightT;
	}
	
	bool BinaryOperator::isCommutative() const {
		return fCommut;
	}
	
	Type const* BinaryOperator::getResultType() const {
		return fResult;
	}
	
	Location const* BinaryOperator::getLocation() const {
		return fLoc;
	}
	
	HYPER_Common::UTF8String BinaryOperator::toString() const {
		return getPrintPrototype(fOpSym, fLeftT, fRightT);
	}
}
