/*!
	\file binaryoperator.h
	\brief Contains the BinaryOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-26
// Last time modified:	2009-09-06

#ifndef DEF_BINARYOPERATOR_H
 #define DEF_BINARYOPERATOR_H
 
 #include "common/utf8string.h"
 
 #include "important_fwdecl.h"
 #include "operators.h"
 
namespace HYPER_Compiler {
	
	/// Represents an binary operator
	class BinaryOperator {
	public:
		static HYPER_Common::UTF8String getPrintPrototype(
			Operators::BinaryOperator op, Type const* leftT,
			Type const* rightT);
		
		BinaryOperator(Operators::BinaryOperator op,
			Type const* leftOperand, Type const* rightOperand,
			bool commutative, Type const* result, Location const* loc);
		
		virtual ~BinaryOperator();
		
		Operators::BinaryOperator getOperator() const;
		
		Type const* getLeftOperandType() const;
		Type const* getRightOperandType() const;
		
		bool isCommutative() const;
		
		Type const* getResultType() const;
		
		Location const* getLocation() const;
		
		HYPER_Common::UTF8String toString() const;
		
		virtual ConstantValue const*
		evaluateCompileTime(ConstantValue const* leftOp,
			ConstantValue const* rightOp) = 0;
		
	private:
		Operators::BinaryOperator fOpSym;
		Type const* fLeftT;
		Type const* fRightT;
		bool fCommut;
		Type const* fResult;
		Location const* fLoc;
	};
}
#endif
