/*!
	\file boolclass.h
	\author Kevin André
	\brief Contains the Bool class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-25
// Last time modified:	2008-12-21

#ifndef DEF_BOOLCLASS_H
 #define DEF_BOOLCLASS_H
 
 #include "assignmentoperator.h"
 #include "binaryoperator.h"
 #include "builtinclass.h"
 #include "builtinconstructor.h"
 #include "builtinprocedure.h"
 #include "unaryoperator.h"

namespace HYPER_Compiler {
	
	/// Represents the built-in `bool' class
	class Bool : public BuiltInClass {
	public:
		Bool(CompileInfo& info);
		~Bool();
		
		HYPER_Common::UTF8String getTypeName() const;
		
		void buildSymbolTable();
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
		bool canBeInvisiblyPassedByValue() const {
			return true;
		}
		
	private:
		class BoolConstructor;
		class ProcedureNegate;
		class NotOperator;
		class AndOperator;
		class OrOperator;
		class ConsequenceOperator;
		class EqualsOperator;
		class DiffersOperator;
		class BoolAssignOp;
		class AndAssignOp;
		class OrAssignOp;
		
		void createConstructors();
		void createMembers();
		void createOperators();
	};
}
#endif
