/*!
	\file builtinclass.h
	\author Kevin André
	\brief Contains the BuiltInClass class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-09-23
// Date modified:	2008-10-13

#ifndef DEF_BUILTINCLASS_H
 #define DEF_BUILTINCLASS_H
 
 #include "class.h"
 
 #include "builtintypes.h"
 #include "simpleparameter.h"

namespace HYPER_Compiler {
	
	class ArrayOperator;
	class AssignmentOperator;
	class BinaryOperator;
	class CrementOperator;
	class OverloadedProcedure;
	class UnaryOperator;
	
	/// Represents a built-in class
	class BuiltInClass : public Class {
	public:
		virtual ~BuiltInClass();
		
		virtual ClassMemberAccess getAccess() const;
		virtual AbstractSealedStatic getASS() const;
		
		virtual const Location* getLocation() const;
		
		virtual void buildSymbolTable() = 0;
		
		BinOpsMap const& getBinaryOperators() const;
		CremOpsMap const& getCrementOperators() const;
		UnOpsMap const& getUnaryOperators() const;
		AssignOpsMap const& getAssignmentOperators() const;
		OverloadedArrayOperator const* getArrayOperators() const;
		
		Class const* getParentClass() const;
		
		/// Create a built-in Parameter
		Parameter* createParameter(HYPER_Common::UTF8String name,
			BuiltInTypes::Primitive type,
			SimpleParameter::Passing p = SimpleParameter::kInByReference) const;
		
		/// Create a built-in Parameter
		Parameter* createParameter(HYPER_Common::UTF8String name,
			Type const* type,
			SimpleParameter::Passing p = SimpleParameter::kInByReference) const;
		
	protected:
		BuiltInClass(CompileInfo& info, AbstractSealedStatic ass,
			Class* parent = 0);
		
		void registerOperator(ArrayOperator* op);
		void registerOperator(AssignmentOperator* op);
		void registerOperator(BinaryOperator* op);
		void registerOperator(CrementOperator* op);
		void registerOperator(UnaryOperator* op);
		
	private:
		Class* fParent;
		Location const* fBuiltInLoc;
		AbstractSealedStatic fASS;
		ClassMemberAccess fPublicAccess;
		
		UnOpsMap fUnOps;
		BinOpsMap fBinOps;
		CremOpsMap fCremOps;
		AssignOpsMap fAssignOps;
		OverloadedArrayOperator* fArrayOps;
	};
}
#endif
