/*!
	\file builtinconstructor.cxx
	\brief Implementation of the BuiltInConstructor class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-12
// Last time modified:	2008-11-08

#include "builtinconstructor.h"

#include "base/asserts.h"
#include "base/location.h"

#include "class.h"

namespace HYPER_Compiler {
	
	BuiltInConstructor::BuiltInConstructor(Class* c,
		bool implicit)
	 : Constructor(c->getCompileInfo()), fClass(c),
		fAccess(ClassMemberAccess::createPublic()),
		fImplicit(implicit)
	{
		// nothing here
	}
	
	BuiltInConstructor::BuiltInConstructor(Class* c,
		ClassMemberAccess access, bool implicit)
	 : Constructor(c->getCompileInfo()), fClass(c),
		fAccess(access), fImplicit(implicit)
	{
		// nothing here
	}
	
	BuiltInConstructor::~BuiltInConstructor() {
		// no action
	}
	
	Class* BuiltInConstructor::getClass() const {
		return fClass;
	}
	
	ClassMemberAccess BuiltInConstructor::getAccess() const {
		return fAccess;
	}
	
	const Location* BuiltInConstructor::getLocation() const {
		return Location::getBuiltIn();
	}
	
	bool BuiltInConstructor::isImplicit() const {
		return fImplicit;
	}
	
	void BuiltInConstructor::makeImplicit() {
		hyp_assert_unreachable();
	}
	
	unsigned BuiltInConstructor::getParameterCount() const {
		return getParameterList()->size();
	}
	
	void BuiltInConstructor::markAsDuplicate() {
		hyp_assert_unreachable();
	}
	
	bool BuiltInConstructor::isDuplicate() const {
		return false;
	}
}
