/*!
	\file builtinconstructor.h
	\brief Contains the BuiltInConstructor class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-12
// Last time modified:	2007-11-08

#ifndef DEF_BUILTINCONSTRUCTOR_H
 #define DEF_BUILTINCONSTRUCTOR_H
 
 #include "constructor.h"

namespace HYPER_Compiler {
	
	/// Base class for constructors of built-in classes
	class BuiltInConstructor : public Constructor {
	public:
		/** Create a built-in constructor with public access.
			\param c  Parent class
			\param implicit  Is the constructor implicit?
		*/
		BuiltInConstructor(Class* c, bool implicit);
		
		/** Create a built-in constructor
			\param c  Parent class
			\param access  Public, protected or private constructor?
			\param implicit  Is the constructor implicit?
		*/
		BuiltInConstructor(Class* c, ClassMemberAccess access, bool implicit);
		
		virtual ~BuiltInConstructor();
		
		Class* getClass() const;
		
		ClassMemberAccess getAccess() const;
		const Location* getLocation() const;
		
		bool isImplicit() const;
		void makeImplicit();
		
		unsigned getParameterCount() const;
		
		void markAsDuplicate();
		bool isDuplicate() const;
		
	private:
		Class* fClass;
		ClassMemberAccess fAccess;
		bool fImplicit;
	};
}
#endif
