/*!
	\file builtinintegralclass.h
	\author Kevin André
	\brief Contains the BuiltInIntegralClass class
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2007-06-23
// Last modified:	2009-02-08

#ifndef DEF_BUILTININTEGRALCLASS_H
 #define DEF_BUILTININTEGRALCLASS_H
 
 #include "base/platform.h"
 
 #include "builtinclass.h"
 #include "constantvalue.h"
 #include "integersizes.h"

namespace HYPER_Compiler {
	
	class Type;
	
	/// Common ancestor for `byte' and `int(...)' and `nat(...)' types
	class BuiltInIntegralClass : public BuiltInClass {
	public:
		
		virtual ~BuiltInIntegralClass();
		
		HYPER_Common::UTF8String getTypeName() const;
		HYPER_Common::UTF8String getDeclarationName() const;
		
		IntegerSize getSize() const;
		bool isSigned() const;
		
		void buildSymbolTable();
		
		Type const* getType() const;
		
		ConstantValue::nat_t getMaxUnsignedValue() const;
		ConstantValue::int_t getMaxSignedValue() const;
		ConstantValue::int_t getMinSignedValue() const;
		
		virtual Type const* getAbsResultType() const = 0;
		
		virtual IntegerSize getNegateResultSize() const = 0;
		virtual Type const* getNegateResultType() const = 0;
		
		virtual Type const* getTruncateToSignedResultType() const = 0;
		
		bool canBeInvisiblyPassedByValue() const;
		
	protected:
		BuiltInIntegralClass(CompileInfo& info,
			HYPER_Common::UTF8String simpleName, IntegerSize s, bool isSigned);
		
		virtual void createConstructors();
		void addConversionConstructor(Type const* paramT, bool paramSignedT);
		virtual Type const* getClassType() const = 0;
		
	private:
		class BIICConstructor;
		class BIICConvConstructor;
		class BIICProcedureTurnSign;
		class BIICProcedureAbs;
		class BIICProcedureTruncateToSigned;
		class BIICProcedureTruncateToByte;
		class BIICProcedureMin;
		class BIICProcedureMax;
		class BIICPositiveOperator;
		class BIICNegativeOperator;
		class BIICSuccessorOperator;
		class BIICPredecessorOperator;
		class BIICEqualsOperator;
		class BIICDiffersOperator;
		class BIICLessOperator;
		class BIICLessEqOperator;
		class BIICGreaterOperator;
		class BIICGreaterEqOperator;
		class BIICAddOperator;
		class BIICSubtractOperator;
		class BIICMultiplyOperator;
		class BIICDivideOperator;
		class BIICModuloOperator;
		class BIICIncrementOperator;
		class BIICDecrementOperator;
		class BIICAssignOp;
		class BIICAddAssignOp;
		class BIICSubtractAssignOp;
		class BIICMultiplyAssignOp;
		class BIICDivideAssignOp;
		class BIICModuloAssignOp;
		
		static ConstantValue::nat_t getMaxUnsignedValue(IntegerSize s);
		static ConstantValue::int_t getMaxSignedValue(IntegerSize s);
		static ConstantValue::int_t getMinSignedValue(IntegerSize s);
		
		void createMembers();
		void createOperators();
		
		Type const* fType;
		HYPER_Common::UTF8String fName;
		IntegerSize fSize;
		bool fSigned;
	};
}
#endif
