/*!
	\file builtinprocedure.cxx
	\brief Implementation of the BuiltInProcedure class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-09
// Last time modified:	2008-11-08

#include "builtinprocedure.h"

#include "base/asserts.h"
#include "base/location.h"

#include "class.h"

namespace HYPER_Compiler {
	
	BuiltInProcedure::BuiltInProcedure(Class* c,
		HYPER_Common::UTF8String caption,
		AbstractSealedStatic ass, ClassMemberAccess access, bool isConst,
		Type const* returnType)
	 : Procedure(c->getCompileInfo()), fClass(c),
		fFullName(QualifiedName(c->getTypeName(), caption)),
		fASS(ass), fAccess(access), fIsConst(isConst), fReturnType(returnType)
	{
		// no action
	}
	
	BuiltInProcedure::~BuiltInProcedure() {
		// no action
	}
	
	HYPER_Common::UTF8String BuiltInProcedure::getCaption() const {
		return fFullName.getShortName();
	}
	
	QualifiedName BuiltInProcedure::getFullName() const {
		return fFullName;
	}
	
	Class* BuiltInProcedure::getClass() const {
		return fClass;
	}
	
	AbstractSealedStatic BuiltInProcedure::getASS() const {
		return fASS;
	}
	
	ClassMemberAccess BuiltInProcedure::getAccess() const {
		return fAccess;
	}
	
	bool BuiltInProcedure::isConst() const {
		return fIsConst;
	}
	
	unsigned BuiltInProcedure::getParameterCount() const {
		hyp_assert(getParameterList() != 0);
		return getParameterList()->size();
	}
	
	bool BuiltInProcedure::hasReturnType() const {
		return fReturnType != 0;
	}
	
	Type const* BuiltInProcedure::getReturnType() const {
		hyp_assert(hasReturnType());
		return fReturnType;
	}
	
	const Location* BuiltInProcedure::getLocation() const {
		return Location::getBuiltIn();
	}
	
	void BuiltInProcedure::markAsDuplicate() {
		// shouldn't happen for built-in types
		hyp_assert_unreachable();
	}
	
	bool BuiltInProcedure::isDuplicate() const {
		return false;
	}
}
