/*!
	\file builtinprocedure.h
	\brief Contains the BuiltInProcedure class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-09
// Last time modified:	2008-11-08

#ifndef DEF_BUILTINPROCEDURE_H
 #define DEF_BUILTINPROCEDURE_H
 
 #include "procedure.h"

namespace HYPER_Compiler {
	
	/// Represents a procedure that is built-in into the compiler
	class BuiltInProcedure : public Procedure {
	public:
		BuiltInProcedure(Class* c, HYPER_Common::UTF8String caption,
			AbstractSealedStatic ass, ClassMemberAccess access,
			bool isConst, Type const* returnType);
		virtual ~BuiltInProcedure();
		
		HYPER_Common::UTF8String getCaption() const;
		QualifiedName getFullName() const;
		
		Class* getClass() const;
		
		AbstractSealedStatic getASS() const;
		ClassMemberAccess getAccess() const;
		bool isConst() const;
		
		unsigned getParameterCount() const;
		
		bool hasReturnType() const;
		Type const* getReturnType() const;
		
		const Location* getLocation() const;
		
		// TODO: try to get rid of these for built-in procedures if possible
		void markAsDuplicate();
		bool isDuplicate() const;
		
	private:
		Class* fClass;
		QualifiedName fFullName;
		AbstractSealedStatic fASS;
		ClassMemberAccess fAccess;
		bool fIsConst;
		Type const* fReturnType;
	};
}
#endif
