/*!
	\file builtintypes.h
	\brief Contains the BuiltInTypes class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-24
// Last time modified:	2008-10-15

#ifndef DEF_BUILTINTYPES_H
 #define DEF_BUILTINTYPES_H
 
 #include "lexer/keywords.h"
 
 #include "important_fwdecl.h"
 
namespace HYPER_Compiler {
	
	class BuiltInTypes {
	public:
		enum Primitive {
			kNone = 0,
			kBool,
			kByte,
			kChar,
			kDouble,
			kInt,
			kInt16,
			kInt32,
			kInt64,
			kNat,
			kNat16,
			kNat32,
			kNat64,
			kObject,
			kReal,
			kSingle,
			kString
		};
		
		static Primitive fromKeyword(Keywords::Keyword k);
		
		static Type const*
		create(CompileInfo& info, Primitive p, bool isConst,
			const Location* loc = 0);
		
		static Type const* createBool(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createByte(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createChar(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createDouble(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createInt(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createInt16(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createInt32(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createInt64(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createNat(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createNat16(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createNat32(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createNat64(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createObject(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createReal(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createSingle(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		static Type const* createString(CompileInfo& info, bool isConst = false,
			const Location* loc = 0);
		
		static ValuePassing getBoolConversion(CompileInfo& info, Type const* t);
		static ValuePassing getNatConversion(CompileInfo& info, Type const* t);
	};
}
#endif
