/*!
	\file byteclass.cxx
	\author Kevin André
	\brief Implements the Byte class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-25
// Last time modified:	2009-02-08

#include "byteclass.h"

#include "base/asserts.h"

#include "builtintypes.h"
#include "parameter.h"

namespace HYPER_Compiler {
	
	Byte::Byte(CompileInfo& info)
	 : BuiltInIntegralClass(info, UTF8STR("byte"), k8Bit, false)
	{
		// no action
	}
	
	Byte::~Byte() {
		// no action
	}
	
	Type const* Byte::getClassType() const {
		return BuiltInTypes::createByte(getCompileInfo());
	}
	
	Type const* Byte::getAbsResultType() const {
		hyp_assert_unreachable();
	}
	
	IntegerSize Byte::getNegateResultSize() const {
		return k16Bit;
	}
	
	Type const* Byte::getNegateResultType() const {
		return BuiltInTypes::createInt16(getCompileInfo());
	}
	
	Type const* Byte::getTruncateToSignedResultType() const {
		return 0; // not supported
	}
}
