/*!
	\file chainedcompareexpression.h
	\brief Contains the ChainedCompareExpression class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-01-15
// Last time modified:	2008-02-20

#ifndef DEF_CHAINEDCOMPAREEXPRESSION_H
 #define DEF_CHAINEDCOMPAREEXPRESSION_H
 
 #include "expression.h"
 #include "operators.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class BinaryOperatorCandidate;
	class ClassContext;
	
	class ChainedCompareExpression : public Expression {
	public:
		ChainedCompareExpression(std::vector<Expression*> exprs,
			std::vector<Operators::BinaryOperator> ops,
			ClassContext const* context);
		~ChainedCompareExpression();
		
		HYPER_Common::UTF8String toString() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
		bool doFullResolve(Trace const& trace);
		
		TriBool isClassLValue() const;
		TriBool isPointerLValue() const;
		TriBool isCall() const;
		TriBool hasResult() const;
		TriBool resultAtCompileTime() const;
		
	private:
		BinaryOperatorCandidate const* findAndCheckOpCandidate(
			unsigned int opIndex) const;
		
		std::vector<Expression*> fExprs;
		std::vector<Operators::BinaryOperator> fOpSymbols;
		std::vector<BinaryOperatorCandidate const*> fOps;
		ClassContext const* fContext;
	};
}
#endif
