/*!
	\file classcontext.h
	\brief Contains the ClassContext class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-10-22
// Last time modified:	2008-12-20

#ifndef DEF_CLASSCONTEXT_H
 #define DEF_CLASSCONTEXT_H

namespace HYPER_Compiler {
	
	class Class;
	class CompileInfo;
	class Type;
	class Location;
	
	class ClassContext {
	public:
		ClassContext(CompileInfo& info);
		virtual ~ClassContext();
		
		/// Get the CompileInfo
		CompileInfo& getCompileInfo() const;
		
		/// Get the class of this context
		virtual Class const* getClass() const = 0;
		
		/// Is the class static in this context? (Then we don't have a "this" pointer)
		virtual bool isStatic() const = 0;
		
		/// Is the class const in this context ?
		virtual bool isConst() const = 0;
		
		/// Copy this context
		virtual ClassContext* copy() const = 0;
		
		/// Resolve the "this" expression in context of the class we're in
		virtual Type const* getThisPointer(const Location* loc) const = 0;
		
	private:
		CompileInfo& fCompileInfo;
	};
}
#endif
