/*!
	\file classtype.h
	\brief Contains the ClassType class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-12-16
// Last time modified:	2008-10-19

#ifndef DEF_CLASSTYPE_H
 #define DEF_CLASSTYPE_H
 
 #include "declaredtype.h"
 #include "qualifiedname.h"

namespace HYPER_Compiler {
	
	/// A class type
	class ClassType : public DeclaredType {
	public:
		ClassType(Class* c, bool isConst, const Location* loc);
		ClassType(CompileInfo& info, QualifiedName const& t, bool isConst,
			const Location* loc);
		
		bool isPointer() const;
		bool isDoublePointer() const;
		bool isArray() const;
		bool isClass() const;
		
		bool isConst() const;
		void makeConst();
		
		HYPER_Common::UTF8String toString() const;
		
	protected:
		bool doInterfaceResolve(Trace const& trace);
		
	private:
		ClassType(ClassType const& c);
		
		bool resolveClassName();
		
		QualifiedName fTypeName;
		bool fConst;
		Class* fClassRef;
	};
}
#endif
