/*!
	\file compileinfo.cxx
	\brief Implements the CompileInfo class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-05-12
// Last time modified:	2008-06-26

#include "compileinfo.h"

#include "base/asserts.h"
#include "base/location.h"

#include "namespace.h"

namespace HYPER_Compiler {
	
	CompileInfo::CompileInfo()
	 : fRootScope(new RootScope()), fRepos(0)
	{
		//
	}
	
	CompileInfo::~CompileInfo() {
		delete fRootScope;
	}
	
	void CompileInfo::setClassRepository(ClassRepository* repo) {
		fRepos = repo;
	}
	
	ClassRepository& CompileInfo::getClassRepository() const {
		hyp_assert(fRepos != 0);
		return *fRepos;
	}
	
	CompileInfo::RootScope& CompileInfo::getRootScope() const {
		return *fRootScope;
	}
	
	Namespace* CompileInfo::getSystemNamespace() {
		static Namespace* systemNS =
			Namespace::create(
				QualifiedName(UTF8STR("system")), Location::getBuiltIn(), *this
			);
		
		return systemNS;
	}
	
	
	// *** CompileInfo::RootScope *** //
	
	bool CompileInfo::RootScope::registerSymbol(NamedDeclaredItem* symbol) {
		return Scope::registerSymbol(symbol);
	}
}
