/*!
	\file compileinfo.h
	\brief Contains the CompileInfo class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-05-12
// Last time modified:	2008-06-26

#ifndef DEF_COMPILEINFO_H
 #define DEF_COMPILEINFO_H
 
 #include "base/basiccompileinfo.h"
 
 #include "common/paths_fwdecl.h"
 
 #include "scopes.h"
 
namespace HYPER_Compiler {
	
	class ClassRepository;
	class Namespace;
	class Scope;
	
	/// Provides info used everywhere in the compiler
	class CompileInfo : public BasicCompileInfo {
	public:
		CompileInfo();
		~CompileInfo();
		
		void setClassRepository(ClassRepository* repo);
		ClassRepository& getClassRepository() const;
		
		/// The root scope of the symbol table
		class RootScope : public Scope {
		public:
			RootScope() {}
			bool registerSymbol(NamedDeclaredItem* symbol);
		};
		
		RootScope& getRootScope() const;
		
		Namespace* getSystemNamespace();
		
	private:
		RootScope* fRootScope;
		ClassRepository* fRepos;
	};
}
#endif
