/*!
	\file constantvalue.cxx
	\brief Implements the ConstantValue class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-02-01
// Last time modified:	2008-09-21

#include "constantvalue.h"

#include "base/asserts.h"

namespace HYPER_Compiler {
	
	ConstantValue::ConstantValue(nat_t x)
	 : fType(kUnsignedInt), fUI(x)
	{
		// no action
	}
	
	ConstantValue::ConstantValue(int_t x)
	 : fType(kSignedInt), fSI(x)
	{
		// no action
	}
	
	ConstantValue::ConstantValue(bool b)
	 : fType(kBool), fB(b)
	{
		// no action
	}
	
	ConstantValue::ConstantType
	ConstantValue::getConstantType() const {
		return fType;
	}
	
	ConstantValue::nat_t ConstantValue::getUnsigned() const {
		hyp_assert(fType == kUnsignedInt);
		return fUI;
	}
	
	ConstantValue::int_t ConstantValue::getSigned() const {
		hyp_assert(fType == kSignedInt);
		return fSI;
	}
	
	bool ConstantValue::getBool() const {
		hyp_assert(fType == kBool);
		return fB;
	}
	
	bool ConstantValue::operator==(ConstantValue const& c) const {
		if (fType != c.fType) {
			return false;
		}
		
		switch (fType) {
		case kSignedInt:
			return fSI == c.fSI;
		case kUnsignedInt:
			return fUI == c.fUI;
		case kBool:
			return fB == c.fB;
		default:
			hyp_assert_unreachable();
		}
	}
}
