/*!
	\file constantvalue.h
	\brief Contains the ConstantValue class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-02-01
// Last time modified:	2007-09-21

#ifndef DEF_CONSTANTVALUE_H
 #define DEF_CONSTANTVALUE_H

namespace HYPER_Compiler {
	
	class ConstantValue {
	public:
		enum ConstantType {
			kSignedInt,
			kUnsignedInt,
			kBool
		};
		
		typedef unsigned long long nat_t;
		typedef signed long long int_t;
		
		ConstantValue(nat_t x);
		ConstantValue(int_t x);
		ConstantValue(bool b);
		
		ConstantType getConstantType() const;
		nat_t getUnsigned() const;
		int_t getSigned() const;
		bool getBool() const;
		
		bool operator==(ConstantValue const& c) const;
		
		bool operator!=(ConstantValue const& c) const {
			return !operator==(c);
		}
		
	private:
		ConstantType fType;
		unsigned long long fUI;
		signed long long fSI;
		bool fB;
	};
}
#endif
