/*!
	\file constructor.cxx
	\brief Implementation of the Constructor class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-10
// Last time modified:	2008-10-16

#include "constructor.h"

#include "parameter.h"
#include "type.h"

namespace HYPER_Compiler {
	
	Constructor::Constructor(CompileInfo& info)
	 : fCompileInfo(info)
	{
		// nothing here
	}
	
	Constructor::~Constructor() {
		// no action
	}
	
	CompileInfo& Constructor::getCompileInfo() const {
		return fCompileInfo;
	}
	
	HYPER_Common::UTF8String Constructor::getDescription() const {
		return UTF8STR("constructor `%1'").compose(headerToString());
	}
	
	Class const* Constructor::getOwnerClass() const {
		return getClass();
	}
	
	ClassMemberAccess Constructor::getAccessInOwner() const {
		return getAccess();
	}
	
	HYPER_Common::UTF8String Constructor::headerToString() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		const std::vector<Parameter const*>& params = *getParameterList();
		
		if (isImplicit() && params.size() == 1) {
			sb += UTF8STR("implicit ");
		}
		
		sb += UTF8STR("procedure new(");
		
		for (unsigned int i = 0; i < params.size(); ++i) {
			if (i > 0) {
				sb += UTF8STR(", ");
			}
			
			sb += params[i]->toString();
		}
		
		sb += UTF8STR(")");
		
		return sb.toString();
	}
	
	HYPER_Common::UTF8String Constructor::getOverloadHash() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		sb += UTF8STR("@"); // indicates constructor
		
		const std::vector<Parameter const*>& params = *getParameterList();
		for (unsigned int i = 0; i < params.size(); ++i) {
			sb += params[i]->getType()->getOverloadHash();
			sb += UTF8STR("$"); // separator & terminator character
		}
		
		return sb.toString();
	}
}
