/*!
	\file constructorcontext.cxx
	\brief Implements the ConstructorContext class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-10-28
// Last time modified:	2008-12-28

#include "constructorcontext.h"

#include "base/easydiagnostics.h"

#include "compileinfo.h"
#include "expression.h"
#include "implementedconstructor.h"

namespace HYPER_Compiler {
	
	ConstructorContext::ConstructorContext(const ImplementedConstructor* ct)
	 : StatementContext(ct->getCompileInfo()), fConstructor(ct)
	{
		// no action
	}
	
	Class const* ConstructorContext::getClass() const {
		return fConstructor->getClass();
	}
	
	bool ConstructorContext::isStatic() const {
		return false;
	}
	
	bool ConstructorContext::isConst() const {
		return false;
	}
	
	const ImplementedConstructor* ConstructorContext::getConstructor() const {
		return fConstructor;
	}
	
	ConstructorContext* ConstructorContext::copy() const {
		return new ConstructorContext(*this);
	}
	
	Type const*
	ConstructorContext::getThisPointer(const Location* loc) const
	{
		getCompileInfo().diag().error(
			UTF8STR("`this' cannot be used in a constructor"), loc
		);
		return 0;
	}
	
	bool
	ConstructorContext::checkReturn(Location const* loc,
		Expression const* expr) const
	{
		if (expr) { // return <expr>
			getCompileInfo().diag().error(
				UTF8STR("cannot return a result from a constructor"),
				expr->getLocation()
			);
			
			return false;
		}
		
		return true;
	}
}
