/*!
	\file crementstatement.h
	\brief Contains the CrementStatement class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-11-02
// Last time modified:	2009-01-25

#ifndef DEF_CREMENTSTATEMENT_H
 #define DEF_CREMENTSTATEMENT_H
 
 #include "statement.h"
 
 #include "operators.h"

namespace HYPER_Compiler {
	
	class Expression;
	
	/// Abstract increment/decrement statement representation
	class CrementStatement : public Statement {
	public:
		CrementStatement(Expression* lValue,
			Operators::CrementOperator op, const Location* loc,
			CompileInfo& info);
		~CrementStatement();
		
		HYPER_Common::UTF8String toString() const;
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		Expression* getLValue() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		
		void putInSymbolTable(ScopeLocation const& loc);
		
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
	private:
		Expression* fLValue;
		Operators::CrementOperator fOpSym;
	};
}
#endif
