/**
	\file declareditem.h
	\brief Contains the DeclaredItem class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-02-22
// Date modified:	2008-02-15

#ifndef DEF_DECLAREDITEM_H
 #define DEF_DECLAREDITEM_H
 
 #include "important_fwdecl.h"
 #include "membertypes.h"

 #include "common/utf8string.h"

namespace HYPER_Compiler {
	
	/// brief Represents something that is declared or defined
	class DeclaredItem {
	public:
		virtual ~DeclaredItem();
		
		/// Get a human description for the DeclaredItem, not just its
		/// name or type.
		virtual HYPER_Common::UTF8String getDescription() const = 0;
		
		virtual Class const* getOwnerClass() const = 0;
		virtual ClassMemberAccess getAccessInOwner() const = 0;
		
		virtual const Location* getLocation() const = 0;
	};
}
#endif
