/*!
	\file declaredparameter.h
	\brief Contains the DeclaredParameter class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-01
// Last time modified:	2008-10-19

#ifndef DEF_DECLAREDPARAMETER_H
 #define DEF_DECLAREDPARAMETER_H
 
 #include "nameddeclareditem.h"
 #include "parameter.h"
 #include "resolveable.h"

namespace HYPER_Compiler {
	
	class DeclaredType;
	
	/// Used in the AST to represent a formal parameter of a procedure,
	/// constructor or operator
	class DeclaredParameter : public NamedDeclaredItem, public Resolveable {
	public:
		DeclaredParameter(CompileInfo& info, HYPER_Common::UTF8String name,
			DeclaredType* type, const Location* loc,
			SimpleParameter::Passing passing);
		
		HYPER_Common::UTF8String getName() const;
		
		const Location* getLocation() const;
		
		HYPER_Common::UTF8String toString() const;
		
		HYPER_Common::UTF8String getDescription() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
		/// Let the parameter know where it is in the symbol table, i.e. what
		/// its parent scope is.
		void putInSymbolTable(ScopeLocation const& loc);
		
		/// Get the CompileInfo
		CompileInfo& getCompileInfo() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
		Parameter const* getResolvedParameter() const;
		
	private:
		CompileInfo& fCompileInfo;
		HYPER_Common::UTF8String fName;
		DeclaredType* fType;
		SimpleParameter::Passing fPassing;
		const Location* fLoc;
		Parameter const* fResolvedParam;
	};
}
#endif
