/*!
	\file declaredtype.cxx
	\brief Implements the DeclaredType class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-01
// Last time modified:	2008-10-15

#include "declaredtype.h"

#include "base/asserts.h"
#include "base/debug.h"
#include "base/location.h"

#include "class.h"
#include "classrepository.h"
#include "expression.h"

namespace HYPER_Compiler {
	
	// constructor
	DeclaredType::DeclaredType(CompileInfo& info, Location const* loc)
	 : fCompileInfo(info), fLoc(loc == 0 ? Location::getBuiltIn() : loc),
		fLookupLoc(0), fDirectAllocate(false), fCompileTimeSize(false),
		fResolvedType(0)
	{
		// no action
	}
	
	// copy constructor
	DeclaredType::DeclaredType(DeclaredType const& t)
	 : fCompileInfo(t.fCompileInfo), fLoc(t.fLoc), fLookupLoc(t.fLookupLoc),
		fDirectAllocate(t.fDirectAllocate),
		fCompileTimeSize(t.fCompileTimeSize), fResolvedType(t.fResolvedType)
	{
		// no action
	}
	
	void DeclaredType::setResolvedType(Type const* t) {
		hyp_assert(t != 0);
		//~ debug.line(UTF8STR("DeclaredType: resolved type %1").compose(unsigned(t)));
		fResolvedType = t;
	}
	
	Type const* DeclaredType::getResolvedType() const {
		return fResolvedType;
	}
	
	void
	DeclaredType::setRestrictions(bool directAllocate, bool compileTimeSize)
	{
		hyp_assert(resolveNotYetStarted());
		fDirectAllocate = directAllocate;
		fCompileTimeSize = compileTimeSize;
	}
	
	bool DeclaredType::mustBeDirectAllocatable() const {
		return fDirectAllocate;
	}
	
	bool DeclaredType::mustHaveCompileTimeSize() const {
		return fCompileTimeSize;
	}
	
	const Location* DeclaredType::getLocation() const {
		return fLoc;
	}
	
	void DeclaredType::dump() const {
		debug.line(this->toString());
	}
	
	DeclaredType::~DeclaredType() {
		// no action
	}
	
	void DeclaredType::putInSymbolTable(LookupLocation const* loc) {
		fLookupLoc = loc;
	}
	
	LookupLocation const* DeclaredType::getLookupLocation() const {
		return fLookupLoc;
	}
	
	void DeclaredType::operator=(const DeclaredType& t) {
		hyp_assert_unreachable();
	}
}
