/*!
	\file doubleclass.h
	\author Kevin André
	\brief Contains the Double class
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-04-19
// Last time modified:	2008-02-28

#ifndef DEF_DOUBLECLASS_H
 #define DEF_DOUBLECLASS_H
 
 #include "assignmentoperator.h"
 #include "binaryoperator.h"
 #include "builtinclass.h"
 #include "builtinconstructor.h"
 #include "builtintypes.h"
 #include "unaryoperator.h"

namespace HYPER_Compiler {
	
	class BinaryOperator;
	
	/// Represents the built-in `double' class
	class Double : public BuiltInClass {
	public:
		Double(CompileInfo& info);
		~Double();
		
		HYPER_Common::UTF8String getTypeName() const;
		
		void buildSymbolTable();
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
	private:
		class DoubleConstructor;
		class PositiveOperator;
		class NegativeOperator;
		class EqualsOperator;
		class DiffersOperator;
		class LessOperator;
		class LessEqOperator;
		class GreaterOperator;
		class GreaterEqOperator;
		class AddOperator;
		class SubtractOperator;
		class MultiplyOperator;
		class DivideOperator;
		class AssignOp;
		class AddAssignOp;
		class SubtractAssignOp;
		class MultiplyAssignOp;
		class DivideAssignOp;
		
		void createConstructors();
		void createMembers();
		void createOperators();
	};
}
#endif
