/*!
	\file doublepointertype.cxx
	\brief Implements the DoublePointerType class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-08-24
// Last time modified:	2008-10-21

#include "doublepointertype.h"

#include "base/asserts.h"
#include "base/easydiagnostics.h"

#include "compileinfo.h"
#include "conversions.h"
#include "expression.h"
#include "initializationhandler.h"
#include "simpleparameter.h"
#include "type.h"

namespace HYPER_Compiler {
	
	// *** DoublePointerType ***
	
	DoublePointerType::DoublePointerType(PointerType* base, const Location* loc)
	 : DeclaredType(base->getCompileInfo(), loc), fBaseType(base)
	{
		//
	}
	
	DoublePointerType::DoublePointerType(DoublePointerType const& dpt)
	 : DeclaredType(dpt), fBaseType(dpt.fBaseType)
	{
		// copy constructor should not be used
		hyp_assert_unreachable();
	}
	
	bool DoublePointerType::isArray() const {
		return false;
	}
	
	bool DoublePointerType::isPointer() const {
		return true;
	}
	
	bool DoublePointerType::isDoublePointer() const {
		return true;
	}
	
	bool DoublePointerType::isClass() const {
		return false;
	}
	
	bool DoublePointerType::isConst() const {
		return false;
	}
	
	void DoublePointerType::makeConst() {
		// no-op
	}
	
	HYPER_Common::UTF8String DoublePointerType::toString() const {
		return UTF8STR("*") + fBaseType->toString();
	}
	
	void DoublePointerType::putInSymbolTable(LookupLocation const* loc) {
		DeclaredType::putInSymbolTable(loc);
		fBaseType->putInSymbolTable(loc);
	}
	
	bool DoublePointerType::doInterfaceResolve(Trace const& trace) {
		bool ok = fBaseType->resolveInterface(trace);
		
		if (ok) {
			setResolvedType(
				Type::createDoublePointer(fBaseType->getResolvedType())
			);
		}
		
		return ok;
	}
}
