/*!
	\file doublepointertype.h
	\brief Contains the DoublePointerType class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-08-24
// Last time modified:	2008-10-19

#ifndef DEF_DOUBLEPOINTERTYPE_H
 #define DEF_DOUBLEPOINTERTYPE_H
 
 #include "pointertype.h"

namespace HYPER_Compiler {
	
	/// Double pointer type
	class DoublePointerType : public DeclaredType {
	public:
		DoublePointerType(PointerType* base, const Location* loc);
		
		bool isPointer() const;
		bool isDoublePointer() const;
		bool isArray() const;
		bool isClass() const;
		
		bool isConst() const;
		void makeConst();
		
		HYPER_Common::UTF8String toString() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
	protected:
		bool doInterfaceResolve(Trace const& trace);
		
	private:
		DoublePointerType(DoublePointerType const& dpt);
		
		PointerType* fBaseType;
	};
}
#endif
