/*!
	\file expressionparser.h
	\brief Contains the ExpressionParser class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-09-21
// Last time modified:	2007-11-10

#ifndef DEF_EXPRESSIONPARSER_H
 #define DEF_EXPRESSIONPARSER_H
 
 #include "parserbase.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class BoolConditionExpression;
	
	/// Parser for expressions
	class ExpressionParser : private ParserBase {
	public:
		ExpressionParser(ParserInfo const& pi, ClassContext const* cCtxt);
		
		/// check if a given token could be the first token of an expression
		static bool canBeStartOfExpression(Token const& t);
		
		BoolConditionExpression* doBoolResultExpression();
		
		Expression* doExpression();
		Expression* doSingleExpression();
		
	private:
		Expression* doConditionalExpression();
		Expression* doOrExpression();
		Expression* doAndExpression();
		Expression* doConsequenceExpression();
		Expression* doCompareExpression();
		Expression* doAddExpression();
		Expression* doMulExpression();
		Expression* doUnaryExpression();
		Expression* doPostfixExpression();
		
		std::vector<Expression*> doArgumentsExpression();
		
		Expression*
			doArraySelectionExpressions(Expression* arrayExpression);
		
		Expression* doNumberLiteral();
		Expression* doStringLiteral();
		Expression* doCharLiteral();
		
		ClassContext const* fCtxt;
	};
}
#endif
