/*!
	\file field.h
	\brief Field representation.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-01-31
// Last time modified:	2008-10-15

#ifndef DEF_FIELD_H
 #define DEF_FIELD_H
 
 #include "classcontext.h"
 #include "compileinfo.h"
 #include "important_fwdecl.h"
 #include "membertypes.h"
 #include "nameddeclareditem.h"
 #include "resolveable.h"
 #include "syntaxitem.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class DeclaredType;
	class FieldContext;
	class FieldDecl;
	class Initializer;
	
	/// Represents a class field
	class Field : public SyntaxItem, public Resolveable {
	public:
		/// Constructor
		Field(Class const* c, bool isStatic, ClassMemberAccess access,
			std::vector<HYPER_Common::UTF8String> const& names,
			std::vector<Location const*> const& locs,
			DeclaredType* t, Initializer* init, const Location* loc,
			FieldContext const* ctxt);
		
		~Field();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		Class const* getClass() const;
		
		DeclaredType const& getType() const;
		
		bool isStatic() const;
		ClassMemberAccess getAccess() const;
		
		const Location* getLocation() const;
		
		void putInSymbolTable(Scope& parent);
		
		CompileInfo& getCompileInfo() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		std::vector<FieldDecl*> const& getDeclarations() const {
			return fDecls;
		}
		
	private:
		Class const* fClass;
		std::vector<FieldDecl*> fDecls;
		DeclaredType* fType;
		Initializer* fInit;
		bool fStatic;
		ClassMemberAccess fAccess;
		const Location* fBeginLoc;
		FieldContext const* fContext;
		CompileInfo& fCompileInfo;
	};
	
	/// Represents a single field
	class FieldDecl : public NamedDeclaredItem {
	public:
		Field const* getField() const;
		
		HYPER_Common::UTF8String getDescription() const;
		HYPER_Common::UTF8String getDeclarationName() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		const Location* getLocation() const;
		
	private:
		FieldDecl(Field* f, HYPER_Common::UTF8String name, Location const* loc);
		friend class Field;
		
		Field* fF;
		HYPER_Common::UTF8String fName;
		Location const* fLoc;
	};
}
#endif
