/*!
	\file fieldcontext.cxx
	\brief Implements the FieldContext class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-10-28
// Last time modified:	2008-06-26

#include "fieldcontext.h"

#include "base/easydiagnostics.h"

#include "field.h"

namespace HYPER_Compiler {
	
	FieldContext::FieldContext(CompileInfo& info)
	 : ClassContext(info), fField(0)
	{
		// no action
	}
	
	Class const* FieldContext::getClass() const {
		return fField->getClass();
	}
	
	bool FieldContext::isStatic() const {
		return fField->isStatic();
	}
	
	bool FieldContext::isConst() const {
		return false;
	}
	
	void FieldContext::setField(const Field* f) {
		fField = f;
	}
	
	const Field* FieldContext::getField() const {
		return fField;
	}
	
	FieldContext* FieldContext::copy() const {
		return new FieldContext(*this);
	}
	
	Type const*
	FieldContext::getThisPointer(const Location* loc) const
	{
		// TODO / FIXME (?)
		getCompileInfo().diag().error(
			UTF8STR("`this' cannot be used in a field"), loc
		);
		return 0;
	}
}
