/*!
	\file identifierexpression.h
	\brief Represents an identifier expression
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-04
// Last time modified:	2008-06-18

#ifndef DEF_IDENTIFIEREXPRESSION_H
 #define DEF_IDENTIFIEREXPRESSION_H
 
 #include "lexer/keywords.h"
 
 #include "nameditemexpression.h"

namespace HYPER_Compiler {
	
	class IdentifierExpression : public NamedItemExpression {
	public:
		IdentifierExpression(HYPER_Common::UTF8String identifier,
			const Location* loc, ClassContext const* cCtxt);
		
		IdentifierExpression(Keywords::Keyword identifier, const Location* loc,
			ClassContext const* cCtxt);
		
		~IdentifierExpression();
		
		HYPER_Common::UTF8String toString() const;
		
		bool doFullResolve(Trace const& trace);
		
	private:
		HYPER_Common::UTF8String fIdent;
		Keywords::Keyword fKeyword;
	};
}
#endif
