/*!
	\file implementedconstructor.h
	\brief Contains the ImplementedConstructor class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-06
// Last time modified:	2008-11-08

#ifndef DEF_IMPLEMENTEDCONSTRUCTOR_H
 #define DEF_IMPLEMENTEDCONSTRUCTOR_H
 
 #include "constructor.h"
 #include "resolveable.h"
 #include "scopes.h"
 #include "syntaxitem.h"
 #include "statementblock.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class CompileInfo;
	class Class;
	class DeclaredParameter;
	
	/// Represents a constructor
	class ImplementedConstructor :
		public SyntaxItem, public Constructor, public Resolveable,
		public OrderedScope
	{
	public:
		ImplementedConstructor(Class* c, ClassMemberAccess access,
			bool isImplicit, const Location* begin);
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		Class* getClass() const;
		
		HYPER_Common::UTF8String headerToString() const;
		
		ClassMemberAccess getAccess() const;
		
		bool isImplicit() const;
		void makeImplicit();
		
		StatementBlock& getCodeBlock();
		
		unsigned getParameterCount() const;
		std::vector<DeclaredParameter*>& getDeclaredParameterList();
		const std::vector<Parameter const*>* getParameterList() const;
		
		const Location* getLocation() const;
		
		void markAsDuplicate();
		bool isDuplicate() const;
		
		CompileInfo& getCompileInfo() const {
			return Constructor::getCompileInfo();
		}
		
		void putInSymbolTable(Scope& parent);
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
	private:
		Class* fClass;
		ClassMemberAccess fAccess;
		bool fImplicit;
		std::vector<DeclaredParameter*> fParams;
		std::vector<Parameter const*>* fResolvedParams;
		StatementBlock fCode;
		const Location* fBeginPos;
		bool fDup;
	};
}
#endif
