/*!
	\file implementedenum.h
	\author Kevin André
	\brief Contains the ImplementedEnum class
	
	Copyright (C) 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2009-08-30
// Last time modified:	2009-09-06

#ifndef DEF_IMPLEMENTEDENUM_H
 #define DEF_IMPLEMENTEDENUM_H
 
 #include "enum.h"
 #include "syntaxitem.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	/// Represents an enum defined by the user
	class ImplementedEnum : public Enum, public SyntaxItem {
	public:
		ImplementedEnum(CompileInfo& info, HYPER_Common::UTF8String caption,
			ClassMemberAccess access, const Location* begin, Class* parent);
		
		~ImplementedEnum();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		HYPER_Common::UTF8String getCaption() const;
		HYPER_Common::UTF8String getTypeName() const;
		HYPER_Common::UTF8String getDeclarationName() const;
		
		EnumValue* addValue(HYPER_Common::UTF8String name, Location const* loc);
		
		const Location* getLocation() const;
		
	protected:
		bool doInterfaceResolve(Trace const& trace);
		//~ bool doFullResolve(Trace const& trace);
		
	private:
		HYPER_Common::UTF8String fCaption;
		std::vector<EnumValue*> fValues;
		QualifiedName fFullName;
		const Location* fBeginPos;
		Class* fParent;
	};
}
#endif
