/*!
	\file implementedprocedure.h
	\brief Contains the ImplementedProcedure class.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-06-20
// Last time modified:	2008-12-29

#ifndef DEF_IMPLEMENTEDPROCEDURE_H
 #define DEF_IMPLEMENTEDPROCEDURE_H
 
 #include "procedure.h"
 #include "resolveable.h"
 #include "scopes.h"
 #include "syntaxitem.h"
 #include "statementblock.h"

namespace HYPER_Compiler {
	
	class DeclaredParameter;
	class DeclaredType;
	
	/// Represents a user-defined procedure
	class ImplementedProcedure :
		public SyntaxItem, public Procedure, public Resolveable,
		public OrderedScope
	{
	public:
		ImplementedProcedure(HYPER_Common::UTF8String caption,
			Class* c,
			ClassMemberAccess access,
			AbstractSealedStatic ass, bool isConst,
			const Location* begin);
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		Class* getClass() const;
		
		HYPER_Common::UTF8String getCaption() const;
		QualifiedName getFullName() const;
		
		HYPER_Common::UTF8String headerToString() const;
		
		ClassMemberAccess getAccess() const;
		
		void setASS(AbstractSealedStatic ass);
		AbstractSealedStatic getASS() const;
		
		bool isConst() const;
		
		void setReturnType(DeclaredType* t);
		bool hasReturnType() const;
		Type const* getReturnType() const;
		
		StatementBlock& getCodeBlock();
		
		unsigned getParameterCount() const;
		std::vector<DeclaredParameter*>& getDeclaredParameterList();
		const std::vector<Parameter const*>* getParameterList() const;
		
		const Location* getLocation() const;
		
		void setEndLocation(const Location* loc);
		const Location* getEndLocation() const;
		
		void markAsDuplicate();
		bool isDuplicate() const;
		
		void putInSymbolTable(Scope& parent);
		
		CompileInfo& getCompileInfo() const {
			return Procedure::getCompileInfo();
		}
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
		bool mustReturnResultInAddressByParameter() const;
		
	private:
		HYPER_Common::UTF8String fCaption;
		Class* fClass;
		ClassMemberAccess fAccess;
		AbstractSealedStatic fASS;
		bool fConst;
		std::vector<DeclaredParameter*> fParams;
		std::vector<Parameter const*>* fResolvedParams;
		DeclaredType* fReturnType;
		Type const* fResolvedReturnType;
		StatementBlock fCode;
		QualifiedName fFullName;
		const Location* fBeginPos;
		const Location* fEndPos;
		bool fDup;
	};
}
#endif
