/*!
	\file initializer.h
	\brief Contains the Initializer class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-08-17
// Last time modified:	2008-10-15

#ifndef DEF_INITIALIZER_H
 #define DEF_INITIALIZER_H
 
 #include "important_fwdecl.h"
 #include "resolveable.h"
 #include "syntaxitem.h"

namespace HYPER_Compiler {
	
	class DeclaredType;
	class InitializationHandler;
	
	/// Represents an initializer for a variable or field
	class Initializer : public SyntaxItem, public Resolveable {
	public:
		/// Constructor for a default value
		Initializer(DeclaredType const* t, ClassContext const* context,
			Location const* useLoc);
		
		/// Constructor from an expression
		Initializer(DeclaredType const* t, Expression* e,
			ClassContext const* context, Location const* useLoc);
		
		/// Destructor
		~Initializer();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		/// Is the initializer a default value?
		bool isDefault() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
		CompileInfo& getCompileInfo() const;
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const { return fUseLoc; }
		
		bool doInterfaceResolve(Trace const& trace) { return true; }
		bool doFullResolve(Trace const& trace);
		
	private:
		DeclaredType const* fType;
		Expression* fExpr;
		ClassContext const* fContext;
		Location const* fUseLoc;
		InitializationHandler* fInitHandler;
	};
}
#endif
