/*!
	\file instanceprovider.cxx
	\brief Implements the InstanceProvider class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-03-16
// Last time modified:	2009-08-22

#include "instanceprovider.h"

#include "base/asserts.h"

#include "classcontext.h"
#include "expression.h"
#include "type.h"

namespace HYPER_Compiler {
	
	InstanceProvider::InstanceProvider(ClassContext const& ctxt)
	 : fCtxt(&ctxt), fClass(ctxt.getClass()), fExpr(0)
	{
		if (fCtxt->isStatic()) {
			fWhat = kNoInstance;
		}
		else {
			fWhat = fCtxt->isConst() ? kConstInstance : kWritableInstance;
		}
	}
	
	InstanceProvider::InstanceProvider(Class const* c)
	 : fCtxt(0), fClass(c), fExpr(0), fWhat(kNoInstance)
	{
		// no action
	}
	
	InstanceProvider::InstanceProvider(Expression const* instance)
	 : fCtxt(0), fExpr(instance)
	{
		Type const* instanceType =
			instance->getResult().getType()->getDereferenced();
		
		fClass = instanceType->getClass();
		fWhat = instanceType->isConst() ? kConstInstance : kWritableInstance;
	}
	
	Class const* InstanceProvider::getClass() const {
		return fClass;
	}
	
	InstanceType InstanceProvider::whatInstance() const {
		return fWhat;
	}
}
