/*!
	\file instanceprovider.h
	\brief Contains the InstanceProvider class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-03-16
// Last time modified:	2009-08-22

#ifndef DEF_INSTANCEPROVIDER_H
 #define DEF_INSTANCEPROVIDER_H

namespace HYPER_Compiler {
	
	class Class;
	class ClassContext;
	class Expression;
	
	enum InstanceType {
		kNoInstance = 0,
		kConstInstance,
		kWritableInstance
	};
	
	/// Provides a class instance ("this" pointer) in different circumstances
	class InstanceProvider {
	public:
		/// The class context provides an instance if possible (not static).
		explicit InstanceProvider(ClassContext const& ctxt);
		
		/// No instance, just the class.
		explicit InstanceProvider(Class const* c);
		
		/// The result of a given expression is the instance.
		explicit InstanceProvider(Expression const* instance);
		
		/// Get the class of this instance
		Class const* getClass() const;
		
		/// See if and what instance we have
		InstanceType whatInstance() const;
		
	private:
		ClassContext const* fCtxt;
		Class const* fClass;
		Expression const* fExpr;
		InstanceType fWhat;
	};
}
#endif
