/*!
	\file int16class.cxx
	\author Kevin André
	\brief Implements the Int16 class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-10-07
// Date modified:	2009-02-08

#include "int16class.h"

#include "base/asserts.h"

#include "builtintypes.h"

namespace HYPER_Compiler {
	
	Int16::Int16(CompileInfo& info)
	 : BuiltInIntegralClass(info, UTF8STR("int16"), k16Bit, true)
	{
		// no action
	}
	
	Int16::~Int16() {
		// no action
	}
	
	void Int16::createConstructors() {
		// create default & copy constructor
		BuiltInIntegralClass::createConstructors();
		
		// create conversion constructors
		addConversionConstructor(
			BuiltInTypes::createByte(getCompileInfo()), false
		);
	}
	
	Type const* Int16::getClassType() const {
		return BuiltInTypes::createInt16(getCompileInfo());
	}
	
	Type const* Int16::getAbsResultType() const {
		return BuiltInTypes::createNat16(getCompileInfo());
	}
	
	IntegerSize Int16::getNegateResultSize() const {
		return getSize(); // same size as 'int16' itself
	}
	
	Type const* Int16::getNegateResultType() const {
		return getType();
	}
	
	Type const* Int16::getTruncateToSignedResultType() const {
		hyp_assert_unreachable();
	}
}
