/*!
	\file int64class.cxx
	\author Kevin André
	\brief Implements the Int64 class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-10-08
// Date modified:	2009-02-08

#include "int64class.h"

#include "base/asserts.h"

#include "builtintypes.h"

namespace HYPER_Compiler {
	
	Int64::Int64(CompileInfo& info)
	 : BuiltInIntegralClass(info, UTF8STR("int64"), k64Bit, true)
	{
		// no action
	}
	
	Int64::~Int64() {
		// no action
	}
	
	void Int64::createConstructors() {
		// create default & copy constructor
		BuiltInIntegralClass::createConstructors();
		
		// create conversion constructors
		addConversionConstructor(
			BuiltInTypes::createByte(getCompileInfo()), false
		);
		addConversionConstructor(
			BuiltInTypes::createInt16(getCompileInfo()), true
		);
		addConversionConstructor(
			BuiltInTypes::createInt32(getCompileInfo()), true
		);
		addConversionConstructor(
			BuiltInTypes::createNat16(getCompileInfo()), false
		);
		addConversionConstructor(
			BuiltInTypes::createNat32(getCompileInfo()), false
		);
	}
	
	Type const* Int64::getClassType() const {
		return BuiltInTypes::createInt64(getCompileInfo());
	}
	
	Type const* Int64::getAbsResultType() const {
		return BuiltInTypes::createNat64(getCompileInfo());
	}
	
	IntegerSize Int64::getNegateResultSize() const {
		return getSize(); // same size as 'int64' itself
	}
	
	Type const* Int64::getNegateResultType() const {
		return getType();
	}
	
	Type const* Int64::getTruncateToSignedResultType() const {
		hyp_assert_unreachable();
	}
}
