/*!
	\file iteratestatement.h
	\brief Abstract iterate-statement representation.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-10
// Last time modified:	2009-03-01

#ifndef DEF_ITERATESTATEMENT_H
 #define DEF_ITERATESTATEMENT_H
 
 #include "nameddeclareditem.h"
 #include "scopes.h"
 #include "statement.h"
 
namespace HYPER_Compiler {
	
	class DeclaredType;
	class IterateStatement;
	class StatementBlock;
	
	/// Represents a variable declared in the 'iterate' statement
	class IterateVar : public NamedDeclaredItem {
	public:
		IterateVar(HYPER_Common::UTF8String name, const Location* loc);
		
		void setIterate(IterateStatement* it);
		
		HYPER_Common::UTF8String getName() const;
		Type const* getResolvedType() const;
		const Location* getLocation() const;
		IterateStatement* getIterate() const;
		
		HYPER_Common::UTF8String getDescription() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
	private:
		friend class IterateStatement;
		
		HYPER_Common::UTF8String fName;
		Type const* fType;
		const Location* fLoc;
		IterateStatement* fIt;
	};
	
	/// Represents the 'iterate' statement
	class IterateStatement : public Statement, public OrderedScope {
	public:
		
		~IterateStatement();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		static IterateStatement*
		create(HYPER_Common::UTF8String var, const Location* varLoc,
			Expression* begin, Expression* end, const Location* loc,
			StatementContext const* context, CompileInfo& info);
		
		void setBlock(StatementBlock* s);
		
		HYPER_Common::UTF8String getResolveDescription() const;
		
		void putInSymbolTable(ScopeLocation const& loc);
		
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
	private:
		IterateStatement(IterateVar* var, Expression* begin, Expression* end,
			const Location* pos, StatementContext const* context,
			CompileInfo& info);
		
		bool checkVarInit();
		bool checkCrementStep();
		bool checkExprStep();
		
		IterateVar* fVarDecl;
		Expression* fBeginExpr;
		Expression* fEndExpr;
		StatementBlock* fBlock;
		StatementContext const* fContext;
	};
}
#endif
