/*!
	\file literalexpression.h
	\brief Represents a literal expression
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-04
// Last time modified:	2008-03-01

#ifndef DEF_LITERALEXPRESSION_H
 #define DEF_LITERALEXPRESSION_H
 
 #include "expression.h"

namespace HYPER_Compiler {
	
	class LiteralExpression : public Expression {
	public:
		static LiteralExpression* createFromUnsignedIntNumber(
			unsigned long long i, const Location* loc,
			ClassContext const* cCtxt);
		static LiteralExpression* createFromFloatNumber(long double f,
			const Location* loc, ClassContext const* cCtxt);
		static LiteralExpression* createFromChar(HYPER_Common::CodePoint c,
			const Location* loc, ClassContext const* cCtxt);
		static LiteralExpression* createFromString(HYPER_Common::UTF8String s,
			const Location* loc, ClassContext const* cCtxt);
		static LiteralExpression* createFromBool(bool isTrue,
			const Location* loc, ClassContext const* cCtxt);
		static LiteralExpression* createNull(const Location* loc,
			ClassContext const* cCtxt);
		static LiteralExpression* createThis(const Location* loc,
			ClassContext const* cCtxt);
		
		~LiteralExpression();
		
		HYPER_Common::UTF8String toString() const;
		
		bool doFullResolve(Trace const& trace);
		
		TriBool isClassLValue() const;
		TriBool isPointerLValue() const;
		TriBool isCall() const;
		TriBool hasResult() const;
		TriBool resultAtCompileTime() const;
		bool isNullLiteral() const;
		
	private:
		enum LiteralType {
			kUnsignedIntNumber = 1,
			kFloatNumber,
			kChar,
			kString,
			kBool,
			kNull,
			kThis
		};
		
		LiteralExpression(LiteralType t,
			unsigned long long ui,
			long double f, HYPER_Common::CodePoint c,
			HYPER_Common::UTF8String s, bool b, const Location* loc,
			ClassContext const* cCtxt);
		
		ClassContext const* fCC;
		LiteralType fLiteralType;
		unsigned long long fUInt; // 64 bit unsigned number
		long double fFloat;
		HYPER_Common::CodePoint fChar;
		HYPER_Common::UTF8String fString;
		bool fBool;
	};
}
#endif
