/*!
	\file loopjumpstatement.h
	\brief Contains the LoopJumpStatement Class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-15
// Last time modified:	2008-03-02

#ifndef DEF_LOOPJUMPSTATEMENT_H
 #define DEF_LOOPJUMPSTATEMENT_H
 
 #include "statement.h"

namespace HYPER_Compiler {
	
	class StatementContext;
	
	enum LoopJumpDestination {
		kUnspecifiedJump = 0,
		kLoopJump,
		kIterateJump
	};
	
	enum LoopJumpType {
		kBreak = 1,
		kContinue
	};
	
	/// Represents jump-out-of-a-loop statements
	class LoopJumpStatement : public Statement {
	public:
		LoopJumpStatement(LoopJumpDestination dest, LoopJumpType t,
			StatementContext const* context, const Location* loc,
			CompileInfo& info);
		~LoopJumpStatement();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		LoopJumpType getJumpType() const;
		LoopJumpDestination getDestination() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
	private:
		LoopJumpDestination fDest;
		LoopJumpType fJumpType;
		StatementContext const* fContext;
	};
}
#endif
