/*!
	\file megaloopstatement.cxx
	\brief Implements the MegaLoopStatement class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-13
// Last time modified:	2008-04-28

#include "megaloopstatement.h"

#include "expression.h"
#include "statementblock.h"

#include "common/utf8textflowstream.h"

namespace HYPER_Compiler {
	
	MegaLoopStatement::MegaLoopStatement(const Location* pos,
		StatementContext const* context, CompileInfo& info)
	 : Statement(info, pos), fReturnProb(SimpleProbability::never),
		fContext(context)
	{
		// nothing here
	}
	
	MegaLoopStatement::~MegaLoopStatement() {
		// nothing here
	}
	
	HYPER_Common::UTF8String MegaLoopStatement::getSyntaxName() const {
		return UTF8STR("Mega-Loop-Statement");
	}
	
	void MegaLoopStatement::myPrint(HYPER_Common::UTF8TextFlowStream& ind) const {
		ind.line(getSyntaxName());
		
		HYPER_Common::Indent out(ind);
		
		for (unsigned int i = 0; i < fTests.size(); ++i) {
			ind.line(UTF8STR("test: ") + fTests[i].getTest()->toString());
			HYPER_Common::Indent out(ind);
			fTests[i].getBlock()->myPrint(ind);
		}
	}
	
	void MegaLoopStatement::addTest(BoolConditionExpression* test,
		StatementBlock* block)
	{
		fTests.push_back(TestCase(test, block, fContext));
	}
	
	HYPER_Common::UTF8String MegaLoopStatement::getResolveDescription() const {
		return UTF8STR("%1 statement").compose(QUOTED_UTF8STR("loop"));
	}
	
	void MegaLoopStatement::putInSymbolTable(ScopeLocation const& loc) {
		Statement::putInSymbolTable(loc);
		for (unsigned int i = 0; i < fTests.size(); ++i) {
			fTests[i].putInSymbolTable(loc);
		}
	}
	
	bool MegaLoopStatement::doFullResolve(Trace const& trace) {
		bool resolveOK = true;
		
		for (unsigned int i = 0; i < fTests.size(); ++i) {
			resolveOK &= fTests[i].resolveFull(trace);
			fReturnProb &= fTests[i].getBlock()->getReturnProbability();
		}
		
		return resolveOK;
	}
	
	SimpleProbability MegaLoopStatement::getReturnProbability() const {
		return fReturnProb;
	}
}
