/*!
	\file memberselectionexpression.h
	\brief Represents a member selection expression
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-10
// Last time modified:	2008-02-20

#ifndef DEF_MEMBERSELECTIONEXPRESSION_H
 #define DEF_MEMBERSELECTIONEXPRESSION_H
 
 #include "nameditemexpression.h"

namespace HYPER_Compiler {
	
	class MemberSelectionExpression : public NamedItemExpression {
	public:
		MemberSelectionExpression(Expression* expr,
			HYPER_Common::UTF8String member, const Location* memberLoc,
			ClassContext const* cCtxt);
		~MemberSelectionExpression();
		
		HYPER_Common::UTF8String toString() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
		bool doFullResolve(Trace const& trace);
		
	private:
		Expression* fExpr;
		HYPER_Common::UTF8String fMember;
	};
}
#endif
