/*!
	\file membertypes.h
	\brief Contains the types of members.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-01-27
// Last time modified:	2008-02-13

#ifndef DEF_MEMBERTYPES_H
 #define DEF_MEMBERTYPES_H
 
 #include "common/utf8string.h"
 
namespace HYPER_Compiler {
	
	/// Represents a default or 'abstract', or 'sealed' or 'static' qualifier
	enum AbstractSealedStatic {
		kNoASS = 0,
		kAbstract,
		kSealed,
		kStatic
	};
	
	HYPER_Common::UTF8String toString(AbstractSealedStatic ass);
	
	/// Represents the access levels of a class member: public, protected or private
	class ClassMemberAccess {
	public:
		static ClassMemberAccess createPublic();
		static ClassMemberAccess createProtected();
		static ClassMemberAccess createPrivate();
		
		bool isPublic() const;
		bool isProtected() const;
		bool isPrivate() const;
		
		bool operator == (const ClassMemberAccess& a) const;
		
		operator HYPER_Common::UTF8String() const;
		
	private:
		enum Level {
			kPrivate = 1,
			kProtected,
			kPublic
		};
		
		ClassMemberAccess(Level a);
		
		int fA;
	};
	
	/// Represents a simple public/private member access level
	class SimpleMemberAccess {
	public:
		static SimpleMemberAccess createPublic();
		static SimpleMemberAccess createPrivate();
		
		bool isPublic() const;
		bool isPrivate() const;
		
		bool operator == (const SimpleMemberAccess& a) const;
		
		operator HYPER_Common::UTF8String() const;
		operator ClassMemberAccess() const;
		
	private:
		enum Level {
			kPrivate = 1,
			kPublic
		};
		
		SimpleMemberAccess(Level a);
		
		int fA;
	};
}
#endif
