/*!
	\file namespace.h
	\brief Contains the Namespace class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-05-16
// Last time modified:	2008-02-20

#ifndef DEF_NAMESPACE_H
 #define DEF_NAMESPACE_H
 
 #include "important_fwdecl.h"
 #include "nameddeclareditem.h"
 #include "qualifiedname.h"
 #include "scopes.h"

namespace HYPER_Compiler {
	
	/// Represents a name space
	class Namespace : public NamedDeclaredItem, public Scope {
	public:
		HYPER_Common::UTF8String getShortName() const;
		QualifiedName getFullName() const;
		
		const Location* getLocation() const;
		
		static Namespace*
		create(QualifiedName name, Location const* loc, CompileInfo& info);
		
		HYPER_Common::UTF8String getDescription() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
		void registerClass(Class* c);
		
	private:
		Namespace(QualifiedName fullName, Location const* loc,
			CompileInfo& info);
		
		void putInSymbolTable(Scope& parent);
		void registerChild(Namespace* ns);
		
		HYPER_Common::UTF8String fShortName;
		QualifiedName fFullName;
		Location const* fLoc;
		CompileInfo& fCompileInfo;
	};
}
#endif
