/*!
	\file nat16class.cxx
	\author Kevin André
	\brief Implements the Nat16 class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-09-27
// Date modified:	2009-02-08

#include "nat16class.h"

#include "base/asserts.h"

#include "builtintypes.h"

namespace HYPER_Compiler {
	
	Nat16::Nat16(CompileInfo& info)
	 : BuiltInIntegralClass(info, UTF8STR("nat16"), k16Bit, false)
	{
		// no action
	}
	
	Nat16::~Nat16() {
		// no action
	}
	
	void Nat16::createConstructors() {
		// create default & copy constructor
		BuiltInIntegralClass::createConstructors();
		
		// conversion constructor
		addConversionConstructor(
			BuiltInTypes::createByte(getCompileInfo()), false
		);
	}
	
	Type const* Nat16::getAbsResultType() const {
		hyp_assert_unreachable();
	}
	
	IntegerSize Nat16::getNegateResultSize() const {
		return k32Bit;
	}
	
	Type const* Nat16::getNegateResultType() const {
		return BuiltInTypes::createInt32(getCompileInfo());
	}
	
	Type const* Nat16::getTruncateToSignedResultType() const {
		return BuiltInTypes::createInt16(getCompileInfo());
	}
	
	Type const* Nat16::getClassType() const {
		return BuiltInTypes::createNat16(getCompileInfo());
	}
}
