/*!
	\file nat32class.cxx
	\author Kevin André
	\brief Implements the Nat32 class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-10-03
// Date modified:	2009-02-08

#include "nat32class.h"

#include "base/asserts.h"

#include "builtintypes.h"

namespace HYPER_Compiler {
	
	Nat32::Nat32(CompileInfo& info)
	 : BuiltInIntegralClass(info, UTF8STR("nat32"), k32Bit, false)
	{
		// no action
	}
	
	Nat32::~Nat32() {
		// no action
	}
	
	void Nat32::createConstructors() {
		// create default & copy constructor
		BuiltInIntegralClass::createConstructors();
		
		// conversion constructor
		addConversionConstructor(
			BuiltInTypes::createByte(getCompileInfo()), false
		);
		addConversionConstructor(
			BuiltInTypes::createNat16(getCompileInfo()), false
		);
	}
	
	Type const* Nat32::getAbsResultType() const {
		hyp_assert_unreachable();
	}
	
	IntegerSize Nat32::getNegateResultSize() const {
		return k32Bit;
	}
	
	Type const* Nat32::getNegateResultType() const {
		return BuiltInTypes::createInt64(getCompileInfo());
	}
	
	Type const* Nat32::getTruncateToSignedResultType() const {
		return BuiltInTypes::createInt32(getCompileInfo());
	}
	
	Type const* Nat32::getClassType() const {
		return BuiltInTypes::createNat32(getCompileInfo());
	}
}
