/*!
	\file newoperatorexpression.h
	\brief Contains the NewOperatorExpression class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-10
// Last time modified:	2008-10-15

#ifndef DEF_NEWOPERATOREXPRESSION_H
 #define DEF_NEWOPERATOREXPRESSION_H
 
 #include "expression.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class DeclaredType;
	
	/// Represents an operator "new" expression
	class NewOperatorExpression : public Expression {
	public:
		NewOperatorExpression(DeclaredType* type, std::vector<Expression*> args,
			const Location* loc, CompileInfo& info);
		~NewOperatorExpression();
		
		HYPER_Common::UTF8String toString() const;
		
		//~ Type const& getType() const;
		
		void putInSymbolTable(LookupLocation const* loc);
		
		bool doFullResolve(Trace const& trace);
		
		TriBool isClassLValue() const;
		TriBool isPointerLValue() const;
		TriBool isCall() const;
		TriBool hasResult() const;
		TriBool resultAtCompileTime() const;
		
	private:
		DeclaredType* fType;
		std::vector<Expression*> fArgs;
	};
}
#endif
