/*!
	\file objectclass.cxx
	\author Kevin André
	\brief Implements the Object class
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-04-21
// Last time modified:	2008-12-26

#include "objectclass.h"

namespace HYPER_Compiler {
	
	class Object::ObjectConstructor : public BuiltInConstructor {
	public:
		ObjectConstructor(Object& oc);
		
		const std::vector<Parameter const*>* getParameterList() const;
		
	private:
		std::vector<Parameter const*> fParam;
	};
	
	// *** Object ***
	
	Object::Object(CompileInfo& info)
	 : BuiltInClass(info, kAbstract)
	{
		// no action
	}
	
	Object::~Object() {
		// no action
	}
	
	HYPER_Common::UTF8String Object::getTypeName() const {
		return UTF8STR("object");
	}
	
	HYPER_Common::UTF8String Object::getDeclarationName() const {
		return UTF8STR("object");
	}
	
	void Object::buildSymbolTable() {
		createConstructors();
	}
	
	void Object::createConstructors() {
		registerConstructor(new ObjectConstructor(*this), false);
	}
	
	
	// *** ObjectConstructor ***
	
	Object::ObjectConstructor::ObjectConstructor(Object& oc)
	 : BuiltInConstructor(&oc, true)
	{
		// no action -- empty parameter list
	}
	
	const std::vector<Parameter const*>*
	Object::ObjectConstructor::getParameterList() const {
		return &fParam;
	}
}
