/*!
	\file operators.cxx
	\brief Contains implementation for the various operators
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-06-13
// Last time modified:	2008-06-26

#include "operators.h"

#include "base/asserts.h"

namespace HYPER_Compiler {
	
	HYPER_Common::UTF8String Operators::toString(UnaryOperator op) {
		switch (op) {
		case kNot:
			return UTF8STR("!");
		case kPositive:
			return UTF8STR("+");
		case kNegative:
			return UTF8STR("-");
		case kSuccessor:
			return UTF8STR("++");
		case kPredecessor:
			return UTF8STR("--");
		default:
			hyp_assert_unreachable();
		}
	}
	
	bool Operators::isRegularCompare(BinaryOperator op) {
		switch (op) {
		case kEquals:
		case kDiffers:
		case kLess:
		case kLessOrEqual:
		case kGreater:
		case kGreaterOrEqual:
			return true;
		default:
			return false;
		}
	}
	
	HYPER_Common::UTF8String Operators::toString(BinaryOperator op) {
		switch (op) {
		case kOr:
			return UTF8STR("|");
		case kAnd:
			return UTF8STR("&");
		case kConsequence:
			return UTF8STR("->");
		case kEquals:
			return UTF8STR("=");
		case kDiffers:
			return UTF8STR("!=");
		case kLess:
			return UTF8STR("<");
		case kLessOrEqual:
			return UTF8STR("<=");
		case kGreater:
			return UTF8STR(">");
		case kGreaterOrEqual:
			return UTF8STR(">=");
		case kPointerEquals:
			return UTF8STR("=$");
		case kPointerDiffers:
			return UTF8STR("!$");
		case kAdd:
			return UTF8STR("+");
		case kSubtract:
			return UTF8STR("-");
		case kMultiply:
			return UTF8STR("*");
		case kDivide:
			return UTF8STR("/");
		case kModulo:
			return UTF8STR("%");
		default:
			hyp_assert_unreachable();
		}
	}
	
	HYPER_Common::UTF8String Operators::toString(AssignmentOperator op) {
		switch (op) {
		case kSimpleAssign:
			return UTF8STR("=");
		case kAddAssign:
			return UTF8STR("+=");
		case kSubtractAssign:
			return UTF8STR("-=");
		case kMultiplyAssign:
			return UTF8STR("*=");
		case kDivideAssign:
			return UTF8STR("/=");
		case kModuloAssign:
			return UTF8STR("%=");
		case kAndAssign:
			return UTF8STR("&=");
		case kOrAssign:
			return UTF8STR("|=");
		case kPointerAssign:
			return UTF8STR("=$");
		default:
			hyp_assert_unreachable();
		}
	}
	
	HYPER_Common::UTF8String Operators::toString(CrementOperator op) {
		switch (op) {
		case kIncrement:
			return UTF8STR("++");
		case kDecrement:
			return UTF8STR("--");
		default:
			hyp_assert_unreachable();
		}
	}
}
