/*!
	\file operators.h
	\brief Contains the various operators
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-06-13
// Last time modified:	2008-02-28

#ifndef DEF_OPERATORS_H
 #define DEF_OPERATORS_H
 
 #include "common/utf8string.h"

namespace HYPER_Compiler {
	
	/// Contains the various operators of the language
	class Operators {
	public:
		/// Represents an unary expression operator
		enum UnaryOperator {
			kNot = 1,
			kPositive,
			kNegative,
			kSuccessor,
			kPredecessor
		};
		
		static HYPER_Common::UTF8String toString(UnaryOperator op);
		
		/// Represents an binary expression operator
		enum BinaryOperator {
			kOr = 1,
			kAnd,
			kConsequence,
			kEquals,
			kDiffers,
			kLess,
			kLessOrEqual,
			kGreater,
			kGreaterOrEqual,
			kPointerEquals,
			kPointerDiffers,
			kAdd,
			kSubtract,
			kMultiply,
			kDivide,
			kModulo
		};
		
		/// Check if the given operator is a non-pointer comparison.
		/** This is true for the following operators:
			'=', '!=', '<=', '>=', '<', '>'
		*/
		static bool isRegularCompare(BinaryOperator op);
		
		static HYPER_Common::UTF8String toString(BinaryOperator op);
		
		/// Represents an assignment operator
		enum AssignmentOperator {
			kSimpleAssign = 1,
			kAddAssign,
			kSubtractAssign,
			kMultiplyAssign,
			kDivideAssign,
			kModuloAssign,
			kAndAssign,
			kOrAssign,
			kPointerAssign
		};
		
		static HYPER_Common::UTF8String toString(AssignmentOperator op);
		
		/// Represents an increment/decrement operator
		enum CrementOperator {
			kIncrement,
			kDecrement
		};
		
		static HYPER_Common::UTF8String toString(CrementOperator op);
	};
}
#endif
