/*!
	\file overloadable.h
	\brief Contains the Overloadable class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-08
// Last time modified:	2008-02-20

#ifndef DEF_OVERLOADABLE_H
 #define DEF_OVERLOADABLE_H
 
 #include "common/utf8string.h"
 
namespace HYPER_Compiler {
	
	class Location;
	
	/// Represents something that can be overloaded
	class Overloadable {
	public:
		virtual ~Overloadable();
		
		virtual HYPER_Common::UTF8String headerToString() const = 0;
		
		virtual const Location* getLocation() const = 0;
		
		virtual void markAsDuplicate() = 0;
		virtual bool isDuplicate() const = 0;
		
		bool canOverloadWith(const Overloadable* o) const;
		
		virtual HYPER_Common::UTF8String getOverloadHash() const = 0;
	};
}
#endif
