/*!
	\file overloadedassignmentoperator.cxx
	\brief Implements the OverloadedAssignmentOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-12-17
// Last time modified:	2007-06-21

#include "overloadedassignmentoperator.h"

namespace HYPER_Compiler {
	
	OverloadedAssignmentOperator::OverloadedAssignmentOperator(
		AssignmentOperator* firstOp)
	 : fOp(firstOp->getOperator())
	{
		fOps.push_back(firstOp);
	}
	
	OverloadedAssignmentOperator::~OverloadedAssignmentOperator() {
		// no action
	}
	
	void OverloadedAssignmentOperator::addOverload(AssignmentOperator* bo) {
		fOps.push_back(bo);
	}
	
	Operators::AssignmentOperator
	OverloadedAssignmentOperator::getOperator() const
	{
		return fOp;
	}
	
	const std::vector<AssignmentOperator*>&
	OverloadedAssignmentOperator::getOverloads() const
	{
		return fOps;
	}
}
