/*!
	\file overloadedbinaryoperator.cxx
	\brief Implements the OverloadedBinaryOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-26
// Last time modified:	2007-03-31

#include "overloadedbinaryoperator.h"

namespace HYPER_Compiler {
	
	OverloadedBinaryOperator::OverloadedBinaryOperator(
		BinaryOperator* firstOp)
	 : fOp(firstOp->getOperator())
	{
		fOps.push_back(firstOp);
	}
	
	OverloadedBinaryOperator::~OverloadedBinaryOperator() {
		// no action
	}
	
	void OverloadedBinaryOperator::addOverload(BinaryOperator* bo) {
		fOps.push_back(bo);
	}
	
	void OverloadedBinaryOperator::addOverloadList(OverloadedBinaryOperator* ov)
	{
		typedef std::vector<BinaryOperator*>::const_iterator Iterator3;
		
		for (Iterator3 it3 = ov->getOverloads().begin(); it3 != ov->getOverloads().end(); ++it3) {
			addOverload(*it3);
		}
	}
	
	Operators::BinaryOperator
	OverloadedBinaryOperator::getOperator() const {
		return fOp;
	}
	
	const std::vector<BinaryOperator*>&
	OverloadedBinaryOperator::getOverloads() const {
		return fOps;
	}
}
