/*!
	\file overloadedbinaryoperator.h
	\brief Contains the OverloadedBinaryOperator class
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-09-26
// Last time modified:	2007-03-31

#ifndef DEF_OVERLOADEDBINARYOPERATOR_H
 #define DEF_OVERLOADEDBINARYOPERATOR_H
 
 #include "binaryoperator.h"
 
 #include <vector>
 
namespace HYPER_Compiler {
	
	/// Represents an overloaded binary operator
	class OverloadedBinaryOperator {
	public:
		OverloadedBinaryOperator(BinaryOperator* firstOp);
		~OverloadedBinaryOperator();
		
		void addOverload(BinaryOperator* bo);
		void addOverloadList(OverloadedBinaryOperator* ov);
		
		Operators::BinaryOperator getOperator() const;
		
		const std::vector<BinaryOperator*>& getOverloads() const;
		
	private:
		Operators::BinaryOperator fOp;
		std::vector<BinaryOperator*> fOps;
	};
}
#endif
