/*!
	\file overloadedconstructor.cxx
	\brief Implements the OverloadedConstructor class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-12-25
// Last time modified:	2008-04-13

#include "overloadedconstructor.h"

#include "class.h"

namespace HYPER_Compiler {
	
	OverloadedConstructor::OverloadedConstructor(Constructor* c)
	 : OverloadList<Constructor>(c)
	{
		// no action
	}
	
	HYPER_Common::UTF8String OverloadedConstructor::getDescription() const {
		return UTF8STR("overloadeded constructor of %1").compose(
			getOwnerClass()->getDescription()
		);
	}
	
	Class const* OverloadedConstructor::getOwnerClass() const {
		return getOverloadList()[0]->getClass();
	}
	
	ClassMemberAccess OverloadedConstructor::getAccessInOwner() const
	{
		// FIXME: this function shouldn't be used
		return getOverloadList()[0]->getAccess();
	}
	
	Location const* OverloadedConstructor::getLocation() const {
		return getFirstOverloadLocation();
	}
	
}
