/*!
	\file overloadedprocedure.cxx
	\brief Implements the OverloadedProcedure class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-05-29
// Last time modified:	2008-11-08

#include "overloadedprocedure.h"

#include "implementedprocedure.h"

namespace HYPER_Compiler {
	
	OverloadedProcedure::OverloadedProcedure(Procedure* p)
	 : OverloadList<Procedure>(p), fCaption(p->getCaption())
	{
		// no action
	}
	
	Class const* OverloadedProcedure::getClass() const {
		return getOverloadList()[0]->getClass();
	}
	
	QualifiedName OverloadedProcedure::getFullName() const {
		return getOverloadList()[0]->getFullName();
	}
	
	HYPER_Common::UTF8String OverloadedProcedure::getCaption() const {
		return fCaption;
	}
	
	HYPER_Common::UTF8String OverloadedProcedure::getDescription() const {
		return UTF8STR("overloaded procedure %1").compose(
			quote(getFullName().toString())
		);
	}
	
	Class const* OverloadedProcedure::getOwnerClass() const {
		return getOverloadList()[0]->getClass();
	}
	
	ClassMemberAccess OverloadedProcedure::getAccessInOwner() const
	{
		return getOverloadList()[0]->getAccess();
	}
	
	HYPER_Common::UTF8String OverloadedProcedure::getDeclarationName() const {
		return getOverloadList()[0]->getCaption();
	}
	
	Location const* OverloadedProcedure::getLocation() const {
		return getFirstOverloadLocation();
	}
	
	bool
	OverloadedProcedure::resolveInterfaces(
		Resolveable::Trace const& trace) const
	{
		bool ok = true;
		for (unsigned i = 0; i < getOverloadList().size(); ++i) {
			Procedure* p = getOverloadList()[i];
			ImplementedProcedure* ip = dynamic_cast<ImplementedProcedure*>(p);
			if (ip != 0) {
				ok &= ip->resolveInterface(trace);
			}
		}
		
		return ok;
	}
}
