/*!
	\file overloadedprocedure.h
	\brief Contains the OverloadedProcedure class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:	2005-05-29
// Date modified:	2008-11-08

#ifndef DEF_OVERLOADEDPROCEDURE_H
 #define DEF_OVERLOADEDPROCEDURE_H
 
 #include "nameddeclareditem.h"
 #include "procedure.h"
 #include "overloadlist.h"
 #include "resolveable.h"
 
namespace HYPER_Compiler {
	
	/// Represents an overloaded procedure
	class OverloadedProcedure
		: public OverloadList<Procedure>, public NamedDeclaredItem
	{
	public:
		OverloadedProcedure(Procedure* p);
		
		Class const* getClass() const;
		
		HYPER_Common::UTF8String getCaption() const;
		QualifiedName getFullName() const;
		
		typedef Procedure OverloadEntity;
		
		HYPER_Common::UTF8String getDescription() const;
		Class const* getOwnerClass() const;
		
		ClassMemberAccess getAccessInOwner() const;
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
		Location const* getLocation() const;
		
		bool resolveInterfaces(Resolveable::Trace const& trace) const;
		
	private:
		HYPER_Common::UTF8String fCaption;
	};
}
#endif
