/*!
	\file overloadlist.cxx
	\brief Implements the OverloadList class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-06-08
// Last time modified:	2008-10-16

#ifndef DEF_OVERLOADLIST_CXX
 #define DEF_OVERLOADLIST_CXX
 
 #include "overloadlist.h"
 
 #include "base/asserts.h"
 #include "base/easydiagnostics.h"
 
 //#include "debug.h"

namespace HYPER_Compiler {
	
	template <typename T>
	OverloadList<T>::OverloadList(T* p)
	 : fChecked(false), fCheckResult(false)
	{
		hyp_assert(p != 0);
		fList.push_back(p);
	}
	
	template <typename T>
	void OverloadList<T>::addOverload(T* newT) {
		//hyp_assert(newP->getCaption() == fName);
		hyp_assert(!fChecked);
		
		fList.push_back(newT);
	}
	
	template <typename T>
	const Location*
	OverloadList<T>::getFirstOverloadLocation() const {
		return fList[0]->getLocation();
	}
	
	template <typename T>
	const std::vector<T*>&
	OverloadList<T>::getOverloadList() const {
		return fList;
	}
	
	template <typename T>
	bool
	OverloadList<T>::checkOverloads(EasyDiagnostics const& diag)
	{
		if (fChecked) {
			return fCheckResult; // do check only once
		}
		
		fChecked = true;
		
		fCheckResult = true;
		for (unsigned int i = 0; i < fList.size(); ++i) {
			T* curP = fList[i];
			
			if (curP->isDuplicate() || curP->getParameterList() == 0) {
				continue;
			}
			
			for (unsigned int j = i + 1; j < fList.size(); ++j) {
				T* newP = fList[j];
				
				if (newP->isDuplicate() || newP->getParameterList() == 0) {
					continue;
				}
				
				if (!curP->canOverloadWith(newP)) {
					newP->markAsDuplicate();
					
					diag.error(
						newP->getLocation(),
						UTF8STR("cannot overload %1 and"),
						quote(newP->headerToString())
					).addInfo(
						quote(curP->headerToString()),
						curP->getLocation()
					);
					
					fCheckResult = false;
				}
			}
		}
		
		return fCheckResult;
	}
}
#endif
