/*!
	\file parameter.cxx
	\brief Implements the Parameter class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-10-13
// Last time modified:	2008-12-25

#include "parameter.h"

#include "base/asserts.h"

#include "type.h"

namespace HYPER_Compiler {
	
	Parameter::Parameter(Type const* type, Passing passing,
		HYPER_Common::UTF8String name, Location const* loc)
	 : SimpleParameter(type, passing), fName(name)
	{
		// type and location must be specified, of course
		hyp_assert(type != 0);
		hyp_assert(loc != 0);
		
		// "kCopyValue" is not supposed to be used for real parameters
		hyp_assert(passing != SimpleParameter::kCopyValue);
	}
	
	HYPER_Common::UTF8String Parameter::toString() const {
		HYPER_Common::UTF8StringBuilder sb;
	
		// add passing prefix if needed
		if (getPassing() == kInOut) {
			sb += UTF8STR("inout ");
		}
		
		sb += fName;
		sb += UTF8STR(" : ");
		sb += getType()->toString();
		
		return sb.toString();
	}
}
