/*!
	\file parser.h
	\brief Contains the Parser class.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-02-05
// Last time modified:	2009-08-30

#ifndef DEF_PARSER_H
 #define DEF_PARSER_H
 
 #include "base/location.h"
 
 #include "common/filepath.h"
 
 #include "lexer/fatalparseerror.h"
 
 #include "membertypes.h"
 #include "parserbase.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	class ClassRepository;
	class DeclaredParameter;
	class Field;
	class ImplementedClass;
	class ImplementedConstructor;
	class ImplementedEnum;
	class ImplementedProcedure;
	
	/// Parses Hyper source code and generates the AST
	class Parser : private ParserBase {
	public:
		Parser(ParserInfo const& pi);
		~Parser();
		
		/// Does the parsing of a source file
		SourceFile* doTranslationUnit(ClassRepository* repos,
			HYPER_Common::FilePath const& fileName, FileID fileID,
			bool isMainSourceFile);
		
	private:
		
		ImplementedClass* doClass(ClassMemberAccess access, Class* parent);
		void doClassPart(ImplementedClass* c, ClassMemberAccess access);
		
		ImplementedEnum* doEnum(ClassMemberAccess access, Class* parent);
		
		Field* doField(Class* c, ClassMemberAccess access);
		
		ImplementedConstructor* doConstructor(Class* c,
			ClassMemberAccess access);
		
		ImplementedProcedure* doProcedure(Class* c, ClassMemberAccess access);
		
		void doParameterList(ClassContext const* cCtxt,
			std::vector<DeclaredParameter*>& params);
	};
}
#endif
