/*!
	\file probability.cxx
	\brief Implements the Probability class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-10-31
// Last time modified:	2008-06-26

#include "probability.h"

#include "base/asserts.h"

namespace HYPER_Compiler {
	
	SimpleProbability::SimpleProbability(SP v)
	 : fValue(v)
	{
		// no action
	}
	
	const SimpleProbability SimpleProbability::never = kNever;
	const SimpleProbability SimpleProbability::sometimes = kSometimes;
	const SimpleProbability SimpleProbability::always = kAlways;
	
	HYPER_Common::UTF8String SimpleProbability::toString() const {
		switch (fValue) {
		case kNever:
			return UTF8STR("never");
		case kSometimes:
			return UTF8STR("sometimes");
		case kAlways:
			return UTF8STR("always");
		default:
			hyp_assert_unreachable();
		}
	}
	
	SimpleProbability
	SimpleProbability::operator&(SimpleProbability sp2) const
	{
		SimpleProbability tmp = *this;
		return tmp &= sp2;
	}
	
	SimpleProbability&
	SimpleProbability::operator&=(SimpleProbability sp2) {
		if (fValue != sp2.fValue) {
			fValue = kSometimes;
		}
		
		return *this;
	}
	
	SimpleProbability
	SimpleProbability::operator|(SimpleProbability sp2) const
	{
		SimpleProbability tmp = *this;
		return tmp |= sp2;
	}
	
	SimpleProbability&
	SimpleProbability::operator|=(SimpleProbability sp2) {
		if (sp2.fValue > fValue) {
			fValue = sp2.fValue;
		}
		
		return *this;
	}
}
