/*!
	\file probability.h
	\brief Contains the Probability class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-10-31
// Last time modified:	2008-07-09

#ifndef DEF_PROBABILITY_H
 #define DEF_PROBABILITY_H
 
 #include "common/utf8string.h"
 
 #include "tribool.h"

namespace HYPER_Compiler {
	
	/// Contains probability functionality
	class SimpleProbability {
	public:
		static const SimpleProbability never;
		static const SimpleProbability sometimes;
		static const SimpleProbability always;
		
		static SimpleProbability fromTriBool(TriBool t) {
			if (t.isTrue()) {
				return always;
			}
			else if (t.isFalse()) {
				return never;
			}
			else {
				return sometimes;
			}
		}
		
		bool isNever() const {
			return fValue == never.fValue;
		}
		
		bool isAlways() const {
			return fValue == always.fValue;
		}
		
		bool isSometimes() const {
			return fValue == sometimes.fValue;
		}
		
		// Two probabilities happen together
		SimpleProbability operator&(SimpleProbability sp2) const;
		SimpleProbability& operator&=(SimpleProbability sp2);
		
		// Two consecutive probabilities.
		SimpleProbability operator|(SimpleProbability sp2) const;
		SimpleProbability& operator|=(SimpleProbability sp2);
		
		HYPER_Common::UTF8String toString() const;
		
	private:
		enum SP {
			kNever,
			kSometimes,
			kAlways
		};
		
		SimpleProbability(SP v);
		
		SP fValue;
	};
	
}
#endif
