/*!
	\file procedure.cxx
	\brief Implementation of the Procedure class.
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-09
// Last time modified:	2008-10-16

#include "procedure.h"

#include "parameter.h"
#include "type.h"

namespace HYPER_Compiler {
	
	Procedure::Procedure(CompileInfo& info)
	 : fCompileInfo(info)
	{
		// nothing here
	}
	
	Procedure::~Procedure() {
		// nothing here
	}
	
	CompileInfo& Procedure::getCompileInfo() const {
		return fCompileInfo;
	}
	
	HYPER_Common::UTF8String Procedure::getDescription() const {
		return UTF8STR("procedure %1").compose(quote(headerToString()));
	}
	
	Class const* Procedure::getOwnerClass() const {
		return getClass();
	}
	
	ClassMemberAccess Procedure::getAccessInOwner() const {
		return getAccess();
	}
	
	HYPER_Common::UTF8String Procedure::headerToString() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		AbstractSealedStatic ass = getASS();
		if (ass != kNoASS) {
			sb += toString(ass);
			sb += UTF8STR(" ");
		}
		
		if (isConst()) {
			sb += UTF8STR("const ");
		}
		
		sb += UTF8STR("procedure ");
		sb += getCaption();
		sb += UTF8STR("(");
		
		const std::vector<Parameter const*>& params = *getParameterList();
		for (unsigned int i = 0; i < params.size(); ++i) {
			if (i > 0) {
				sb += UTF8STR(", ");
			}
			
			sb += params[i]->toString();
		}
		
		sb += UTF8STR(")");
		
		if (hasReturnType()) {
			sb += UTF8STR(" : ");
			sb += getReturnType()->toString();
		}
		
		return sb.toString();
	}
	
	HYPER_Common::UTF8String Procedure::getOverloadHash() const {
		HYPER_Common::UTF8StringBuilder sb;
		
		if (isConst()) { // const procedure ?
			sb += UTF8STR("C"); // mark as const
		}
		
		sb += UTF8STR("("); // identify as procedure
		
		const std::vector<Parameter const*>& params = *getParameterList();
		for (unsigned int i = 0; i < params.size(); ++i) {
			sb += params[i]->getType()->getOverloadHash();
			sb += UTF8STR("$"); // separator & terminator character
		}
		
		return sb.toString();
	}
}
