/*!
	\file procedureoverloadresolution.h
	\brief Contains the ProcedureOverloadResolution class
	\author Kevin André
	
	Copyright (C) 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2009-08-23
// Last time modified:	2009-08-23

#ifndef DEF_PROCEDUREOVERLOADRESOLUTION_H
 #define DEF_PROCEDUREOVERLOADRESOLUTION_H
 
 #include "overloadedprocedure.h"
 #include "overloadresolution.h"

namespace HYPER_Compiler {
	
	class InstanceProvider;
	
	/// Overload resolution specifically for procedure calls
	class ProcedureOverloadResolution {
	public:
		
		typedef OverloadResolution<OverloadedProcedure>::Candidate Candidate;
		typedef OverloadResolution<OverloadedProcedure>::Result Result;
		
		// TODO: use SequenceView<Expression> etc. for these functions
		
		static Result
		resolveCall(InstanceProvider const* instance,
			OverloadedProcedure const* list,
			const std::vector<const Expression*>& arguments, CompileInfo& info);
		
		static Result
		resolveCall(InstanceProvider const* instance,
			OverloadedProcedure const* list,
			const std::vector<const Type*>& arguments, CompileInfo& info);
		
		
		static HYPER_Common::UTF8String
		getCallPrototype(QualifiedName entityName,
			const std::vector<const Expression*>& args);
		
		static bool checkAccess(Candidate const* c, ClassContext const& from,
			Location const* loc, CompileInfo& info);
		
	};
	
}

#endif
