/*!
	\file qualifiedname.h
	\brief Contains the QualifiedName class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-06-19
// Last time modified:	2008-02-20

#ifndef DEF_QUALIFIEDNAME_H
 #define DEF_QUALIFIEDNAME_H
 
 #include "common/utf8string.h"
 
 #include <vector>

namespace HYPER_Compiler {
	
	/**
		\brief Represents a qualified name.
		
		It can contain names like "foo", "foo.bar", "foo.bar.Baz", etc.
		QualifiedName's can be compared with and assigned to each other with the
		normal comparison operators.
	*/
	class QualifiedName {
	public:
		QualifiedName(HYPER_Common::UTF8String identifier);
		QualifiedName(const QualifiedName & parent,
			HYPER_Common::UTF8String ident);
		QualifiedName(const QualifiedName &);
		
		~QualifiedName();
		
		void addIdentifier(HYPER_Common::UTF8String identifier);
		
		HYPER_Common::UTF8String getShortName() const;
		HYPER_Common::UTF8String getRootName() const;
		HYPER_Common::UTF8String toString() const;
		QualifiedName getParent() const;
		bool hasParent() const;
		
		unsigned int countNames() const;
		HYPER_Common::UTF8String operator[](unsigned i) const;
		
		bool operator==(const QualifiedName & q) const;
		bool operator!=(const QualifiedName & q) const;
		bool operator<(const QualifiedName & q) const;
		
		QualifiedName& operator=(const QualifiedName& q);
		
		QualifiedName subset(unsigned int length) const;
		
		QualifiedName* copy() const;
		
	private:
		QualifiedName(std::vector<HYPER_Common::UTF8String> childParents,
			unsigned int childNameLength,
			HYPER_Common::UTF8String childStringRep);
		
		std::vector<HYPER_Common::UTF8String> fParent;
		HYPER_Common::UTF8String fName;
		HYPER_Common::UTF8String fStringRep;
	};
}
#endif
