/*!
	\file resolveable.h
	\brief Contains the Resolveable class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-12-31
// Last time modified:	2008-02-15

#ifndef DEF_RESOLVEABLE_H
 #define DEF_RESOLVEABLE_H
 
 #include "common/cxx_macros.h"
 #include "common/utf8string.h"

namespace HYPER_Compiler {
	
	class CompileInfo;
	class Location;
	
	/// Base class for items that have interface and full resolve methods
	class Resolveable {
	public:
		
		Resolveable();
		virtual ~Resolveable() {/*empty*/}
		
		class Trace;
		
		/// Resolve the item's interface only
		bool ATTR_WARN_UNUSED_RESULT
		resolveInterface(Trace const& trace);
		
		/// Resolve the item's interface and all the rest as well
		bool ATTR_WARN_UNUSED_RESULT
		resolveFull(Trace const& trace);
		
		/// Resolve for top-level, not called by another Resolveable. Does a
		/// full resolve.
		bool ATTR_WARN_UNUSED_RESULT
		topLevelResolve();
		
		bool resolveNotYetStarted() const;
		
		virtual CompileInfo& getCompileInfo() const = 0;
		
	protected:
		virtual HYPER_Common::UTF8String getResolveDescription() const = 0;
		virtual const Location* getResolveLocation() const = 0;
		
		virtual bool doInterfaceResolve(Trace const& trace) = 0;
		virtual bool doFullResolve(Trace const& trace) = 0;
		
	private:
		void circularDependencyError(Trace const& trace);
		
		enum ResolveState {
			kNotStarted, kStarted, kFinished, kFailed
		};
		
		ResolveState fIFaceState;
		ResolveState fFullState;
	};
}
#endif
