/*!
	\file resolver.cxx
	\brief Implementation of the Resolver.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-21
// Last time modified:	2008-06-26

#include "resolver.h"

#include "base/asserts.h"
#include "base/errormanager.h"

#include "compileinfo.h"
#include "sourcefile.h"

namespace HYPER_Compiler {
	
	// *** RESOLVER ***
	
	Resolver::Resolver(SourceFile& sf, CompileInfo& info)
	 : fSourceFile(sf), fInfo(info)
	{
		// nothing here
	}
	
	void Resolver::run() {
		registerBuiltins();
		
		unsigned errorsBeforeResolve =
			fInfo.getErrorManager().count(Diagnostic::kError);
		
		bool resolveOK = fSourceFile.topLevelResolve();
		//debug << "overall resolve success: " << resolveOK << '\n';
		
		unsigned errorsAfterResolve =
			fInfo.getErrorManager().count(Diagnostic::kError);
		
		//debug << "resolve errors: "
		//	<< (errorsAfterResolve - errorsBeforeResolve) << '\n';
		
		hyp_assert(
			(errorsBeforeResolve < errorsAfterResolve || resolveOK)
				&& "toplevel resolve result should be true because there were "
				"no errors"
		);
		
		hyp_assert(
			(errorsBeforeResolve == errorsAfterResolve || !resolveOK)
				&& "toplevel resolve result should be false because there were "
				"errors"
		);
	}
	
	void Resolver::registerBuiltins() {
		// Register 'system' namespace. It must be available even when nothing
		// from the standard library is imported. Just getting it causes it to
		// be created.
		fInfo.getSystemNamespace();
	}
}
