/*!
	\file resolver.h
	\brief Resolver for the HYPER compiler.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-21
// Last time modified:	2007-08-17

#ifndef DEF_RESOLVER_H
 #define DEF_RESOLVER_H
 
 #include "important_fwdecl.h"

/// Contains all code of the compiler
namespace HYPER_Compiler {
	
	/// Does semantic analysis.
	class Resolver {
	public:
		/**
			\brief Constructs a new Resolver object.
			\param sf   The source file
			\param info  Compile info
		*/
		Resolver(SourceFile& sf, CompileInfo& info);
		
		/**
			\brief This performs the actual semantic analysis.
		*/
		void run();
		
	private:
		void registerBuiltins();
		
		SourceFile& fSourceFile;
		CompileInfo& fInfo;
	};
};
#endif
