/*!
	\file simpleloopstatement.h
	\brief Contains the SimpleLoopStatement class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-13
// Last time modified:	2008-03-02

#ifndef DEF_SIMPLELOOPSTATEMENT_H
 #define DEF_SIMPLELOOPSTATEMENT_H
 
 #include "statement.h"

namespace HYPER_Compiler {
	
	class BoolConditionExpression;
	class StatementBlock;
	class StatementContext;
	
	/// Abstract simple-loop-statement representation
	class SimpleLoopStatement : public Statement {
	public:
		SimpleLoopStatement(const Location* pos,
			StatementContext const* context, CompileInfo& info);
		~SimpleLoopStatement();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		void setTest(bool checkBeforeLoop,
			BoolConditionExpression* exitLoopTest, bool mustInvertTest);
		
		void setBlock(StatementBlock* s);
		StatementBlock* getBlock() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		
		void putInSymbolTable(ScopeLocation const& loc);
		
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
	private:
		bool fCheckBeforeLoop;
		bool fInvertTest;
		BoolConditionExpression* fTest;
		StatementBlock* fBlock;
		StatementContext const* fContext;
	};
}
#endif
