/*!
	\file simpleparameter.cxx
	\brief Implements the SimpleParameter class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-09-23
// Last time modified:	2008-12-25

#include "simpleparameter.h"

#include "base/asserts.h"

#include "type.h"

//~ #include "base/debug.h"

namespace HYPER_Compiler {
	
	SimpleParameter::SimpleParameter(Type const* type, Passing passing)
	 : fType(type), fPassing(passing)
	{
		hyp_assert(type != 0);
		
		// in parameter -> type needs to be completely const
		if (passing == kInByValue || passing == kInByReference) {
			fType = fType->getAllConstType();
			//~ debug.line(
				//~ UTF8STR("INPUT TYPE: converted to const: %1").compose(
					//~ fType->toString()
				//~ )
			//~ );
		}
		
		// convert inByReference to inByValue for simple types (like "byte")
		if (fPassing == kInByReference && fType->canBeInvisiblyPassedByValue())
		{
			fPassing = kInByValue;
		}
	}
	
	Type const* SimpleParameter::getType() const {
		return fType;
	}
	
	SimpleParameter::Passing SimpleParameter::getPassing() const {
		return fPassing;
	}
}
