/*!
	\file simpleparameter.h
	\brief Contains the SimpleParameter class.
	\author Kevin André
	
	Copyright (C) 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2008-09-23
// Last time modified:	2008-12-25

#ifndef DEF_SIMPLEPARAMETER_H
 #define DEF_SIMPLEPARAMETER_H
 
 #include "common/utf8string.h"
 
namespace HYPER_Compiler {
	
	class Type;
	
	/// Represents a parameter; a type and a passing
	class SimpleParameter {
	public:
		/// How to pass a value to a parameter
		enum Passing {
			kInByReference,
			kInByValue,
			kInOut,
			kCopyValue ///< copy a value; for return statements etc.
		};
		
		explicit SimpleParameter(Type const* type,
			Passing passing = kInByReference);
		
		Passing getPassing() const;
		Type const* getType() const;
		
		/// Get a string name for a Passing  (primarily for debug output)
		static HYPER_Common::UTF8String nameOf(Passing p) {
			switch (p) {
			case kInByReference:
				return UTF8STR("in_by_reference");
			case kInByValue:
				return UTF8STR("in_by_value");
			case kInOut:
				return UTF8STR("in/out");
			case kCopyValue:
				return UTF8STR("copy_value");
			}
			
			// should be unreachable
			return UTF8STR("invalid_passing");
		}
		
	private:
		Type const* fType;
		Passing fPassing;
	};
}
#endif
