/*!
	\file sourcefile.h
	\brief Contains the SourceFile class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-04-04
// Last time modified:	2009-08-30

#ifndef DEF_SOURCEFILE_H
 #define DEF_SOURCEFILE_H
 
 #include "base/location.h"
 
 #include "common/paths_fwdecl.h"
 
 #include "important_fwdecl.h"
 #include "qualifiedname.h"
 #include "resolveable.h"
 #include "syntaxitem.h"
 
 #include <list>
 #include <vector>

namespace HYPER_Compiler {
	
	class ClassRepository;
	class ImplementedClass;
	class ImplementedEnum;
	class Procedure;
	
	/// Represents a sourcefile
	class SourceFile : public SyntaxItem, public Resolveable {
	public:
		SourceFile(FileID fileID, HYPER_Common::FilePath const& fileName,
			CompileInfo& info, bool isMainSourceFile, Namespace* ns);
		~SourceFile();
		
		FileID getFileID() const;
		HYPER_Common::FilePath const& getFileName() const;
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		void setBegin(QualifiedName const& name, const Location* loc);
		void addImport(QualifiedName const& name, const Location* loc);
		void addMember(ImplementedClass* c);
		void addMember(ImplementedEnum* e);
		
		CompileInfo& getCompileInfo() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
	private:
		class QNLoc {
		public:
			QNLoc(QualifiedName const& qn, Location const* loc)
			 : fgQN(qn), fgLoc(loc)
			{
				// no action
			}
			
			QualifiedName fgQN;
			Location const* fgLoc;
		};
		
		class AnonymousNamespace;
		
		void doImport(QNLoc const& qni);
		Class* findBeginClassSpecified();
		Procedure* findProcedureMain();
		
		FileID fFileID;
		HYPER_Common::FilePath const& fFileName;
		bool fMainSource;
		std::list<QNLoc> fImports;
		QNLoc* fBegin;
		Class* fBeginClass;
		Procedure* fBeginProc;
		CompileInfo& fCompileInfo;
		Scope* fContentScope;
		Namespace* fNS;
		AnonymousNamespace* fAnonNS;
		std::vector<ImplementedClass*> fClasses;
		std::vector<ImplementedEnum*> fEnums;
	};
}
#endif
