/*!
	\file statement.cxx
	\brief Implements the Statement class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-19
// Last time modified:	2008-06-26

#include "statement.h"

//~ #include "base/debug.h"

#include "scopes.h"

namespace HYPER_Compiler {
	
	Statement::Statement(CompileInfo& info, const Location* beginLoc)
	 : fInfo(info), fBeginLoc(beginLoc), fScopeLoc(0)
	{
		// no action
	}
	
	Statement::~Statement() {
		// no action
	}
	
	const Location* Statement::getLocation() const {
		return fBeginLoc;
	}
	
	void Statement::putInSymbolTable(ScopeLocation const& loc) {
		fScopeLoc = &loc;
	}
	
	ScopeLocation const* Statement::getScopeLocation() const {
		return fScopeLoc;
	}
	
	CompileInfo& Statement::getCompileInfo() const {
		return fInfo;
	}
}
