/*!
	\file statement.h
	\brief Contains the Statement class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-07-19
// Last time modified:	2008-03-02

#ifndef DEF_STATEMENT_H
 #define DEF_STATEMENT_H
 
 #include "important_fwdecl.h"
 #include "probability.h"
 #include "resolveable.h"
 #include "syntaxitem.h"

namespace HYPER_Compiler {
	
	/// Base class for all statements
	class Statement : public SyntaxItem, public Resolveable {
	public:
		Statement(CompileInfo& info, const Location* beginLoc);
		virtual ~Statement();
		
		/// Get the location in the source where the statement begins
		const Location* getLocation() const;
		
		/// Let the statement know where it is in the symbol table, i.e. what
		/// its parent scope is.
		virtual void putInSymbolTable(ScopeLocation const& loc);
		
		virtual HYPER_Common::UTF8String getResolveDescription() const = 0;
		
		const Location* getResolveLocation() const { return getLocation(); }
		
		virtual bool doInterfaceResolve(Trace const& trace) { return true; }
		virtual bool doFullResolve(Trace const& trace) = 0;
		
		/// Does the statement return from the procedure/constructor ?
		virtual SimpleProbability getReturnProbability() const = 0;
		
		/// Get the CompileInfo
		CompileInfo& getCompileInfo() const;
		
	protected:
		ScopeLocation const* getScopeLocation() const;
		
	private:
		CompileInfo& fInfo;
		const Location* fBeginLoc;
		ScopeLocation const* fScopeLoc;
	};
}
#endif
