/*!
	\file statementblock.h
	\brief Abstract statement block representation.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-08-14
// Last time modified:	2008-02-20

#ifndef DEF_STATEMENTBLOCK_H
 #define DEF_STATEMENTBLOCK_H
 
 #include "syntaxitem.h"
 #include "probability.h"
 #include "resolveable.h"
 #include "scopes.h"
 
 #include <vector>

namespace HYPER_Compiler {
	class Statement;
	
	class StatementBlock : public SyntaxItem, public OrderedScope,
		public Resolveable
	{
	public:
		StatementBlock(CompileInfo& info, const Location* beginPos);
		~StatementBlock();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		void addStatement(Statement* s);
		bool isEmpty() const;
		unsigned int length() const;
		
		const Location* getBeginLocation() const;
		
		void putInSymbolTable(Scope& parent);
		void putInSymbolTable(ScopeLocation const& loc);
		
		CompileInfo& getCompileInfo() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
	private:
		CompileInfo& fCompileInfo;
		std::vector<Statement*> fStatements;
		const Location* fPos;
		SimpleProbability fReturnProb;
	};
}
#endif
