/*!
	\file statementcontext.cxx
	\brief Implements the StatementContext class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-09-18
// Last time modified:	2008-01-06

#include "statementcontext.h"

namespace HYPER_Compiler {
	
	void StatementContext::setCurrentLoop(SimpleLoopStatement const* s) {
		fILDomination = kLoopDominates;
		fLoopType = kSimpleLoop;
		fSLoop = s;
		fMLoop = 0;
	}
	
	void StatementContext::setCurrentLoop(MegaLoopStatement const* s) {
		fILDomination = kLoopDominates;
		fLoopType = kMegaLoop;
		fSLoop = 0;
		fMLoop = s;
	}
	
	StatementContext::LoopType
	StatementContext::getCurrentLoopType() const {
		return fLoopType;
	}
	
	const SimpleLoopStatement* 
	StatementContext::getCurrentSimpleLoop() const {
		return fSLoop;
	}
	
	const MegaLoopStatement*
	StatementContext::getCurrentMegaLoop() const {
		return fMLoop;
	}
	
	void StatementContext::setCurrentIterate(IterateStatement const* i) {
		fILDomination = kIterateDominates;
		fIterate = i;
	}
	
	const IterateStatement*
	StatementContext::getCurrentIterate() const {
		return fIterate;
	}
	
	
	StatementContext::IterateVersusLoop
	StatementContext::getIterateVersusLoop() const {
		return fILDomination;
	}
	
	
	StatementContext::StatementContext(CompileInfo& info)
	 : ClassContext(info), fLoopType(kNoLoop), fSLoop(0), fMLoop(0),
		fIterate(0), fILDomination(kNeitherDominates)
	{
		// empty
	}
}
