/*!
	\file statementcontext.h
	\brief Contains the StatementContext class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-09-18
// Last time modified:	2008-01-12

#ifndef DEF_STATEMENTCONTEXT_H
 #define DEF_STATEMENTCONTEXT_H
 
 #include "classcontext.h"

namespace HYPER_Compiler {
	
	class SimpleLoopStatement;
	class MegaLoopStatement;
	class IterateStatement;
	class Expression;
	
	/// Contains info about the context in which a statement is used
	class StatementContext : public ClassContext {
	public:
		virtual ~StatementContext() {}
		
		enum LoopType {
			kNoLoop,
			kSimpleLoop,
			kMegaLoop
		};
		
		void setCurrentLoop(SimpleLoopStatement const* s);
		void setCurrentLoop(MegaLoopStatement const* s);
		LoopType getCurrentLoopType() const;
		const SimpleLoopStatement* getCurrentSimpleLoop() const;
		const MegaLoopStatement* getCurrentMegaLoop() const;
		
		void setCurrentIterate(IterateStatement const* i);
		const IterateStatement* getCurrentIterate() const;
		
		/// Describes what was set most recently
		enum IterateVersusLoop {
			kNeitherDominates,
			kLoopDominates,
			kIterateDominates
		};
		
		IterateVersusLoop getIterateVersusLoop() const;
		
		virtual StatementContext* copy() const = 0;
		
		virtual bool checkReturn(Location const* loc,
			Expression const* e) const = 0;
		
	protected:
		StatementContext(CompileInfo& info);
	
	private:
		LoopType fLoopType;
		const SimpleLoopStatement* fSLoop;
		const MegaLoopStatement* fMLoop;
		const IterateStatement* fIterate;
		IterateVersusLoop fILDomination;
	};
}
#endif
