/*!
	\file statementparser.h
	\brief Contains the StatementParser class.
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-09-22
// Last time modified:	2009-02-28

#ifndef DEF_STATEMENTPARSER_H
 #define DEF_STATEMENTPARSER_H
 
 #include "parserbase.h"

namespace HYPER_Compiler {
	
	class IterateStatement;
	class StatementBlock;
	class StatementContext;
	
	/// Parser for statements
	class StatementParser : private ParserBase {
	public:
		StatementParser(ParserInfo const& pi);
		
		void doBlockBody(StatementBlock& b, StatementContext const* ctxt);
		
		Statement* doStatement(StatementContext const* ctxt);
		
	private:
		Statement* doVar(StatementContext const* ctxt);
		Statement* doReturn(StatementContext const* ctxt);
		Statement* doIf(StatementContext const* ctxt);
		Statement* doLoop(StatementContext const* ctxt);
		Statement* doIterate(StatementContext const* ctxt);
		Statement* doJump(StatementContext const* ctxt);
		Statement* doMiscStatement(StatementContext const* ctxt);
	};
}
#endif
