/*!
	\file stringclass.h
	\author Kevin André
	\brief Contains the String class
	
	Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-10-08
// Last time modified:	2008-02-28

#ifndef DEF_STRINGCLASS_H
 #define DEF_STRINGCLASS_H
 
 #include "builtinclass.h"

namespace HYPER_Compiler {
	
	class BinaryOperator;
	
	/// Represents the built-in `string' class
	class String : public BuiltInClass {
	public:
		String(CompileInfo& info);
		~String();
		
		HYPER_Common::UTF8String getTypeName() const;
		
		void buildSymbolTable();
		
		HYPER_Common::UTF8String getDeclarationName() const;
		
	private:
		class StringConstructor;
		
		class ProcedureClear;
		class ProcedureIsEmpty;
		class ProcedureLeft;
		class ProcedureRight;
		class ProcedureSize;
		class ProcedureSubString;
		
		class DiffersOperator;
		class EqualsOperator;
		class GreaterEqOperator;
		class GreaterOperator;
		class LessEqOperator;
		class LessOperator;
		
		class AddOperator;
		
		class IndexOperator;
		
		class StringAssignOp;
		class CharAddAssignOp;
		class StringAddAssignOp;
		
		void createConstructors();
		void createMembers();
		void createOperators();
	};
}
#endif
