/*!
	\file testcase.cxx
	\brief Implementation of the TestCase class.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-13
// Last time modified:	2008-02-15

#include "testcase.h"

#include "boolconditionexpression.h"
#include "expression.h"
#include "statementblock.h"
#include "statementcontext.h"

//#include "debug.h"

namespace HYPER_Compiler {
	
	TestCase::TestCase(BoolConditionExpression* test, StatementBlock* block,
		StatementContext const* context)
	 : fTest(test), fBlock(block), fContext(context)
	{
		// no action
	}
	
	Expression* TestCase::getTest() const {
		return fTest;
	}
	
	StatementBlock* TestCase::getBlock() const {
		return fBlock;
	}
	
	void TestCase::putInSymbolTable(ScopeLocation const& loc) {
		fTest->putInSymbolTable(new LookupLocation(loc));
		fBlock->putInSymbolTable(loc);
	}
	
	CompileInfo& TestCase::getCompileInfo() const {
		return fTest->getCompileInfo();
	}
	
	HYPER_Common::UTF8String TestCase::getResolveDescription() const {
		// TODO: make this invisible in traces?
		return UTF8STR("test case");
	}
	
	const Location* TestCase::getResolveLocation() const {
		return fTest->getLocation(); // FIXME: not completely accurate
	}
	
	bool TestCase::doInterfaceResolve(Trace const& trace) {
		// no action needed
		return true;
	}
	
	bool TestCase::doFullResolve(Trace const& trace) {
		bool ok = fTest->resolveFull(trace);
		ok &= fBlock->resolveFull(trace);
		return ok;
	}
}
