/*!
	\file testcase.h
	\brief Abstract test-case representation.
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-09-13
// Last time modified:	2008-01-11

#ifndef DEF_TESTCASE_H
 #define DEF_TESTCASE_H
 
 #include "resolveable.h"

namespace HYPER_Compiler {
	
	class BoolConditionExpression;
	class CompileInfo;
	class Expression;
	class ScopeLocation;
	class StatementBlock;
	class StatementContext;
	
	/// Represents a boolean condition expression together with a block of
	/// statements to be executed when the condition is true.
	class TestCase : public Resolveable {
	public:
		TestCase(BoolConditionExpression* test, StatementBlock* block,
			StatementContext const* context);
		
		Expression* getTest() const;
		StatementBlock* getBlock() const;
		
		void putInSymbolTable(ScopeLocation const& loc);
		
		CompileInfo& getCompileInfo() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		const Location* getResolveLocation() const;
		
		bool doInterfaceResolve(Trace const& trace);
		bool doFullResolve(Trace const& trace);
		
	private:
		Expression* fTest;
		StatementBlock* fBlock;
		StatementContext const* fContext;
	};
};
#endif
