/*!
	\file textdiagnosticprinter.h
	\brief Contains the TextDiagnosticPrinter class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created: 		2007-08-28
// Last time modified: 	2008-06-01

#ifndef DEF_TEXTDIAGNOSTICPRINTER_H
 #define DEF_TEXTDIAGNOSTICPRINTER_H
 
 #include "common/paths_fwdecl.h"
 #include "common/streams_fwdecl.h"

namespace HYPER_Compiler {
	
	class Diagnostic;
	class ErrorManager;
	class Location;
	
	/// Provides text output for diagnostics
	class TextDiagnosticPrinter {
	public:
		TextDiagnosticPrinter(ErrorManager const& errMan);
		
		void printList(HYPER_Common::UTF8TextFlowStream& out) const;
		void printSummary(HYPER_Common::UTF8TextFlowStream& out) const;
		
		void print(HYPER_Common::UTF8TextFlowStream& out,
			bool printListing) const
		{
			if (printListing) {
				printList(out);
			} 
			printSummary(out);
		}
		
	private:
		HYPER_Common::FilePath const* getFile(Location const& loc) const;
		
		void printReducedLocation(Location const& loc,
			HYPER_Common::FilePath const* prefix,
			Location const* omitLocation,
			HYPER_Common::UTF8TextFlowStream& out) const;
		
		ErrorManager const& fEM;
	};
}
#endif
