/*!
	\file unaryoperator.cxx
	\brief Implements the UnaryOperator class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-25
// Last time modified:	2008-10-15

#include "unaryoperator.h"

#include "type.h"

namespace HYPER_Compiler {
	
	HYPER_Common::UTF8String
	UnaryOperator::getPrintPrototype(Operators::UnaryOperator op,
		Type const* operandT)
	{
		return UTF8STR("operator%1(%2)").compose(
			Operators::toString(op), operandT->toString()
		);
	}
	
	UnaryOperator::UnaryOperator(Operators::UnaryOperator op,
		Type const* operand, Type const* result, Location const* loc)
	 : fOpSym(op), fOperand(operand), fResult(result), fLoc(loc)
	{
		// no action
	}
	
	UnaryOperator::~UnaryOperator() {
		// no action
	}
	
	Operators::UnaryOperator UnaryOperator::getOperator() const {
		return fOpSym;
	}
	
	Type const* UnaryOperator::getOperandType() const {
		return fOperand;
	}
	
	Type const* UnaryOperator::getResultType() const {
		return fResult;
	}
	
	Location const* UnaryOperator::getLocation() const {
		return fLoc;
	}
}
