/*!
	\file unaryoperator.h
	\brief Contains the UnaryOperator class
	\author Kevin André
	
	Copyright (C) 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2006-08-24
// Last time modified:	2008-10-15

#ifndef DEF_UNARYOPERATOR_H
 #define DEF_UNARYOPERATOR_H
 
 #include "important_fwdecl.h"
 #include "operators.h"
 
namespace HYPER_Compiler {
	
	/// Represents a unary operator
	class UnaryOperator {
	public:
		static HYPER_Common::UTF8String getPrintPrototype(
			Operators::UnaryOperator op, Type const* operandT);
		
		UnaryOperator(Operators::UnaryOperator op,
			Type const* operand, Type const* result, Location const* loc);
		
		virtual ~UnaryOperator();
		
		Operators::UnaryOperator getOperator() const;
		
		Type const* getOperandType() const;
		Type const* getResultType() const;
		
		Location const* getLocation() const;
		
		virtual ConstantValue const*
			evaluateCompileTime(ConstantValue const* operand) = 0;
		
		// TODO : change return type
		virtual void generateCode() = 0;
		
	private:
		Operators::UnaryOperator fOpSym;
		Type const* fOperand;
		Type const* fResult;
		Location const* fLoc;
	};
}
#endif
