/*!
	\file varstatement.h
	\brief Contains the Var class
	\author Kevin André
	
	Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2004-06-30
// Last time modified:	2008-10-13

#ifndef DEF_VARSTATEMENT_H
 #define DEF_VARSTATEMENT_H
 
 #include "nameddeclareditem.h"
 #include "statement.h"

 #include <vector>

/// Contains all code of the compiler
namespace HYPER_Compiler {
	
	class DeclaredType;
	class Initializer;
	class VarDecl;
	
	/// Represents a 'var' statement
	class Var : public Statement {
	public:
		Var(
			bool isStatic, std::vector<HYPER_Common::UTF8String> const& names,
			std::vector<const Location*> locs,
			DeclaredType* t, Initializer* init, const Location* loc,
			StatementContext const* context, CompileInfo& info
		);
		
		~Var();
		
		HYPER_Common::UTF8String getSyntaxName() const;
		void myPrint(HYPER_Common::UTF8TextFlowStream& ind) const;
		
		DeclaredType const* getType() const;
		Type const* getResolvedType() const;
		
		//bool isStatic() const;  // NOT YET USED
		
		const Location* getLocation() const;
		
		HYPER_Common::UTF8String getResolveDescription() const;
		
		void putInSymbolTable(ScopeLocation const& loc);
		
		bool doFullResolve(Trace const& trace);
		
		SimpleProbability getReturnProbability() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
	private:
		std::vector<VarDecl*> fDecls;
		DeclaredType* fType;
		Type const* fResolvedType;
		Initializer* fInit;
		// bool fStatic;  // NOT YET USED
		const Location* fLoc;
		StatementContext const* fContext;
	};
	
	/// Represents a single variable
	class VarDecl : public NamedDeclaredItem {
	public:
		Var const* getVar() const;
		
		HYPER_Common::UTF8String getDescription() const;
		HYPER_Common::UTF8String getDeclarationName() const;
		
		Class const* getOwnerClass() const;
		ClassMemberAccess getAccessInOwner() const;
		
		const Location* getLocation() const;
		
	private:
		VarDecl(Var* v, HYPER_Common::UTF8String name, Location const* loc);
		friend class Var;
		
		Var* fV;
		HYPER_Common::UTF8String fName;
		Location const* fLoc;
	};
}
#endif
