/**
	\file version.h
	\brief Contains the Version class
	\author Kevin André
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-12-25
// Last time modified:	2009-05-31

/*
	Hyper Compiler Version Milestones :
		
		version		date			milestone
		-----------		-------			---------------
		
		0.2.0		2004-09-14		Does parsing and builds AST
		0.3.0		2005-06-10		New semantic framework in use
		0.4.0		2008-05-08		Semantic processes reworked & UTF-8 support
		
*/

#ifndef DEF_VERSION_H
 #define DEF_VERSION_H
 

// *******************  BEGIN OF VERSION SETTINGS  ******************* //

// IMPORTANT:
// please modify these settings if you maintain or distribute a modified version
// of the Hyper sourcecode. These settings are used by the executables for their
// "--version" output.

// current compiler version number; grepped by the package_source script.
// DO NOT CHANGE FORMATTING!
#define COMPILERVERSION "0.4.4"

// name of compiler branch or vendor (empty string for main distribution)
#define COMPILERBRANCHORVENDORNAME ""

// an optional appendix (useful as a description)
#define COMPILERVERSIONAPPENDIX "front end only"

// the URL or e-mail address to report bugs to
#define BUGREPORTLOCATION "<http://bugs.launchpad.net/hyper>"

// ********************  END OF VERSION SETTINGS  ******************** //


namespace HYPER_Compiler {
	
	/// Provides version info
	class Version {
		static bool isEmpty(char const* s) {
			return s[0] == '\0';
		}
		
	public:
		/// Is this the main distribution?
		static bool isMainDistribution() {
			return isEmpty(COMPILERBRANCHORVENDORNAME);
		}
		
		/// Get branch or vendor name
		static char const* getBranchOrVendorName() {
			return COMPILERBRANCHORVENDORNAME;
		}
		
		/// Get the compiler version identification
		static char const* get() {
#define COMPILERFULLVERSIONSTR_A \
			COMPILERVERSION " (" COMPILERVERSIONAPPENDIX ")"
#define COMPILERFULLVERSIONSTR_B \
			COMPILERVERSION " <" COMPILERBRANCHORVENDORNAME ">"
#define COMPILERFULLVERSIONSTR_C \
			COMPILERVERSION " (" COMPILERVERSIONAPPENDIX \
			") <" COMPILERBRANCHORVENDORNAME ">"
			
			if (isEmpty(COMPILERBRANCHORVENDORNAME)) {
				if (isEmpty(COMPILERVERSIONAPPENDIX)) {
					return COMPILERVERSION;
				}
				else {
					return COMPILERFULLVERSIONSTR_A;
				}
			}
			else {
				if (isEmpty(COMPILERVERSIONAPPENDIX)) {
					return COMPILERFULLVERSIONSTR_B;
				}
				else {
					return COMPILERFULLVERSIONSTR_C;
				}
			}
		}
		
		static char const* getBugReportLocation() {
			return BUGREPORTLOCATION;
		}
	};
}
#endif
