/**
	\file htmlcommentdecoder.h
	\author Kevin André
	\brief Contains the HtmlCommentDecoder class.
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-03-08
// Last time modified:	2008-05-04

#ifndef DEF_HTMLCOMMENTDECODER_H
 #define DEF_HTMLCOMMENTDECODER_H
 
 #include "common/streamprocessor.h"
 #include "common/utf8.h"

namespace HYPER_Highlighter {
	
	/// Reads data inside a HTML comment
	class HtmlCommentDecoder
		: public HYPER_Common::StreamProcessor<HYPER_Common::UTF8Byte,
			HYPER_Common::UTF8Byte>
	{
	public:
		
		HtmlCommentDecoder() {
			// dash1 : "&#045;"
			fDash1[0] = HYPER_Common::UTF8Byte('&');
			fDash1[1] = HYPER_Common::UTF8Byte('#');
			fDash1[2] = HYPER_Common::UTF8Byte('0');
			fDash1[3] = HYPER_Common::UTF8Byte('4');
			fDash1[4] = HYPER_Common::UTF8Byte('5');
			fDash1[5] = HYPER_Common::UTF8Byte(';');
			
			// dash2 : "&#45;"
			fDash2[0] = HYPER_Common::UTF8Byte('&');
			fDash2[1] = HYPER_Common::UTF8Byte('#');
			fDash2[2] = HYPER_Common::UTF8Byte('4');
			fDash2[3] = HYPER_Common::UTF8Byte('5');
			fDash2[4] = HYPER_Common::UTF8Byte(';');
		}
		
		ProcessStatus processData(std::deque<HYPER_Common::UTF8Byte>& input,
			bool endOfInputReached, std::deque<HYPER_Common::UTF8Byte>& output,
			State& processState) const;
		
	private:
		HYPER_Common::UTF8Byte fDash1[6];
		static const unsigned kDash1Length = 6;
		HYPER_Common::UTF8Byte fDash2[5];
		static const unsigned kDash2Length = 5;
	};
}
#endif
