/**
	\file htmlcommentencoder.cxx
	\author Kevin André
	\brief Implements the HtmlCommentEncoder class.
	
	Copyright (C) 2007, 2008, 2009  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2007-03-08
// Last time modified:	2008-07-12

#include "htmlcommentencoder.h"

//~ #include <iostream> // for debugging

using HYPER_Common::CodePoint;
using HYPER_Common::UTF8Byte;

namespace HYPER_Highlighter {
	
	HtmlCommentEncoder::ProcessStatus
	HtmlCommentEncoder::processData(std::deque<HYPER_Common::UTF8Byte>& input,
		bool endOfInputReached, std::deque<HYPER_Common::UTF8Byte>& output,
		State& processState) const
	{
		static UTF8Byte dash = UTF8Byte('-');
		//~ std::cout << "HtmlCommentEncoder::processData called" << std::endl;
		
		UTF8Byte first = input[0];
		if (first == dash) {
			//~ std::cout << " first is a dash char" << std::endl;
			
			if (input.size() < 2) {
				// request the next byte before we can do anything
				//~ std::cout << " need extra input after dash" << std::endl;
				processState.fgInputNeeded = 2;
				return kNeedMoreData;
			}
			
			if (input[1] == first) {
				// double dash found, encode the characters
				//~ std::cout << " got double dash" << std::endl;
				input.pop_front();
				input.pop_front();
				output.push_front(UTF8Byte('&'));
				output.push_front(UTF8Byte('#'));
				output.push_front(UTF8Byte('0'));
				output.push_front(UTF8Byte('4'));
				output.push_front(UTF8Byte('5'));
				output.push_front(UTF8Byte(';'));
				output.push_front(UTF8Byte('&'));
				output.push_front(UTF8Byte('#'));
				output.push_front(UTF8Byte('0'));
				output.push_front(UTF8Byte('4'));
				output.push_front(UTF8Byte('5'));
				output.push_front(UTF8Byte(';'));
				processState.fgInputNeeded = 1;
				return kOK;
			}
		}
		
		// if we get here, no match was found.
		//~ std::cout << " non-dash-important char processed: " << char(first.toByte()) << std::endl;
		output.push_front(first);
		input.pop_front();
		processState.fgInputNeeded = 1;
		return kOK;
	}
}
