/*!
	\file keywords.cxx
	\brief Implements the Keywords class.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-08-29
// Last time modified:	2008-07-20

#include "keywords.h"

#include "base/asserts.h"

namespace HYPER_Compiler {
	
	Keywords::Keywords() {
		createKeywordsList();
	}
	
	void Keywords::createKeywordsList() {
		/*
			Generate a list of statements to add each keyword to our lists.
			Example statement:
			
			addKeyword(UTF8STR("abstract"), kAbstract);
		*/
		#define HYPER_KEYWORD(ENUMVALUE,KEYWSTRING) \
			addKeyword(UTF8STR(KEYWSTRING), ENUMVALUE);
		
		#include "keywords.def"
		#undef HYPER_KEYWORD
	}
	
	void Keywords::addKeyword(HYPER_Common::UTF8String str, Keyword enumCst)
	{
		hyp_assert(fEnumToStringMap.find(enumCst) == fEnumToStringMap.end());
		fEnumToStringMap[enumCst] = str;
		
		hyp_assert(fStringToEnumMap.find(str) == fStringToEnumMap.end());
		fStringToEnumMap[str] = enumCst;
	}
	
	Keywords::Keyword
	Keywords::checkKeyword(HYPER_Common::UTF8String s) const
	{
		std::map<HYPER_Common::UTF8String, Keyword>::const_iterator it =
			fStringToEnumMap.find(s);
		
		if (it == fStringToEnumMap.end()) {
			return kNoKeyword;
		}
		else {
			return it->second;
		}
	}
	
	HYPER_Common::UTF8String Keywords::getString(Keyword k) const {
		hyp_assert(k != kNoKeyword);
		return fEnumToStringMap.find(k)->second;
	}
	
	Keywords const& Keywords::list() {
		static Keywords list;
		return list;
	}
}
