/*!
	\file keywords.h
	\brief Contains the Keywords class.
	\author Kevin André
	
	Copyright (C) 2005, 2006, 2007, 2008  Kevin André
	
	This file is part of the Hyper Compiler, a compiler for the Hyper
	programming language.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; see the file GPLv3_license.txt.  If not, see
	<http://www.gnu.org/licenses/>.
	
*/
// Date created:		2005-08-29
// Last time modified:	2008-07-20

#ifndef DEF_KEYWORDS_H
 #define DEF_KEYWORDS_H
 
 #include "common/utf8string.h"
 
 #include <map>

namespace HYPER_Compiler {
	
	/// Handles reserved keywords
	class Keywords {
	public:
		enum Keyword {
			/*
				Generate enum members from the list of reserved keywords.
			*/
			#define HYPER_KEYWORD(ENUMVALUE,KEYWSTRING) ENUMVALUE,
			#include "keywords.def"
			#undef HYPER_KEYWORD
			kNoKeyword	// placeholder used to indicate absence of valid keyword
		};
		
		Keywords();
		
		Keyword checkKeyword(HYPER_Common::UTF8String s) const;
		HYPER_Common::UTF8String getString(Keyword k) const;
		
		static Keywords const& list();
		
	private:
		void createKeywordsList();
		void addKeyword(HYPER_Common::UTF8String str, Keyword enumCst);
		
		std::map<Keyword, HYPER_Common::UTF8String> fEnumToStringMap;
		std::map<HYPER_Common::UTF8String, Keyword> fStringToEnumMap;
	};
}
#endif
