#! /usr/bin/env bash
#
# Create a diff file that lists the difference between the front end and the
# back end. New files in the back end are not listed in the diff.
#


# define back end path (relative to script directory)
BACKENDDIR="../hyper-llvm-new"


EXECDIR="$PWD"
SCRIPTDIR=$(dirname "$0")


# Diff file name
if [ "x$1" == x ] ; then
	DIFFFILE="front_back_end.diff"
else
	DIFFFILE="$1"
fi

# Check diff file existance  (no accidental overwrites)
if [ -e "$DIFFFILE" ] ; then
	echo "ERROR: diff file already exists.  (name = $DIFFFILE )"
	echo "       -> delete it first or specify another name."
	exit 1
fi



check_Hyper_source() {
	[ -d "$1"/common ] && [ -d "$1"/base ] && [ -d "$1"/lexer ] \
		&& [ -d "$1"/compiler ]
}

check_Hyper_backendsrc() {
	[ -f "$1"/compiler/codegenerator.cxx ]
}



# Check if we are in a Hyper source dir
if check_Hyper_source "$SCRIPTDIR" ; then
	OK=1
else
	echo "ERROR: this script is not in a Hyper source dir"
	exit 1
fi
if check_Hyper_backendsrc "$SCRIPTDIR" ; then
	echo "ERROR: this script is in a Hyper-with-back-end source dir"
	exit 1
fi


# Check the back end dir
if [ ! -d "$SCRIPTDIR"/"$BACKENDDIR" ] ; then
	echo "ERROR: back end variable not set correctly"
	exit 1
fi
if check_Hyper_source "$SCRIPTDIR"/"$BACKENDDIR" ; then
	OK=1
else
	echo "ERROR: back end variable not set to a Hyper source dir"
	exit 1
fi
if check_Hyper_backendsrc "$SCRIPTDIR"/"$BACKENDDIR" ; then
	OK=1
else
	echo "ERROR: back end variable not set to a Hyper-with-back-end source dir"
	exit 1
fi

# Go into source dir
cd "$SCRIPTDIR"



make_root_diffs() {
	find . -maxdepth 1 -type f \
		'(' '!' -name '.*' ')' \
		'(' -name '*.h' -o -name '*.cxx' -o -name '*.txt' -o -name '*.hyp' ')' \
		-exec diff -u '{}' "$BACKENDDIR"/'{}'  ';' >>"$EXECDIR"/"$DIFFFILE"
}

make_diffs() {
	find "$1" -type f \
		'(' '!' -name '.*' ')' \
		'(' -name '*.h' -o -name '*.cxx' -o -name '*.txt' -o -name '*.hyp' ')' \
		-exec diff -u '{}' "$BACKENDDIR"/'{}'  ';' >>"$EXECDIR"/"$DIFFFILE"
}



make_root_diffs && \
make_diffs common && \
make_diffs base && \
make_diffs lexer && \
make_diffs compiler && \
make_diffs testing || {
	echo "ERROR while creating diffs."
	exit 2
}

# OK, finished

