#! /usr/bin/env bash
#
# =======================
# Source packaging script
# =======================
#
# Copyright (C) 2005, 2006, 2007, 2008, 2009  Kevin André
#
#	This file is part of the Hyper Compiler, a compiler for the Hyper
#	programming language.
#
#	This program is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License along
#	with this program; see the file GPLv3_license.txt.  If not, see
#	<http://www.gnu.org/licenses/>.
#
#
# USAGE:
#  package_source [description]
#
#    'description' is used in the archive name. If no description specified,
#    then the compiler version is extracted from compiler/version.h and used as
#    the description.

echo "Source Packaging script started"

# go into the directory this script is in
cd `dirname $0`

# get the simple name of this directory (probably 'hyper')
ROOT_DIR=`basename "$PWD"`

echo "top-level directory name is '$ROOT_DIR'"

# go to directly above the directory of this script (so one above the ROOT_DIR)
cd ..

# From now on all files we need and use are below $ROOT_DIR

# determing description string
if [ -z "$1" ] ; then
	# use compiler version, extract it from the sources
	DESCR=$(grep '#define COMPILERVERSION ' $ROOT_DIR/compiler/version.h \
		| cut -f3 -d' ' | sed 's/\"//g')
else
	# use cmdline parameter $1
	DESCR="$1"
fi

# target name without .tar.gz suffix and without directory
TARGET="hypercompiler-$DESCR-src"

# tar name
TAR="$TARGET".tar
GZ="$TARGET".tar.gz
TGZ="$TARGET".tgz

# delete existing archives
if [ -e "$ROOT_DIR/$TAR" ] ; then
	echo "deleting existing archive: $TAR"
	rm -rf "$ROOT_DIR/$TAR"
fi
if [ -e "$ROOT_DIR/$GZ" ] ; then
	echo "deleting existing archive: $GZ"
	rm -rf "$ROOT_DIR/$GZ"
fi
if [ -e "$ROOT_DIR/$TGZ" ] ; then
	echo "deleting existing archive: $TGZ"
	rm -rf "$ROOT_DIR/$TGZ"
fi

echo "collecting files into archive..."

# Put all necessary files into the tar.
# We exclude object files, backup files, '.bzr' (Bazaar branch) directories,
# directories named 'build', directories named 'doxygen' (files generated
# by Doxygen) and the tar file itself.
tar -cpf "$ROOT_DIR/$TAR" "$ROOT_DIR"/README "$ROOT_DIR"/AUTHORS
find "$ROOT_DIR" \
	-type f \
	-a \( \! -name '*.o' \) \
	-a \( \! -name '*~' \) \
	-a \( \! -wholename '*/.bzr/*' \) \
	-a \( \! -wholename '*/build/*' \) \
	-a \( \! -wholename '*/doxygen/*' \) \
	-a \( \! -wholename '*/'"$TAR" \) \
	-exec tar -uf "$ROOT_DIR/$TAR" '{}' \;

# go back into the ROOT_DIR
cd "$ROOT_DIR"

echo "compressing archive..."

# gzip the tar file
gzip --best ./"$TAR"

# check the name of the generated archive
if [ -e ./"$GZ" ] ; then
	ARCH="$GZ"
elif [ -e ./"$TGZ" ] ; then
	ARCH="$TGZ"
fi

echo "source package: $ARCH"
echo "done."
